SystemOrganization addCategory: #'SWE-IssueTracker'!
SystemOrganization addCategory: #'SWE-IssueTracker-Components'!
SystemOrganization addCategory: #'SWE-IssueTracker-Tests'!

!BlockContext methodsFor: '*SWE-IssueTracker' stamp: 'mig 1/26/2008 22:55'!
on: exception1 do: handlerAction1 on: exception2 do: handlerAction2
	"Convenience for catching two exceptions."
	
	[[^self value]
		on: exception1
		do: handlerAction1]
		on: exception2
		do: handlerAction2.
! !

GlorpSession subclass: #ITGlorpSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

ITGlorpSession class
	instanceVariableNames: 'instance'!

!ITGlorpSession commentStamp: 'mt 12/15/2007 10:12' prior: 0!
I'm a usual GlorpSession and control a cache manager, a description system and all unit of works. Every manipulation of the SQL database goes through my "fingers".

Use #inUnitOfWorkDo: for watching over "dirty" objects and change all relevant data in the database. But DO NOT forget to register them with #register inside a unit of work.

Any read of data doesn't have to be embedded in a unit of work. Just use the result of #read:, #readOneOf:, #readManyOf:, #read:where: a. s. o.

If you want to delete an object you doesn't have to use a unit of work either.!

ITGlorpSession class
	instanceVariableNames: 'instance'!

!ITGlorpSession class methodsFor: 'accessing' stamp: 'mfp 1/18/2008 17:42'!
getInstance
	"Using singleton pattern to ensure that there is only one instance of ITGlorpSession"
	
	instance ifNil:
		[ instance := ITGlorpDB connect.].
	instance isLoggedIn ifFalse:
		[ instance := ITGlorpDB connect.].
	^ instance! !

!ITGlorpSession class methodsFor: 'instance creation' stamp: 'mt 12/26/2007 10:52'!
new
	Error signal: 'Use getInstance because', self name, ' is a singleton.'! !

!TimeStamp methodsFor: '*SWE-IssueTracker' stamp: 'mig 1/12/2008 15:31'!
printRelativeTimeStamp
	^(self date printRelativeDate, ', ', self time print24)! !

DescriptorSystem subclass: #ITGlorpDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITGlorpDescriptorSystem commentStamp: 'mt 12/30/2007 14:38' prior: 0!
I'm the descriptor system that should be used in every new ITGlorpSession. I'm responsible for all mappings between the IT-Classes and the relevant tables on the SQL server.

You should modify #allTableNames and #constructAllClasses whenever you want to add new objects of the domain space for usage here.

#tableNamed<TABLE_NAME>    ... descriptions about the table structure on the server
#descriptorFor<CLASS_NAME> ... defines mappings between classes and tables
#classModelFor<CLASS_NAME> ... defines types (classes) for symbols (attributes) used by mappings

The relational schemes can be found in the comments for any #tableNamed<TABLE_NAME>. Primary keys are underlined, e.g. 

SCHEME(ATTRIBUTE_A, ATTRIBUTE_B)
                    -----------!

!ITGlorpDescriptorSystem methodsFor: 'accessing' stamp: 'afk 1/6/2008 13:22'!
allTableNames
	"Here are all tables listed which hold all persistent data."
	
	^ #('ISSUE' 'COMMENT' 'DETAIL' 'USERS' 'ROLE' 'USERROLE' 'ROLEPRIVILEGE' 'TAGS')! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:54'!
classModelForITIssue: aClassModel
	"The unique ID can be up to 1.073.741.823 - should be enough.
	 -> SmallInteger maxVal
	 4-Byte-Integers (signed) are used in the database"
	
	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #comments collection: OrderedCollection  of: ITIssueComment.
	aClassModel newAttributeNamed: #details collection: OrderedCollection  of: ITIssueDetail.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 1/5/2008 11:16'!
classModelForITIssueComment: aClassModel
	"See #classModelForITIssue for explanation of ID."
	
	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #content type: String.
	aClassModel newAttributeNamed: #timestamp type: TimeStamp.
	aClassModel newAttributeNamed: #user type: ITUser.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mfp 1/22/2008 15:24'!
classModelForITIssueDetail: aClassModel
	"See #classModelForITIssue for explanation of ID."

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #tag type: ITTag.
	aClassModel newAttributeNamed: #content type: String.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 1/5/2008 23:28'!
classModelForITRole: aClassModel

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #rolename type: String.
	aClassModel newAttributeNamed: #privileges collection: Set of: String.
	aClassModel newAttributeNamed: #users collection: Set of: ITUser.
	! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mig 1/12/2008 12:14'!
classModelForITTag: aClassModel

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #tagName type: String.
	aClassModel newAttributeNamed: #tagDescription type: String.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'amb 1/29/2008 15:44'!
classModelForITUser: aClassModel

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #username type: String.
	aClassModel newAttributeNamed: #passwordHash type: String.
	aClassModel newAttributeNamed: #isActive type: Character.
	aClassModel newAttributeNamed: #roles collection: Set of: ITRole.
! !

!ITGlorpDescriptorSystem methodsFor: 'accessing' stamp: 'afk 1/6/2008 12:23'!
constructAllClasses
	"Here are all classes listed which are necessary for the mappings."
	
	^ (super constructAllClasses)
		add: ITIssue;
		add: ITIssueComment;
		add: ITIssueDetail;
		add: ITUser;
		add: ITRole;
		add: ITTag;
		yourself! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 1/7/2008 21:25'!
descriptorForITIssue: aDescriptor
	"Every issue contains a unique ID, a collection of comments and a collection of details."
	
	| issueTable commentTable detailTable |
	issueTable   := self tableNamed: 'ISSUE'.
	commentTable := self tableNamed: 'COMMENT'.
	detailTable  := self tableNamed: 'DETAIL'.

	aDescriptor table: issueTable.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (issueTable fieldNamed: 'ID').

	(aDescriptor newMapping: ToManyMapping)
		attributeName: #comments;
		join: (Join
			from: (issueTable fieldNamed: 'ID')
			to: (commentTable	fieldNamed: 'ISSUE_ID')
		); beExclusive.
	
	(aDescriptor newMapping: ToManyMapping)
		attributeName: #details;
		join: (Join
			from: (issueTable fieldNamed: 'ID')
			to:  (detailTable fieldNamed: 'ISSUE_ID')
		); beExclusive.! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 1/5/2008 11:04'!
descriptorForITIssueComment: aDescriptor
	"Every comment contains a unique ID and its content - should be text."
	
	| table |
	table := self tableNamed: 'COMMENT'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #content to: (table fieldNamed: 'CONTENT').
	(aDescriptor newMapping: DirectMapping) from: #timestamp to: (table fieldNamed: 'TIMESTAMP').

	(aDescriptor newMapping: OneToOneMapping)
		attributeName: #user.! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mfp 1/22/2008 15:24'!
descriptorForITIssueDetail: aDescriptor
	"Every detail contains a unique ID, a tag and its content."

	| table |
	table := self tableNamed: 'DETAIL'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: OneToOneMapping) attributeName: #tag.
	(aDescriptor newMapping: DirectMapping) from: #content to: (table fieldNamed: 'CONTENT').! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 1/7/2008 21:44'!
descriptorForITRole: aDescriptor
	
	| roleTable userRoleTable rolePrivilegeTable |
	roleTable := self tableNamed: 'ROLE'.
	userRoleTable := self tableNamed: 'USERROLE'.
	rolePrivilegeTable := self tableNamed: 'ROLEPRIVILEGE'.

	aDescriptor table: roleTable.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (roleTable fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #rolename to: (roleTable fieldNamed: 'ROLENAME').

	(aDescriptor newMapping: ManyToManyMapping)
		attributeName: #users;
		referenceClass: ITUser;
		useLinkTable;
		mappingCriteria: (Join
			from: (roleTable fieldNamed: 'ID')
			to:  (userRoleTable fieldNamed: 'ROLE_ID')
		).
	
	(aDescriptor newMapping: DirectToManyMapping)
		attributeName: #privileges;
		resultField: (rolePrivilegeTable fieldNamed: 'PRIVILEGE');
		writeTheOrderField;
		orderBy: [:each | (each getTable: rolePrivilegeTable) getField: 'PRIVILEGE_ORDER'];
		join: (Join from: (roleTable fieldNamed: 'ID') to: (rolePrivilegeTable fieldNamed: 'ROLE_ID')).! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mfp 1/22/2008 15:20'!
descriptorForITTag: aDescriptor
	"Every tag contains a unique ID and its content - should be text."
	
	| table |
	table := self tableNamed: 'TAGS'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #tagName to: (table fieldNamed: 'TAGNAME').
	(aDescriptor newMapping: DirectMapping) from: #tagDescription to: (table fieldNamed: 'TAGDESCRIPTION').
! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'amb 1/29/2008 15:39'!
descriptorForITUser: aDescriptor
	
	| userTable userRoleTable |
	userTable   := self tableNamed: 'USERS'.
	userRoleTable := self tableNamed: 'USERROLE'.

	aDescriptor table: userTable.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (userTable fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #username to: (userTable fieldNamed: 'USERNAME').
	(aDescriptor newMapping: DirectMapping) from: #passwordHash to: (userTable fieldNamed: 'PASSWORDHASH').
	(aDescriptor newMapping: DirectMapping) from: #isActive to: (userTable fieldNamed: 'IS_ACTIVE').

	(aDescriptor newMapping: ManyToManyMapping)
		attributeName: #roles;
		referenceClass: ITRole;
		useLinkTable;
		mappingCriteria: (Join
			from: (userTable fieldNamed: 'ID')
			to:  (userRoleTable fieldNamed: 'USER_ID')
		).! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 1/7/2008 20:47'!
tableForCOMMENT: aTable
	"COMMENTS(ID, ISSUE_ID, TIMESTAMP, CONTENT)
	          --"
	
	| issueId userId|
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.
	issueId := aTable createFieldNamed: 'ISSUE_ID' type: (platform int4).
	userId := aTable createFieldNamed: 'USER_ID' type: (platform int4).	
	aTable createFieldNamed: 'TIMESTAMP' type: (platform timestamp).
	aTable createFieldNamed: 'CONTENT' type: (platform text).
	
	"foreign key definition for the IDs of issues"
	aTable 
		addForeignKeyFrom: issueId 
		to: ((self tableNamed: 'ISSUE') fieldNamed: 'ID')
		"suffixExpression: 'ON DELETE CASCADE'".
	aTable 
		addForeignKeyFrom: userId 
		to: ((self tableNamed: 'USERS') fieldNamed: 'ID')
		"suffixExpression: 'ON DELETE CASCADE'".! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mfp 1/22/2008 15:22'!
tableForDETAIL: aTable
	"DETAIL(ID, ISSUE_ID, TAG_ID, CONTENT)
	       --"
	
	| issueID tagID |
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.
	issueID := aTable createFieldNamed: 'ISSUE_ID' type: (platform int4).
	tagID := aTable createFieldNamed: 'TAG_ID' type: (platform int4).
	aTable createFieldNamed: 'CONTENT' type: (platform varchar: 255).
	
	"foreign key definition for IDs of issues"
	aTable
		addForeignKeyFrom: issueID
		to: ((self tableNamed: 'ISSUE') fieldNamed: 'ID');
		addForeignKeyFrom: tagID
		to: ((self tableNamed: 'TAGS') fieldNamed: 'ID')
		"suffixExpression: 'ON DELETE CASCADE'"! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 12/14/2007 09:38'!
tableForISSUE: aTable
	"ISSUE(ID)
	       --"
	
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mig 12/17/2007 00:50'!
tableForROLE: aTable
	"ROLE(ID, ROLENAME)"
	
	(aTable createFieldNamed: 'ID' type: platform sequence) bePrimaryKey.
	aTable createFieldNamed: 'ROLENAME' type: (platform varchar: 255).! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 1/7/2008 21:36'!
tableForROLEPRIVILEGE: aTable
	"ROLEPRIVILEGE(ROLE_ID, PRIVILEGE, PRIVILEGE_ORDER)"
	
	| roleid privilege |
	
	roleid := aTable createFieldNamed: 'ROLE_ID' type: platform int4.
	privilege := aTable createFieldNamed: 'PRIVILEGE' type: platform text.
	aTable createFieldNamed: 'PRIVILEGE_ORDER' type: platform int4.
	
	roleid bePrimaryKey.
	privilege bePrimaryKey.
	
	aTable 
		addForeignKeyFrom: roleid 
		to: ((self tableNamed: 'ROLE') fieldNamed: 'ID')
		suffixExpression: 'ON DELETE CASCADE'.
! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mfp 1/22/2008 15:19'!
tableForTAGS: aTable
	"TAGS(ID, TAGNAME, TAGDESCRIPTION)"
	
	(aTable createFieldNamed: 'ID' type: platform sequence) bePrimaryKey.
	aTable createFieldNamed: 'TAGNAME' type: (platform varchar: 255).
	aTable createFieldNamed: 'TAGDESCRIPTION' type: (platform varchar: 255).! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 1/7/2008 20:47'!
tableForUSERROLE: aTable
	"USERROLE(USER_ID, ROLE_ID)"

	| userid roleid |
	
	userid := aTable createFieldNamed: 'USER_ID' type: (platform int4).
	roleid := aTable createFieldNamed: 'ROLE_ID' type: (platform int4).
		
	aTable 
		addForeignKeyFrom: userid 
		to: ((self tableNamed: 'USERS') fieldNamed: 'ID')
		"suffixExpression: 'ON DELETE CASCADE'".
		
	aTable 
		addForeignKeyFrom: roleid 
		to: ((self tableNamed: 'ROLE') fieldNamed: 'ID')
		"suffixExpression: 'ON DELETE CASCADE'".
! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'amb 1/29/2008 15:34'!
tableForUSERS: aTable
	"USERS(ID, USERNAME, PASSWORDHASH)"
	
	(aTable createFieldNamed: 'ID' type: platform sequence) bePrimaryKey.
	aTable createFieldNamed: 'USERNAME' type: (platform varchar: 255).
	aTable createFieldNamed: 'PASSWORDHASH' type: (platform varchar: 255).
	aTable createFieldNamed: 'IS_ACTIVE' type: (platform char: 1).! !

WAFileLibrary subclass: #ITFileLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITFileLibrary commentStamp: 'mt 12/30/2007 14:39' prior: 0!
This is the place for all (former) external resources. Style sheets, images, other files. You have access to these files with: "ITFileLibrary / #filenameExtension" which creates you a WAUrl instance for proper usage.

You should modify these files through the web frontend of seaside. But feel free to modify my methods as well.!

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
acceptPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 159 73 68 65 84 56 203 165 147 235 75 83 97 28 199 253 59 118 206 118 108 3 9 100 68 33 130 132 80 123 21 36 18 59 154 13 197 188 44 75 211 221 189 210 54 99 216 76 139 50 114 94 198 72 41 45 179 212 106 115 78 109 234 212 230 214 148 50 113 217 81 66 204 189 233 66 181 97 116 177 111 231 236 197 76 18 35 122 224 11 15 15 207 231 243 123 174 113 0 226 254 39 127 12 20 248 14 137 114 167 15 234 179 61 41 76 198 227 253 97 233 136 56 44 117 37 50 82 103 162 62 221 190 87 180 171 32 207 155 74 203 60 201 33 157 71 134 155 65 11 250 150 187 162 233 92 108 70 137 235 24 36 189 84 72 210 67 209 59 10 216 170 116 230 120 82 228 234 156 17 206 213 126 216 94 94 131 105 174 50 26 174 239 88 237 67 227 76 21 14 216 248 145 100 27 159 222 38 200 241 164 8 51 221 73 235 28 204 77 172 9 148 161 194 95 2 205 204 25 232 216 148 179 169 246 157 133 253 245 61 92 156 170 128 216 66 174 139 175 147 194 152 64 230 78 50 168 198 178 162 149 57 152 3 85 222 83 80 76 23 66 49 85 0 245 84 33 220 107 131 48 120 149 112 176 146 220 158 35 72 184 66 26 98 130 140 17 49 211 25 108 134 53 216 132 10 95 49 148 79 228 44 152 15 229 36 27 79 62 198 223 184 192 181 80 100 13 109 207 27 97 155 107 68 124 61 201 196 4 71 237 9 27 15 86 110 163 54 160 129 214 91 196 174 100 0 139 31 230 161 154 40 196 216 218 80 20 254 177 249 29 109 207 46 193 48 81 140 190 96 39 4 70 98 35 38 144 220 167 54 250 151 187 97 244 171 80 235 215 226 211 215 143 81 232 253 151 183 49 216 50 91 15 181 43 27 122 247 105 244 7 59 32 168 249 93 208 67 49 55 230 155 208 190 112 25 186 73 57 204 190 106 68 190 125 142 194 155 63 55 97 121 122 1 101 206 44 104 88 65 75 160 14 237 94 51 168 42 98 107 11 169 183 4 6 249 64 26 236 43 119 81 61 33 135 218 125 18 117 211 229 88 122 183 128 182 217 6 148 14 30 135 194 113 2 58 103 14 236 175 186 145 110 61 12 170 146 216 58 196 100 43 95 184 143 189 26 179 71 131 135 204 29 84 142 230 65 59 156 3 213 144 12 74 7 23 14 206 198 163 165 46 24 135 138 33 80 243 214 41 53 33 220 246 144 18 155 72 58 190 129 136 152 220 101 112 176 146 214 128 25 250 209 34 156 27 150 163 149 221 130 157 133 245 206 34 240 75 121 17 22 166 119 124 202 123 26 72 154 50 17 33 105 135 4 237 126 51 122 95 88 209 59 111 133 197 107 66 90 75 42 4 10 94 136 82 17 244 174 159 137 58 79 138 40 3 161 167 106 8 70 160 229 133 5 42 94 152 173 200 176 209 83 165 132 232 175 191 241 95 243 11 103 208 172 229 121 186 212 99 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
addPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 111 73 68 65 84 56 203 165 147 235 75 147 97 24 135 253 91 182 47 5 131 164 134 89 40 168 41 37 88 40 111 217 108 138 138 78 219 150 115 107 138 206 246 110 46 157 186 205 3 45 157 19 15 21 181 161 219 104 164 59 56 15 195 3 102 226 212 210 69 234 235 80 162 141 34 8 106 195 175 191 222 21 77 71 203 136 30 248 125 121 224 186 158 231 190 185 239 36 0 73 255 147 223 46 116 211 117 204 14 143 132 108 117 137 168 230 9 65 88 254 162 58 220 240 188 130 146 90 203 72 209 104 49 243 68 129 110 90 74 180 79 74 66 207 214 123 177 26 90 132 255 243 6 214 63 173 96 50 96 135 210 83 3 190 137 19 226 61 185 78 36 20 252 132 197 145 233 61 59 14 190 237 97 246 131 11 38 106 16 143 119 12 176 239 155 176 244 113 14 218 5 25 74 71 242 35 220 193 60 34 78 160 157 146 50 104 56 232 222 181 96 239 235 54 134 183 245 120 184 214 129 110 95 43 12 126 29 250 252 90 116 111 182 192 125 96 7 233 17 162 208 120 37 88 208 155 205 136 9 104 88 209 191 172 198 187 47 155 244 139 125 232 221 210 66 243 74 137 95 71 189 38 131 124 81 8 197 114 45 156 251 54 220 28 201 65 222 131 12 69 76 208 226 172 161 92 1 27 253 85 51 58 87 85 104 91 145 67 54 43 138 9 4 54 46 26 102 202 32 157 42 198 192 27 29 244 75 205 184 220 157 70 197 4 138 113 254 161 247 189 11 253 111 117 52 220 132 63 29 241 100 17 20 243 124 88 252 195 200 210 165 30 198 4 77 118 222 225 68 96 12 218 13 18 42 95 253 137 2 210 91 13 171 127 8 233 29 236 35 65 189 173 156 50 250 180 48 108 105 208 244 82 8 241 124 5 120 214 27 113 96 52 119 61 92 244 251 212 232 242 222 195 5 117 242 81 9 226 177 18 197 109 43 1 71 192 2 217 124 37 36 222 210 31 53 31 23 136 156 28 212 185 139 225 216 53 227 82 79 42 216 247 89 71 77 20 154 139 24 85 79 11 131 164 71 128 113 106 52 214 176 40 88 235 140 38 10 115 49 177 99 194 29 203 173 40 28 76 86 158 102 196 13 82 249 163 2 162 100 232 106 164 209 81 9 39 45 49 250 218 65 206 84 65 62 85 9 227 106 27 28 52 44 180 112 192 86 177 34 52 76 36 28 101 206 64 46 65 24 114 66 249 3 153 232 89 32 97 126 109 128 121 195 0 141 183 17 89 93 41 81 56 116 28 78 184 76 215 244 153 204 220 158 116 50 187 243 34 149 161 73 9 167 181 159 13 159 111 61 67 157 83 177 200 100 229 41 230 95 183 241 95 243 29 65 70 203 31 0 40 211 193 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
arrowdownPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 4 0 0 0 181 250 55 234 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 0 91 73 68 65 84 40 207 99 248 207 128 31 50 12 41 5 7 254 239 250 191 249 255 234 255 139 254 207 248 223 247 13 139 130 237 95 119 253 223 255 127 39 80 209 156 255 245 231 177 40 88 45 177 232 203 138 255 27 255 207 252 95 245 173 80 17 171 27 38 75 116 126 159 242 191 236 91 174 28 78 71 86 73 20 190 206 148 27 130 33 9 0 142 20 10 167 81 131 80 58 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
arrowupPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 4 0 0 0 181 250 55 234 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 0 91 73 68 65 84 40 207 99 248 207 128 31 50 12 57 5 155 37 86 191 94 36 135 83 193 122 137 101 223 87 255 159 252 173 83 14 171 130 213 18 139 190 172 248 191 241 255 204 255 85 223 10 21 177 40 152 251 117 233 255 245 255 87 254 95 244 191 237 127 234 121 44 10 250 254 183 252 175 250 95 248 63 243 127 210 255 152 111 67 44 36 1 211 6 9 208 168 105 182 94 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
binPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 4 0 0 0 181 250 55 234 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 110 73 68 65 84 24 25 5 193 191 75 84 113 0 0 240 207 123 94 202 157 215 21 69 22 53 84 168 68 100 17 221 210 208 32 68 129 109 70 109 246 7 164 155 213 214 218 84 91 52 59 244 131 22 201 90 130 104 9 196 254 0 27 36 78 7 3 15 2 233 56 61 125 119 239 251 222 183 207 39 137 0 0 214 158 141 76 116 222 222 94 5 146 8 128 167 183 226 189 201 7 179 147 85 91 217 187 55 253 149 215 171 136 162 40 138 158 204 45 182 190 197 141 184 31 139 248 55 126 143 11 173 249 185 40 5 32 157 191 63 62 233 164 125 125 195 174 154 30 79 231 169 0 159 147 194 212 212 21 137 3 35 50 163 118 53 109 93 167 194 151 161 242 124 113 182 24 62 86 111 153 80 85 74 28 56 225 43 85 42 148 151 138 231 97 58 212 179 164 106 195 161 68 48 34 115 193 22 42 159 234 197 171 169 153 211 142 232 88 183 109 92 162 16 4 29 3 164 225 238 216 204 25 135 50 13 55 29 181 105 84 162 166 45 202 145 230 143 47 218 181 167 107 95 162 105 83 148 26 50 16 229 72 243 83 21 127 100 50 61 7 234 186 254 129 129 66 142 116 176 221 115 89 219 47 93 61 244 245 149 74 185 82 64 58 248 248 91 221 53 55 52 144 43 28 87 138 134 212 228 72 23 62 172 47 255 208 208 80 85 179 102 194 152 115 26 238 104 11 59 84 24 60 252 185 220 157 109 42 229 86 98 61 121 41 8 138 188 216 41 154 36 17 44 62 202 95 132 118 216 11 239 151 150 0 128 255 200 133 159 94 23 25 132 161 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
bookopenPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 0 73 68 65 84 56 203 165 83 207 107 19 81 16 158 149 183 77 92 104 126 40 90 77 83 209 146 165 24 138 61 68 111 129 82 193 10 226 65 15 253 15 42 61 121 233 169 23 79 5 255 0 241 102 161 135 30 197 67 65 133 244 82 122 82 75 42 193 244 210 4 49 178 7 145 228 146 152 66 246 215 219 238 55 219 183 36 16 79 29 120 217 188 153 239 155 249 102 118 86 11 130 128 46 98 2 63 47 182 14 75 202 33 125 159 252 243 179 243 114 241 8 190 231 175 63 150 60 215 37 151 143 67 174 19 61 191 188 93 59 210 86 223 125 3 185 122 235 170 65 82 130 40 57 137 29 130 126 253 237 222 3 113 58 123 249 7 5 62 121 158 71 184 227 121 252 243 15 37 146 137 251 34 144 126 149 180 75 180 241 116 46 150 37 165 164 202 247 223 212 176 218 21 16 30 45 152 244 228 193 236 136 244 210 234 27 154 208 245 42 183 0 185 176 122 189 206 255 81 97 105 126 158 238 92 17 55 39 39 83 52 51 149 165 90 173 198 177 126 191 79 229 114 153 28 199 14 91 153 136 102 0 2 204 52 77 6 161 87 203 178 200 106 181 168 88 44 146 109 27 28 131 191 215 235 49 214 177 109 114 18 97 2 188 4 200 132 53 26 13 150 15 96 58 157 102 176 29 2 85 12 133 160 32 151 203 133 10 48 76 135 132 244 61 38 40 5 60 168 240 180 219 109 26 126 197 133 66 129 253 221 110 151 239 32 131 39 224 196 5 224 102 179 25 207 32 149 74 197 100 21 83 10 242 249 124 148 192 25 74 48 172 0 242 58 157 14 19 149 10 165 64 205 0 213 189 112 23 132 231 70 4 88 50 153 140 43 26 134 65 153 76 134 143 16 130 116 93 31 73 168 90 21 216 40 76 116 113 125 59 222 48 37 239 89 249 46 19 118 62 127 141 42 134 4 255 156 200 173 186 156 192 165 219 211 215 198 238 249 135 131 67 6 61 94 94 26 27 223 221 253 68 98 48 24 80 171 247 111 100 199 149 130 27 83 215 185 82 101 111 127 172 2 172 188 246 191 175 209 92 217 220 62 61 237 63 196 142 164 51 217 147 230 251 87 203 227 112 218 69 63 231 51 134 70 155 238 48 139 180 96 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
cancelPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 221 73 68 65 84 56 203 165 147 107 107 26 65 20 134 247 107 72 32 33 41 166 52 132 150 94 99 155 160 168 161 198 152 20 52 101 141 74 155 218 75 164 105 40 189 209 203 255 204 135 92 32 127 163 196 149 160 178 206 174 187 59 79 103 86 55 85 148 150 210 15 47 11 195 190 207 153 247 156 51 6 96 252 143 198 15 13 195 10 143 7 10 70 101 253 25 96 24 13 30 221 135 239 239 224 219 1 242 235 91 228 151 58 242 211 27 228 135 87 248 15 110 227 25 70 123 50 32 50 255 56 4 179 136 204 100 144 169 20 65 50 73 176 182 134 159 207 226 31 62 167 119 239 38 66 255 59 2 24 54 31 212 225 252 28 89 171 17 36 18 4 171 171 248 149 10 242 236 12 175 90 194 219 223 197 189 179 140 61 128 68 128 254 149 85 101 109 166 211 129 102 19 191 90 197 47 151 193 82 209 219 109 228 201 9 189 196 67 122 47 10 116 66 235 16 32 204 155 78 19 236 237 133 230 8 18 74 155 21 164 183 189 141 179 184 136 251 236 9 173 97 128 238 176 252 188 255 59 175 186 114 100 12 165 204 238 214 22 78 44 134 88 88 192 169 228 39 0 62 190 238 231 141 199 241 75 165 17 128 174 238 230 114 136 249 121 186 179 179 56 165 236 56 32 120 95 195 95 89 193 55 205 171 204 250 171 205 180 90 200 139 11 132 154 140 61 61 141 243 116 157 203 97 128 167 1 106 68 222 227 20 242 244 180 95 181 209 192 221 220 196 217 216 8 205 82 65 130 163 35 186 203 215 17 197 244 40 192 85 27 166 231 235 213 203 120 102 129 224 248 56 52 235 188 221 185 57 132 110 174 50 139 245 36 98 39 67 123 233 26 77 195 248 57 178 72 106 57 44 61 223 222 203 29 220 248 221 171 188 246 204 12 246 212 20 246 82 44 172 220 26 50 143 173 178 90 14 203 190 117 3 183 154 199 217 205 225 152 217 48 175 40 102 16 133 212 152 121 226 99 82 221 181 116 135 35 93 14 212 236 171 249 247 215 248 143 250 5 231 149 184 131 253 158 221 191 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
commentaddPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 164 73 68 65 84 56 203 173 146 79 40 131 97 28 199 119 82 202 201 77 202 65 14 46 148 11 237 232 162 68 57 56 224 64 104 40 118 112 84 56 208 106 147 205 68 83 138 18 57 224 52 105 241 218 129 131 112 80 195 144 109 54 108 94 255 135 229 79 218 222 231 235 247 188 155 197 218 94 138 167 190 61 61 239 251 126 62 207 239 253 61 143 10 128 234 47 81 253 155 160 86 191 156 73 153 165 188 83 160 16 129 146 151 76 96 157 91 119 33 28 145 160 52 150 182 125 92 226 166 164 197 5 180 200 160 68 36 198 240 248 202 112 29 98 16 159 24 2 15 12 103 65 6 239 29 131 251 70 130 231 38 42 239 158 218 228 146 226 175 130 28 173 101 77 126 121 69 160 248 200 224 39 248 244 158 225 36 6 31 93 73 56 16 163 130 129 249 29 46 168 72 42 184 32 248 60 24 131 111 25 92 215 81 216 73 240 110 32 181 32 254 11 188 124 31 237 234 185 149 112 76 240 225 165 132 125 130 29 1 46 136 200 130 222 233 45 46 80 39 109 34 151 124 29 188 26 135 95 146 197 9 77 76 79 117 140 145 207 35 155 92 62 136 139 172 27 11 232 28 107 64 235 80 21 234 116 165 207 229 93 133 61 63 94 36 146 120 46 131 47 232 159 25 69 223 66 19 150 156 22 56 68 1 230 213 118 212 152 11 80 210 145 101 84 130 77 227 54 39 175 228 185 94 95 22 182 238 15 195 122 56 34 87 99 180 183 192 108 111 227 130 183 84 112 46 197 75 209 240 117 101 119 17 108 206 137 111 189 89 220 179 112 193 239 238 187 90 155 253 54 40 52 67 47 52 202 176 126 165 81 185 130 196 208 135 186 106 99 62 76 130 70 222 153 207 124 173 216 131 36 18 3 37 196 203 142 205 6 254 252 3 11 15 250 116 92 1 187 59 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
commentsPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 191 73 68 65 84 56 203 141 146 189 79 194 80 20 197 89 77 76 156 220 140 137 131 137 131 139 206 254 27 38 46 234 160 127 129 198 93 23 25 36 36 206 186 168 147 6 72 212 65 196 60 226 128 147 17 67 88 208 224 71 16 249 40 21 40 31 133 182 226 245 220 71 73 196 208 234 240 75 251 94 239 57 239 222 211 231 89 240 134 167 128 31 156 58 112 12 86 192 56 17 121 126 227 193 135 221 104 34 43 106 186 41 180 30 205 46 85 80 172 234 34 24 75 11 212 109 58 25 156 177 184 130 226 114 195 20 31 117 83 148 234 134 40 213 12 81 212 218 114 63 163 212 217 224 208 209 128 79 101 177 98 139 10 213 182 200 85 90 34 91 110 201 245 95 6 231 181 150 41 84 156 204 197 121 136 175 147 185 219 245 189 88 121 201 119 101 45 3 126 162 206 0 21 155 44 216 230 252 216 32 160 66 104 88 29 41 230 147 55 246 111 170 200 133 48 26 105 61 154 93 144 11 33 23 66 46 4 109 144 13 86 125 129 184 136 167 21 97 126 118 36 139 59 17 139 197 152 159 48 26 33 23 66 46 132 92 8 93 202 125 140 197 6 143 114 14 188 204 129 19 78 252 242 46 195 243 90 124 42 139 21 91 132 92 8 221 17 114 145 235 62 3 219 228 32 18 151 226 48 168 35 23 66 46 178 56 255 67 156 81 117 202 227 253 93 109 176 193 75 79 60 196 97 130 11 48 9 10 200 133 218 86 167 79 204 93 40 216 71 94 244 90 172 177 65 170 103 48 10 182 192 132 189 246 250 67 247 148 120 46 17 50 145 176 217 91 169 65 79 121 141 142 162 41 22 235 96 205 51 232 223 218 38 243 64 211 116 131 144 11 11 190 64 26 36 249 246 130 17 121 15 156 12 108 147 7 228 194 226 54 152 25 120 145 92 196 195 220 1 104 129 89 167 58 55 131 49 16 2 211 110 93 186 142 240 31 190 1 193 159 9 148 122 211 60 183 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
datePng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 4 73 68 65 84 56 203 149 82 109 107 26 65 16 126 238 188 212 139 231 75 36 20 26 130 136 53 88 164 37 162 32 8 237 199 130 33 224 143 243 179 191 35 144 63 80 138 133 64 80 17 141 90 40 42 69 250 65 197 35 190 220 185 219 153 141 119 152 212 22 58 240 48 187 59 51 207 188 236 64 74 9 6 137 150 205 102 63 87 171 213 111 153 76 230 138 238 6 191 237 96 20 10 133 79 181 90 237 43 251 240 155 23 167 14 3 77 147 191 216 235 230 6 231 249 60 134 119 119 16 149 10 94 227 73 216 22 188 189 197 155 203 75 252 188 191 199 230 250 90 217 210 82 106 186 70 194 78 86 48 8 43 22 195 118 187 69 56 30 199 113 32 128 184 174 43 152 228 98 70 34 202 118 28 141 194 220 17 115 172 190 59 195 52 12 116 6 3 229 212 237 245 32 136 196 62 57 129 29 14 99 107 154 232 116 187 202 214 121 120 240 9 224 245 216 7 68 140 178 55 206 206 208 179 109 92 80 246 20 181 166 83 128 116 28 172 93 23 223 73 15 8 111 133 64 129 244 35 217 47 0 221 216 175 224 195 233 41 222 91 22 228 102 3 151 156 4 131 200 94 17 209 187 163 35 100 72 107 132 224 98 129 71 34 245 171 168 215 235 210 117 221 191 194 113 28 185 92 46 21 88 248 141 99 56 214 192 1 89 175 215 104 183 219 72 167 211 232 247 251 170 247 249 124 142 114 185 236 125 185 47 250 33 2 14 30 141 70 136 208 228 115 185 28 82 169 20 66 161 16 40 179 15 143 232 25 129 247 152 167 93 240 22 133 209 106 181 84 53 14 15 148 170 163 86 252 152 63 90 120 89 226 108 54 131 160 201 115 240 100 50 81 103 134 231 119 112 6 251 100 220 14 173 182 154 67 50 153 228 229 193 112 56 132 65 191 198 3 213 229 142 138 148 216 15 174 208 42 179 169 88 44 194 162 175 229 172 28 60 30 143 125 34 127 6 11 250 87 155 132 142 130 13 251 8 208 30 176 230 10 56 115 34 145 192 106 181 82 45 249 45 116 105 77 201 49 202 101 253 75 188 10 152 172 217 108 162 84 42 61 17 76 167 211 47 141 70 227 35 254 67 232 55 126 176 254 13 45 58 77 7 87 49 185 249 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
dePng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 11 8 2 0 0 0 249 128 154 110 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 179 73 68 65 84 120 218 98 244 241 113 102 248 251 157 225 223 63 6 102 150 95 127 254 253 250 253 231 255 191 223 12 12 255 24 112 0 128 0 98 249 253 251 123 113 113 57 144 245 15 12 254 254 253 243 247 47 132 252 3 5 191 127 255 249 11 166 126 255 94 183 110 53 64 0 177 48 49 49 1 85 159 56 121 9 36 1 22 254 5 4 63 65 16 14 126 252 248 1 196 254 1 78 64 149 0 1 196 34 203 192 32 240 239 159 44 88 245 159 95 191 64 58 126 254 4 34 160 174 223 63 126 252 250 249 3 168 225 247 247 31 64 253 18 127 128 78 101 0 8 64 98 28 220 0 0 130 64 16 228 129 21 209 140 253 55 225 29 8 104 52 217 76 86 103 208 50 45 162 221 251 75 222 10 44 226 13 122 177 28 35 182 138 28 1 196 242 255 247 175 255 127 255 50 0 213 253 250 253 31 168 226 215 175 127 80 13 96 213 64 244 253 59 68 63 72 37 3 3 64 0 177 0 49 195 223 63 8 131 129 140 31 63 128 170 161 218 128 170 127 253 252 11 84 13 20 255 251 251 47 3 3 64 0 177 252 182 98 96 80 251 255 159 249 223 255 191 255 254 253 6 162 191 16 198 127 80 240 2 73 134 191 191 255 3 201 127 64 210 132 241 199 97 6 128 0 98 249 175 199 240 95 230 223 127 65 160 42 16 250 15 36 129 22 130 66 248 15 36 152 129 98 96 46 195 127 97 198 127 174 12 0 1 196 194 240 139 225 63 208 251 172 70 76 12 191 24 65 26 128 228 31 134 255 191 255 131 216 64 242 207 255 255 64 29 127 64 182 252 250 195 244 149 1 32 128 88 126 188 101 216 51 111 210 63 160 54 102 134 159 223 65 49 252 159 143 129 233 7 195 63 96 64 48 51 252 253 5 98 176 50 49 252 251 193 192 192 204 240 237 23 3 64 128 1 0 119 116 136 201 24 173 95 88 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
deletePng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 93 73 68 65 84 56 203 165 147 251 75 83 97 24 199 253 91 182 31 162 4 137 110 132 132 81 80 152 243 50 119 214 218 220 166 206 179 76 143 91 44 98 105 142 157 97 154 65 141 197 92 248 67 168 165 118 213 202 95 50 77 108 148 90 70 106 215 161 229 177 242 210 78 77 106 109 158 179 107 202 183 185 96 38 46 35 122 225 251 203 203 251 249 188 60 15 207 147 2 32 229 127 178 230 98 86 175 23 206 80 21 230 247 84 25 51 165 37 185 73 173 134 123 71 170 153 113 82 105 118 149 200 133 235 10 230 116 122 226 35 69 177 223 28 54 132 7 157 136 188 25 69 100 100 8 252 221 14 204 26 74 241 170 144 96 159 171 197 68 82 193 50 60 93 78 241 11 183 59 176 52 245 22 209 190 59 136 182 219 17 109 62 135 31 55 155 176 56 220 7 143 201 128 81 101 54 255 76 158 73 172 18 204 232 116 130 24 236 230 174 183 99 137 113 33 122 241 12 124 118 11 252 182 106 132 47 88 16 105 160 17 182 158 64 248 222 13 204 29 37 49 124 104 159 251 177 108 143 32 33 136 193 244 124 173 25 139 174 177 248 143 33 7 13 239 89 35 130 117 186 85 225 78 146 8 119 93 193 203 188 12 12 72 51 232 132 224 3 117 132 9 116 93 69 180 179 25 62 107 37 190 22 73 147 102 161 146 4 111 171 129 199 82 133 135 68 58 147 16 48 229 218 96 228 126 23 162 14 11 124 167 13 248 211 241 40 114 224 165 10 225 111 110 132 51 111 71 48 33 152 36 139 130 161 206 86 132 235 13 8 158 42 91 87 48 95 170 130 191 169 17 253 226 45 43 130 137 18 21 227 181 214 32 100 167 193 29 87 199 31 38 141 50 15 190 90 19 102 77 70 244 137 210 86 74 112 21 203 105 70 38 66 176 179 13 62 173 12 222 82 201 26 152 149 103 131 45 144 32 208 126 9 67 226 109 232 205 218 180 210 196 215 69 82 193 11 13 225 158 171 208 32 112 171 53 222 176 121 149 248 23 168 200 5 43 139 193 50 49 248 182 22 140 23 75 151 97 119 183 104 163 96 213 32 141 21 228 16 35 138 3 252 244 97 21 2 215 90 241 189 158 134 135 84 226 179 90 1 111 157 25 252 229 22 76 168 243 209 147 149 202 199 96 34 233 40 63 149 239 39 158 28 220 203 142 74 118 225 75 181 17 222 134 243 241 124 170 58 134 71 57 91 151 97 246 119 56 233 50 13 74 119 11 7 196 233 102 39 177 147 121 144 191 157 235 23 109 230 122 115 211 152 158 236 84 115 119 230 6 225 95 183 241 95 243 19 29 210 206 185 73 114 27 254 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
deutschTs
	^ '<?xml version="1.0" encoding="utf-8"?>
<!!DOCTYPE TS><TS version="1.1" language="de_DE">
<defaultcodec></defaultcodec>
<context>
    <name>SWE-IssueTracker</name>
    <message>
        <location filename="" line="0"/>
        <source>Select a operation</source>
        <translation>Operation wählen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You have to add a title to the issue.</source>
        <translation>Sie müssen dem Ticket einen Titel geben.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Install a new language</source>
        <translation>Neue Sprache installieren</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Available Privileges</source>
        <translation>Verfügbare Rechte</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Your current password was not correct. Please try again.</source>
        <translation>Das Passwort war ungültig. Bitte erneut versuchen.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You have to add a comment to the issue.</source>
        <translation>Sie müssen dem Ticket einen Kommentar hinzufügen.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Currently available tags:</source>
        <translation>Verfügbare Tags:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Attached Roles</source>
        <translation>Zugeordnete Rollen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>attach roles to users</source>
        <translation>Rollen zu Benutzern zuordnen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete_user</source>
        <translation>Nutzer löschen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to list all issues!!</source>
        <translation>Sie sind nicht autorisiert, alle Aufgaben anzuzeigen!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Attached Privileges</source>
        <translation>Zugeordnete Rechte</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>No</source>
        <translation>Nein</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>append comments on those issues</source>
        <translation>Kommentare zu diesen Tickets hinzufügen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>There is no role available to be configured. You need to add a new one.</source>
        <translation>Es gibt keine Rollen zum Konfigurieren. Sie müssen eine neue Rolle erstellen.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Uninstall</source>
        <translation>Deinstallieren</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Add a new tag:</source>
        <translation>Neuen Tag hinzufügen:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>view_admin_panel</source>
        <translation>Adminbereich anzeigen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Change Password</source>
        <translation>Passwort ändern</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please apply any changes with caution!!</source>
        <translation>Gehen Sie behutsam mit den Einstellungen um!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Change Username</source>
        <translation>Benutzername ändern</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Empty Comment</source>
        <translation>Leerer Kommentar</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>From</source>
        <translation>Von</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Apply changes</source>
        <translation>Übernehmen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Additionally, you can create your own new tags. These custom tags can be added to any issue by all users which are allowed to add or modify them. Every custom tag contains:</source>
        <translation>Zudem können Sie Ihre eigenen Tags erstellen. Diese benutzerdefinierten Tags können in jedem Ticket genutzt werden. Sie beinhalten:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Roles -&gt; Privileges</source>
        <translation>Rollen -&gt; Rechte</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Edit security settings for user &apos;{1}&apos;</source>
        <translation>Sicherheitsprofil von &apos;{1}&apos;</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>User Control</source>
        <translation>Benutzerverwaltung</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select the language ID of an installed phrasebook and press &apos;Uninstall&apos; to remove all translations which are in that book from the application.</source>
        <translation>Wählen Sie ein installiertes Wörterbuch aus und drücken Sie &apos;Deinstallieren&apos;, um alle Übersetzungen, welche sich in diesem Buch finden, aus der Anwendung zu löschen.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please enter your password and choose a new username. You cannot choose:</source>
        <translation>Geben Sie Ihr Passwort ein und wählen Sie einen neuen Benutzernamen. Folgende sind nicht zugelassen:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>list_issues</source>
        <translation>Alle Aufgaben auflisten</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>custom filters for all issues</source>
        <translation>benutzerdefinierte Filter für alle Tickets</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>At this moment, yaiiit!! tracks itself during its developing phase.</source>
        <translation>Momentan verwaltet yaiiit!! die Tickets seiner eigenen Projektentwicklung.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Register Error</source>
        <translation>Fehler beim Registrieren</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Enter a proper language ID and press the Install-Button. Note, that present IDs will be overwritten.</source>
        <translation>Geben Sie eine passende ID für das Wörterbuch an und drücken Sie &apos;Installieren&apos;. Bücher unter vorhandener ID werden überschrieben.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You have several options for different tags. For example:</source>
        <translation>Sie haben verschiedene Optionen in den jeweiligen Tags. Beispielsweise:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Note, that you cannot remove #en because this is the default language.</source>
        <translation>Das Wörterbuch #en kann nicht entfernt werden, da es die Standardsprache darstellt.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Yes</source>
        <translation>Ja</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Welcome to the User Settings!!</source>
        <translation>Willkommen in den Benutzereinstellungen!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Edit settings for role: </source>
        <translation>Einstellungen für Rolle: </translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>list_users</source>
        <translation>Nutzer auflisten</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>{1} by {2}</source>
        <translation>{1} von {2}</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to add a role!!</source>
        <translation>Sie sind nicht autorisiert, eine Rolle hinzuzufügen!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Available Roles</source>
        <translation>Verfügbare Rollen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select another user:</source>
        <translation>Weiteren Benutzer auswählen:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>The new passwords do not match. Please try again.</source>
        <translation>Die neuen Passwörter stimmen nicht überein. Bitte nochmal versuchen.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Delete Issue #{1}</source>
        <translation>Lösche Ticket #{1}</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Administration</source>
        <translation>Administration</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Create a new Issue</source>
        <translation>Neues Ticket erstellen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Do you really want to delete the role: &apos;{1}&apos;?</source>
        <translation>Wollen Sie wirklich die Rolle &apos;{1}&apos; löschen?</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Create Issue</source>
        <translation>Ticket erstellen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select argument(s)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You cannot choose an empty username.</source>
        <translation>Der Benutzername darf nicht leer sein.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please enter some content to the selected tag.</source>
        <translation>Geben Sie bitte einen Text für den ausgewählten Tag ein.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>And Remember that usernames are compared case-insensitive. So &apos;USER&apos; and &apos;uSeR&apos; will be treated as the same.</source>
        <translation>Benutzernamen werden case-insensitive verglichen. Deswegen sind &apos;USER&apos; und &apos;uSeR&apos; die gleichen Benutzer.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Our current features include:</source>
        <translation>Unsere aktuellen Produktfeatures beinhalten:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You cannot choose an empty username or &apos;admin&apos; or &apos;guest &apos;</source>
        <translation>Der Benutzername darf weder leer noch &apos;admin&apos; oder &apos;guest&apos; sein</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select a tag:</source>
        <translation>Tag wählen:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Tags</source>
        <translation>Tags</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Information</source>
        <translation>Informationen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add_role</source>
        <translation>Rolle hinzufügen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Click to add a comment...</source>
        <translation>Hier klicken, um Kommentar zu erstellen...</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Settings</source>
        <translation>Einstellungen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Or selecting one of those self explaining options (equals, until, between, since)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Do you really want to delete the current issue?</source>
        <translation>Wollen Sie wirklich das aktuelle Ticket entfernen?</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Verify Password</source>
        <translation>Passwort verifizieren</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Today</source>
        <translation>Heute</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Apply Changes</source>
        <translation>Änderungen übernehmen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add, modify and delete issues</source>
        <translation>Tickets hinzufügen, bearbeiten und entfernen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Request</source>
        <translation>Abfrage</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>User Settings</source>
        <translation>Benutzereinstellungen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>All Issues ({1})</source>
        <translation>Alle Tickets ({1})</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Comments:</source>
        <translation>Kommentare:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Home</source>
        <translation>Start</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Logout</source>
        <translation>Abmelden</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Custom Tags</source>
        <translation>Eigene Tags</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Delete Issue</source>
        <translation>Ticket entfernen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>No Title</source>
        <translation>Titel fehlt</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Feel free to register and contribute to our growing community. We are 5 developers at the moment. If you face any problems do not hesitate to mail to developer@yaiiit.net. We will enable you to post your found issues and help making yaiiit!! more reliable.</source>
        <translation>Registrieren Sie sich noch heute und werden Sie Teil unserer stets wachsenden Community. Momentan sind wir 5 Entwickler. Sollten irgendwelche Probleme mit der Anwendung auftauchen, schreiben Sie einfach an developer@yaiiit.net. Wir werden Ihnen die nötigen Rechte freischalten, damit Sie selbst Tickets erstellen können.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>the required input field with default values (not yet)</source>
        <translation>das benötigte Eingabefeld mit Standardwerten (noch nicht)</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>New Issue</source>
        <translation>Neues Ticket</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Delete Role</source>
        <translation>Rolle löschen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add role</source>
        <translation>neue Rolle</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>a security system based on privileges and roles</source>
        <translation>ein Sicherheitssystem, welches auf Privilegien und Rollen basiert</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>View Issues</source>
        <translation>Alle Tickets</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Uninstall a language</source>
        <translation>Sprache deinstallieren</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>User -&gt; Roles</source>
        <translation>Benutzer -&gt; Rollen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>an administration area to control security, languages and tags</source>
        <translation>ein Administrationsbereich um Sicherheits-, Sprach- und Tageinstellungen vorzunehmen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>test</source>
        <translation>Test</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>a generic tag approach to customize the issue&apos;s details</source>
        <translation>ein generischer Tag-Ansatz um die Details in einem Ticket zu setzen, die man braucht</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>OK</source>
        <translation>OK</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please enter your username and the correct password to login.</source>
        <translation>Bitte geben Sie Ihren Benutzernamen und das richtige Passwort ein, um sich anzumelden.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please choose a username and type your password twice to verify.</source>
        <translation>Bitte wählen Sie einen Benutzernamen und geben Sie ein Passwort zur Überprüfung zweimal ein.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Delete Detail</source>
        <translation>Detail entfernen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to add issues!!</source>
        <translation>Sie sind nicht autorisiert, um Tickets zu erstellen!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>logged in as {1}</source>
        <translation>Angemeldet als {1}</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>set a new username to an unused one</source>
        <translation>neuen, verfügbaren Benutzernamen wählen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>The user &apos;{1}&apos; already exists or you are not allowed to register at the moment.</source>
        <translation>Der Benutzer &apos;{1}&apos; existiert bereits oder es ist Ihnen momentan nicht gestattet, sich zu registrieren.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Description</source>
        <translation>Beschreibung</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Apply</source>
        <translation>Übernehmen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>create new roles</source>
        <translation>Neue Rollen erstellen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add_comment</source>
        <translation>Kommentar hinzufügen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Option</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Do nothing</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete_tag</source>
        <translation>Tag entfernen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Actual Privileges</source>
        <translation>Effektive Rechte</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Username</source>
        <translation>Benutzername</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Apply filter</source>
        <translation>Filter übernehmen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add_tag</source>
        <translation>Tag hinzufügen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Register</source>
        <translation>Registrieren</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Here you are able to create and edit conditions to filter the issues you see.</source>
        <translation>Sie können hier Bedingungen erstellen und bearbeiten um nur gewünschte Tickets auszuwählen.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Filter the issues</source>
        <translation>Tickets filtern</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>rename_user</source>
        <translation>Nutzer umbenennen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Install Languages</source>
        <translation>Sprache installieren</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Fill all input fields.</source>
        <translation>Füllen Sie alle Eingabefelder.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Add this tag</source>
        <translation>Diesen Tag hinzufügen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Install</source>
        <translation>Installieren</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Add Comment</source>
        <translation>Kommentar hinzufügen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>view_home</source>
        <translation>Startseite anzeigen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You can install any new language to the internal translator which is present in the application&apos;s file library.</source>
        <translation>Sie können jede Sprache zum internen Übersetzer hinzufügen, die in der Anwendungsdateibibliothek vorhanden ist.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Add a new role</source>
        <translation>Neue Rolle hinzufügen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Choose a tag and an operation first.</source>
        <translation>Wählen Sie einen Tag und einen Operator.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Here you are able to create a new issue. Any detail can be attached to this issue. You have to insert an initial comment so that this issue can be referenced to you.</source>
        <translation>Hier können Sie ein neues Ticket erstellen. Beliebige Details können diesem hinzugefügt werden. Sie müssen einen initialen Kommentar abgeben, sodass dieses Ticket mit ihnen in Verbindung gebracht werden kann.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to delete a role!!</source>
        <translation>Sie sind nicht autorisiert, eine Rolle zu löschen!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete</source>
        <translation>Löschen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Saving...</source>
        <translation>Speichern...</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Do you really want to delete &apos;{1}&apos;?</source>
        <translation>Wollen Sie wirklich &apos;{1}&apos; entfernen?</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You cannot delete that tag while it is used by an issue.</source>
        <translation>Sie dürfen keinen Tag löschen, solange dieser von einem Ticket benutzt wird.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>This is just another issue tracker. You can track all tasks, issues or bugs of your software project. yaiiit!! tries to integrate easily in your project&apos;s workflow with a clear design and simple usage.</source>
        <translation>Sie können alle Aufgaben, Fehler oder sonstige Probleme von ihrem Softwareprojekt verwalten. yaiiit!! integriert sich mit seinem klaren Design und der einfachen Bedienung problemlos in Ihren Softwareentwicklungsprozess.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>The password was changed successfully. Don&apos;t forget it.</source>
        <translation>Das Passwort wurde erfolgreich geändert. Vergessen Sie es nicht.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Bad Username</source>
        <translation>Falscher Benutzername</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Attached Users</source>
        <translation>Zugeordnete Benutzer</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Add</source>
        <translation>Hinzufügen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Visit yaiiit!! project page on http://www.yaiiit.net</source>
        <translation>Besuchen Sie die yaiiit!!-Projektseite auf http://www.yaiiit.net</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Welcome to the Administration Area!!</source>
        <translation>Willkommen im Administrationsbereich!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>New Username</source>
        <translation>Neuer Benutzername</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to delete an issue!!</source>
        <translation>Sie sind nicht autorisiert, eine Aufgabe zu löschen!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Comment:</source>
        <translation>Kommentar:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Enter a new comment here:</source>
        <translation>Geben Sie hier Ihren neuen Kommentar ein:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Password</source>
        <translation>Passwort</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Register a new user</source>
        <translation>Neuen Benutzer registrieren</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>a clear and brief description of the tag</source>
        <translation>kurze Beschreibung des Tags</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Issues per page: </source>
        <translation>Tickets pro Seite: </translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete_role</source>
        <translation>Rolle entfernen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select another role:</source>
        <translation>Weitere Rolle auswählen:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>an user area to setup username and password</source>
        <translation>Benutzereinstellungen um Name und Passwort zu ändern</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Cancel</source>
        <translation>Abbrechen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Choose language:</source>
        <translation>Sprache wählen:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to administrate this application!!</source>
        <translation>Sie sind nicht autorisiert, diese Anwendung zu administrieren!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please enter a value for the selected tag.</source>
        <translation>Bitte geben Sie einen Wert für den ausgewählten Tag an.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Your User-ID cannot be changed. You&apos;ll keep on being the same user as before with all created issues or posted comments.</source>
        <translation>Ihre User-ID kann nicht geändert werden. Sie bleiben der gleiche Nutzer mit all Ihren erstellten Tickets und geposteten Kommentaren.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Created</source>
        <translation>Erstellt</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>That username already exists. Please choose another one.</source>
        <translation>Der gewählte Benutzername existiert bereits. Wählen Sie einen anderen.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to add an issue!!</source>
        <translation>Sie sind nicht autorisiert, eine Aufgabe hinzuzufügen!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Your password is not long enough.</source>
        <translation>Ihr Passwort ist zu kurz.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please type your old password once and your new password twice to verify. Keep in mind that you cannot login with the old credentials anymore.</source>
        <translation>Bitte geben Sie Ihr aktuelles Passwort einmal und Ihr neues Passwort zweimal ein, um es zu verifizieren. Denken Sie daran, dass sie sich nach erfolgreicher Änderung nicht mehr mit den alten Daten anmelden können.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>read_issue</source>
        <translation>Aufgaben lesen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add_user</source>
        <translation>Nutzer hinzufügen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Name</source>
        <translation>Name</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Translation</source>
        <translation>Übersetzung</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>So enjoy your visit on http://www.yaiiit.net.</source>
        <translation>Genießen Sie Ihren Besuch auf http://www.yaiiit.net.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Yesterday</source>
        <translation>Gestern</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Login</source>
        <translation>Anmelden</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Owner</source>
        <translation>Eigentümer</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete_issue</source>
        <translation>Aufgabe löschen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>The passwords you entered do not match.</source>
        <translation>Die eingegebenen Kennwörter stimmen nicht überein.</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Yet Another Issue Issue Issue Tracker!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to create a tag!!</source>
        <translation>Sie sind nicht autorisiert, einen Tag hinzuzufügen!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>In this area you can modify several security settings:</source>
        <translation>In diesem Bereich können Sie einige Sicherheitseinstellungen modifizieren:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Tag</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Content</source>
        <translation>Wert</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete roles</source>
        <translation>Rollen löschen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>New Role</source>
        <translation>Neue Rolle</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Do not forget to keep any changes in mind because you&apos;ll not be able to login with the deprecated credentials anymore!!</source>
        <translation>Vergessen Sie nicht, sich jegliche Änderungen zu merken. Sie werden nicht in der Lage sein, sich mit den alten Daten anzumelden!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>detach roles from users</source>
        <translation>Rollen von Nutzern entfernen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>New Password</source>
        <translation>Neues Passwort</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>a descriptive title</source>
        <translation>beschreibender Titel</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>By selecting &quot;matches&quot; you can use two wildcards (# = one char and * = arbitrary number of chars)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>change your password</source>
        <translation>Ihr Passwort ändern</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized change user settings!!</source>
        <translation>Sie sind nicht autorisiert, die Benutzereinstellungen zu ändern!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Change your Password</source>
        <translation>Passwort ändern</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Edit settings for role &apos;{1}&apos;</source>
        <translation>Einstellungen für Rolle &apos;{1}&apos;</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>modify privileges for each role</source>
        <translation>zugeordnete Rechte für jede Rolle bearbeiten</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Change your Username</source>
        <translation>Benutzername ändern</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Filter Issues</source>
        <translation>Tickets filtern</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select a tag: </source>
        <translation>Tag wählen: </translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Title</source>
        <translation>Titel</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to delete a tag!!</source>
        <translation>Sie sind nicht autorisiert, einen Tag zu löschen!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Last Comment</source>
        <translation>Letzter Kommentar</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add_issue</source>
        <translation>Aufgabe hinzufügen</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>In here you are able to change your account&apos;s details without any need to get permissions from your webmaster or administrator. This includes the following:</source>
        <translation>Sie sind in der Lage Ihre persönlichen Einstellungen zu bearbeiten ohne die Erlaubnis von einem Webmaster oder Administrator zu benötigen. Dies beinhaltet folgendes:</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to view the details of an issue!!</source>
        <translation>Sie sind nicht autorisiert, Details einer Aufgabe anzuzeigen!!</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Current Password</source>
        <translation>Aktuelles Passwort</translation>
    </message>
</context>
</TS>
'! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
enPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 11 8 2 0 0 0 249 128 154 110 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 249 73 68 65 84 120 218 98 156 52 115 159 154 141 222 225 67 183 94 190 97 96 96 248 251 247 223 223 138 24 221 43 109 83 254 255 253 171 91 155 95 208 119 242 255 111 166 175 223 126 200 170 114 58 93 216 118 223 39 140 69 136 159 199 253 206 62 183 244 208 187 143 191 30 63 255 252 239 127 70 85 69 33 53 67 110 134 95 63 255 43 10 121 154 201 254 254 243 215 70 91 216 244 208 76 134 53 93 147 182 94 103 218 119 231 251 94 9 23 198 229 43 84 30 156 140 245 87 121 254 252 39 35 35 3 195 197 187 12 151 238 0 25 143 222 254 79 103 189 163 119 120 5 99 105 243 130 181 55 59 155 78 48 1 221 241 244 249 251 250 55 122 143 254 42 253 109 110 168 204 212 2 138 48 124 255 204 240 253 59 144 238 190 63 157 219 205 230 102 96 146 147 215 138 43 55 62 253 248 249 147 225 230 221 183 255 254 253 71 7 161 161 255 253 252 208 196 128 202 206 92 123 201 114 185 101 130 154 17 55 195 197 59 12 191 127 51 252 250 5 37 129 146 63 127 254 113 112 248 255 235 215 191 223 191 255 1 69 254 254 101 209 144 222 121 230 11 14 27 108 109 255 152 89 64 204 5 226 191 127 255 254 249 251 231 247 159 63 39 175 190 96 74 107 60 218 55 239 66 116 250 206 171 215 62 48 188 123 251 43 37 1 232 244 255 95 191 3 109 0 121 134 225 255 51 103 167 67 115 118 200 200 205 139 76 217 108 110 215 204 244 227 199 31 113 97 182 165 51 221 229 215 244 127 156 59 47 89 48 29 168 234 223 207 159 64 199 48 252 255 111 237 176 92 100 239 62 83 198 23 23 92 15 233 153 202 51 252 251 195 20 19 171 26 237 175 241 163 163 242 146 125 108 227 55 115 33 158 191 255 255 51 48 106 74 51 106 74 0 29 35 171 46 20 19 191 230 176 166 173 232 156 57 153 167 235 179 19 109 89 158 191 126 59 205 45 109 189 170 253 185 165 7 127 255 249 254 231 239 223 196 48 213 109 199 63 252 255 243 199 253 250 155 149 27 247 50 252 251 187 122 235 33 17 57 193 202 238 174 91 149 211 1 2 12 0 13 196 65 10 229 75 126 129 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
englishTs
	^ '<?xml version="1.0" encoding="utf-8"?>
<!!DOCTYPE TS><TS version="1.1" language="de_DE">
<defaultcodec></defaultcodec>
<context>
    <name>SWE-IssueTracker</name>
    <message>
        <location filename="" line="0"/>
        <source>Select a operation</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You have to add a title to the issue.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Install a new language</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Available Privileges</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Your current password was not correct. Please try again.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You have to add a comment to the issue.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Currently available tags:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Attached Roles</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>attach roles to users</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete_user</source>
        <translation>Delete User</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to list all issues!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Attached Privileges</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>No</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>append comments on those issues</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>There is no role available to be configured. You need to add a new one.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Uninstall</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Add a new tag:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>view_admin_panel</source>
        <translation>View Admin</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Change Password</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please apply any changes with caution!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Change Username</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Empty Comment</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>From</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Apply changes</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Additionally, you can create your own new tags. These custom tags can be added to any issue by all users which are allowed to add or modify them. Every custom tag contains:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Roles -&gt; Privileges</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Edit security settings for user &apos;{1}&apos;</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>User Control</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select the language ID of an installed phrasebook and press &apos;Uninstall&apos; to remove all translations which are in that book from the application.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please enter your password and choose a new username. You cannot choose:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>list_issues</source>
        <translation>List Issues</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>custom filters for all issues</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>At this moment, yaiiit!! tracks itself during its developing phase.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Register Error</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Enter a proper language ID and press the Install-Button. Note, that present IDs will be overwritten.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You have several options for different tags. For example:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Note, that you cannot remove #en because this is the default language.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Yes</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Welcome to the User Settings!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Edit settings for role: </source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>list_users</source>
        <translation>List Users</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>{1} by {2}</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to add a role!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Available Roles</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select another user:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>The new passwords do not match. Please try again.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Delete Issue #{1}</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Administration</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Create a new Issue</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Do you really want to delete the role: &apos;{1}&apos;?</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Create Issue</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select argument(s)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You cannot choose an empty username.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please enter some content to the selected tag.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Our current features include:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You cannot choose an empty username or &apos;admin&apos; or &apos;guest &apos;</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select a tag:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Tags</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Information</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add_role</source>
        <translation>Add Role</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Click to add a comment...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Settings</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Or selecting one of those self explaining options (equals, until, between, since)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Do you really want to delete the current issue?</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Verify Password</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Today</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Apply Changes</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add, modify and delete issues</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Request</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>User Settings</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>All Issues ({1})</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Comments:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Home</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Logout</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Custom Tags</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Delete Issue</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>No Title</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Feel free to register and contribute to our growing community. We are 5 developers at the moment. If you face any problems do not hesitate to mail to developer@yaiiit.net. We will enable you to post your found issues and help making yaiiit!! more reliable.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>the required input field with default values (not yet)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>New Issue</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Delete Role</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add role</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>a security system based on privileges and roles</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>View Issues</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Uninstall a language</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>User -&gt; Roles</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>an administration area to control security, languages and tags</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>test</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>a generic tag approach to customize the issue&apos;s details</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>OK</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please enter your username and the correct password to login.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please choose a username and type your password twice to verify.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Delete Detail</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to add issues!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>logged in as {1}</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>set a new username to an unused one</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>The user &apos;{1}&apos; already exists or you are not allowed to register at the moment.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Description</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Apply</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>create new roles</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add_comment</source>
        <translation>Add Comment</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Option</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Do nothing</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete_tag</source>
        <translation>Delete Tag</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Actual Privileges</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Username</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Apply filter</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add_tag</source>
        <translation>Add Tag</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Register</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Here you are able to create and edit conditions to filter the issues you see.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Filter the issues</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>rename_user</source>
        <translation>Rename User</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Install Languages</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Fill all input fields.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Add this tag</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Install</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Add Comment</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>view_home</source>
        <translation>View Home</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You can install any new language to the internal translator which is present in the application&apos;s file library.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Add a new role</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Choose a tag and an operation first.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Here you are able to create a new issue. Any detail can be attached to this issue. You have to insert an initial comment so that this issue can be referenced to you.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to delete a role!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Saving...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Do you really want to delete &apos;{1}&apos;?</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You cannot delete that tag while it is used by an issue.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>This is just another issue tracker. You can track all tasks, issues or bugs of your software project. yaiiit!! tries to integrate easily in your project&apos;s workflow with a clear design and simple usage.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>The password was changed successfully. Don&apos;t forget it.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Bad Username</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Attached Users</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Add</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Visit yaiiit!! project page on http://www.yaiiit.net</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Welcome to the Administration Area!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>New Username</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to delete an issue!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Comment:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Enter a new comment here:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Password</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Register a new user</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>a clear and brief description of the tag</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Issues per page: </source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete_role</source>
        <translation>Delete Role</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select another role:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>an user area to setup username and password</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Cancel</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Choose language:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to administrate this application!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please enter a value for the selected tag.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to add an issue!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Created</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>That username already exists. Please choose another one.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>read_issue</source>
        <translation>Read Issue</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Your password is not long enough.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Please type your old password once and your new password twice to verify. Keep in mind that you cannot login with the old credentials anymore.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add_user</source>
        <translation>Add User</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Name</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Translation</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>So enjoy your visit on http://www.yaiiit.net.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Yesterday</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Login</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Owner</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete_issue</source>
        <translation>Delete Issue</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>The passwords you entered do not match.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Yet Another Issue Issue Issue Tracker!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to create a tag!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>In this area you can modify several security settings:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Tag</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Content</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>delete roles</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>New Role</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>By selecting &quot;matches&quot; you can use two wildcards (# = one char and * = arbitrary number of chars)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>detach roles from users</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>a descriptive title</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>New Password</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Last Comment</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>change your password</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized change user settings!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Change your Password</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Edit settings for role &apos;{1}&apos;</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>modify privileges for each role</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Change your Username</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Filter Issues</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Select a tag: </source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Title</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to delete a tag!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>add_issue</source>
        <translation>Add Issue</translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>In here you are able to change your account&apos;s details without any need to get permissions from your webmaster or administrator. This includes the following:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>You are not authorized to view the details of an issue!!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="" line="0"/>
        <source>Current Password</source>
        <translation type="unfinished"></translation>
    </message>
</context>
</TS>
'! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
exampleTs
	^ '<?xml version="1.0" encoding="utf-8"?>
<!!DOCTYPE TS><TS version="1.1" language="de_DE">
<defaultcodec></defaultcodec>
<context>
    <name>SWE-IssueTracker-Test-Translation</name>
    <message>
        <source>test</source>
        <translation>test_translated</translation>
    </message>
</context>
</TS>'! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
findPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 37 73 68 65 84 56 203 99 248 255 255 63 3 37 152 129 234 6 52 247 207 121 180 100 195 158 255 173 19 231 254 7 241 39 47 88 7 102 135 198 166 222 34 202 128 138 230 190 71 235 118 28 6 105 0 27 144 81 84 243 191 111 246 138 255 64 131 159 224 53 32 41 187 228 127 215 244 37 255 107 59 167 254 111 155 52 255 191 137 165 29 216 0 7 55 159 255 89 37 117 255 11 170 90 193 98 86 246 174 203 21 148 84 121 4 133 69 153 192 6 44 88 179 147 163 168 182 221 163 169 111 54 216 169 133 213 109 255 61 252 195 254 187 120 5 252 207 41 107 248 31 155 150 247 63 34 167 233 127 237 196 101 255 75 90 166 128 229 184 185 121 170 216 216 217 25 193 6 76 152 187 154 101 202 194 245 15 23 173 219 245 63 183 188 9 140 171 39 174 250 63 101 249 174 255 125 75 182 255 106 156 190 225 127 109 239 252 255 209 185 205 255 195 147 11 193 46 100 102 97 121 192 192 192 192 12 54 0 232 92 86 144 205 211 22 111 252 159 93 90 255 31 228 231 148 220 242 255 209 201 57 255 213 180 244 192 54 130 92 2 19 143 75 47 0 234 98 248 7 196 172 96 3 210 10 42 89 253 66 99 126 90 216 58 253 247 10 140 248 111 108 97 251 95 91 223 248 191 172 130 242 83 17 49 113 3 73 25 185 173 98 18 82 255 129 254 254 47 42 46 249 159 151 143 255 63 35 35 227 59 160 1 76 240 64 244 9 138 58 12 10 184 93 135 207 252 95 181 229 192 255 238 25 203 64 1 182 2 36 167 170 169 179 162 178 101 194 255 181 59 14 129 229 64 49 194 199 47 120 24 37 22 220 124 130 255 228 87 52 131 13 0 186 232 127 124 70 225 127 61 99 243 171 32 57 160 205 87 65 49 0 50 32 53 191 226 127 122 97 245 127 14 78 174 63 40 6 56 186 251 254 135 25 0 83 164 161 173 255 83 81 69 157 81 66 90 246 29 40 224 96 6 128 194 130 149 149 237 63 138 1 214 142 110 224 120 6 37 32 144 2 80 96 234 26 153 253 7 250 159 95 73 85 227 10 200 0 80 44 193 228 128 97 128 106 0 40 129 24 152 88 130 3 15 232 231 255 64 155 255 75 203 42 252 7 6 162 20 208 191 183 57 185 184 255 3 227 29 20 125 160 24 0 99 218 101 38 82 49 0 158 50 104 141 14 90 60 117 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
headerPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 3 32 0 0 0 80 8 6 0 0 1 150 189 33 162 0 0 10 58 105 67 67 80 80 104 111 116 111 115 104 111 112 32 73 67 67 32 112 114 111 102 105 108 101 0 0 120 156 157 150 119 84 84 215 22 135 207 189 119 122 161 205 48 20 41 67 239 189 13 32 189 55 169 210 68 97 152 25 96 40 3 14 51 52 177 33 162 2 17 69 68 4 21 65 130 34 6 140 134 34 177 34 138 133 128 96 193 30 144 32 160 196 96 20 81 81 121 51 178 86 116 229 229 189 151 151 223 31 103 125 107 159 189 247 61 103 239 125 214 186 0 144 188 253 185 188 116 88 10 128 52 158 128 31 226 229 74 143 140 138 166 99 251 1 12 240 0 3 204 0 96 178 50 51 2 66 61 195 128 72 62 30 110 244 76 145 19 248 34 8 128 55 119 196 43 0 55 141 188 131 232 116 240 255 73 154 149 193 23 136 210 4 137 216 130 205 201 100 137 184 80 196 169 217 130 12 177 125 70 196 212 248 20 49 195 40 49 243 69 7 20 177 188 152 19 23 217 240 179 207 34 59 139 153 157 198 99 139 88 124 230 12 118 26 91 204 61 34 222 154 37 228 136 24 241 23 113 81 22 151 147 45 226 91 34 214 76 21 166 113 69 252 86 28 155 198 97 102 2 128 34 137 237 2 14 43 73 196 166 34 38 241 195 66 220 68 188 20 0 28 41 241 43 142 255 138 5 156 28 129 248 82 110 233 25 185 124 110 98 146 128 174 203 210 163 155 217 218 50 232 222 156 236 84 142 64 96 20 196 100 165 48 249 108 186 91 122 90 6 147 151 11 192 226 157 63 75 70 92 91 186 168 200 214 102 182 214 214 70 230 198 102 95 21 234 191 110 254 77 137 123 187 72 175 130 63 247 12 162 245 125 177 253 149 95 122 61 0 140 89 81 109 118 124 177 197 239 5 160 99 51 0 242 247 191 216 52 15 2 32 41 234 91 251 192 87 247 161 137 231 37 73 32 200 176 51 49 201 206 206 54 230 114 88 198 226 130 254 161 255 233 240 55 244 213 247 140 197 233 254 40 15 221 157 147 192 20 166 10 232 226 186 177 210 83 211 133 124 122 102 6 147 197 161 27 253 121 136 255 113 224 95 159 195 48 132 147 192 225 115 120 162 136 112 209 148 113 121 137 162 118 243 216 92 1 55 157 71 231 242 254 83 19 255 97 216 159 180 56 215 34 81 26 62 1 106 172 49 144 26 160 2 228 215 62 128 162 16 1 18 115 64 180 3 253 209 55 127 124 56 16 191 188 8 213 137 197 185 255 44 232 223 179 194 101 226 37 147 155 248 57 206 45 36 140 206 18 242 179 22 247 196 207 18 160 1 1 72 2 42 80 0 42 64 3 232 2 35 96 14 108 128 61 112 6 30 192 23 4 130 48 16 5 86 1 22 72 2 105 128 15 178 65 62 216 8 138 64 9 216 1 118 131 106 80 11 26 64 19 104 1 39 64 7 56 13 46 128 203 224 58 184 1 110 131 7 96 4 140 131 231 96 6 188 1 243 16 4 97 33 50 68 129 20 32 85 72 11 50 128 204 33 6 228 8 121 64 254 80 8 20 5 197 65 137 16 15 18 66 249 208 38 168 4 42 135 170 161 58 168 9 250 30 58 5 93 128 174 66 131 208 61 104 20 154 130 126 135 222 195 8 76 130 169 176 50 172 13 155 192 12 216 5 246 131 195 224 149 112 34 188 26 206 131 11 225 237 112 21 92 15 31 131 219 225 11 240 117 248 54 60 2 63 135 103 17 128 16 17 26 162 134 24 33 12 196 13 9 68 162 145 4 132 143 172 67 138 145 74 164 30 105 65 186 144 94 228 38 50 130 76 35 239 80 24 20 5 69 71 25 161 236 81 222 168 229 40 22 106 53 106 29 170 20 85 141 58 130 106 71 245 160 110 162 70 81 51 168 79 104 50 90 9 109 128 182 67 251 160 35 209 137 232 108 116 17 186 18 221 136 110 67 95 66 223 70 143 163 223 96 48 24 26 70 7 99 131 241 198 68 97 146 49 107 48 165 152 253 152 86 204 121 204 32 102 12 51 139 197 98 21 176 6 88 7 108 32 150 137 21 96 139 176 123 177 199 176 231 176 67 216 113 236 91 28 17 167 138 51 199 121 226 162 113 60 92 1 174 18 119 20 119 22 55 132 155 192 205 227 165 240 90 120 59 124 32 158 141 207 197 151 225 27 240 93 248 1 252 56 126 158 32 77 208 33 56 16 194 8 201 132 141 132 42 66 11 225 18 225 33 225 21 145 72 84 39 218 18 131 137 92 226 6 98 21 241 56 241 10 113 148 248 142 36 67 210 39 185 145 98 72 66 210 118 210 97 210 121 210 61 210 43 50 153 172 77 118 38 71 147 5 228 237 228 38 242 69 242 99 242 91 9 138 132 177 132 143 4 91 98 189 68 141 68 187 196 144 196 11 73 188 164 150 164 139 228 42 201 60 201 74 201 147 146 3 146 211 82 120 41 109 41 55 41 166 212 58 169 26 169 83 82 195 82 179 210 20 105 51 233 64 233 52 233 82 233 163 210 87 165 39 101 176 50 218 50 30 50 108 153 66 153 67 50 23 101 198 40 8 69 131 226 70 97 81 54 81 26 40 151 40 227 84 12 85 135 234 67 77 166 150 80 191 163 246 83 103 100 101 100 45 101 195 101 115 100 107 100 207 200 142 208 16 154 54 205 135 150 74 43 163 157 160 221 161 189 151 83 150 115 145 227 200 109 147 107 145 27 146 155 147 95 34 239 44 207 145 47 150 111 149 191 45 255 94 129 174 224 161 144 162 176 83 161 67 225 145 34 74 81 95 49 88 49 91 241 128 226 37 197 233 37 212 37 246 75 88 75 138 151 156 88 114 95 9 86 210 87 10 81 90 163 116 72 169 79 105 86 89 69 217 75 57 67 121 175 242 69 229 105 21 154 138 179 74 178 74 133 202 89 149 41 85 138 170 163 42 87 181 66 245 156 234 51 186 44 221 133 158 74 175 162 247 208 103 212 148 212 188 213 132 106 117 106 253 106 243 234 58 234 203 213 11 212 91 213 31 105 16 52 24 26 9 26 21 26 221 26 51 154 170 154 1 154 249 154 205 154 247 181 240 90 12 173 36 173 61 90 189 90 115 218 58 218 17 218 91 180 59 180 39 117 228 117 124 116 242 116 154 117 30 234 146 117 157 116 87 235 214 235 222 210 195 232 49 244 82 244 246 235 221 208 135 245 173 244 147 244 107 244 7 12 96 3 107 3 174 193 126 131 65 67 180 161 173 33 207 176 222 112 216 136 100 228 98 148 101 212 108 52 106 76 51 246 55 46 48 238 48 126 97 162 105 18 109 178 211 164 215 228 147 169 149 105 170 105 131 233 3 51 25 51 95 179 2 179 46 179 223 205 245 205 89 230 53 230 183 44 200 22 158 22 235 45 58 45 94 90 26 88 114 44 15 88 222 181 162 88 5 88 109 177 234 182 250 104 109 99 205 183 110 177 158 178 209 180 137 179 217 103 51 204 160 50 130 24 165 140 43 182 104 91 87 219 245 182 167 109 223 217 89 219 9 236 78 216 253 102 111 100 159 98 127 212 126 114 169 206 82 206 210 134 165 99 14 234 14 76 135 58 135 17 71 186 99 156 227 65 199 17 39 53 39 166 83 189 211 19 103 13 103 182 115 163 243 132 139 158 75 178 203 49 151 23 174 166 174 124 215 54 215 57 55 59 183 181 110 231 221 17 119 47 247 98 247 126 15 25 143 229 30 213 30 143 61 213 61 19 61 155 61 103 188 172 188 214 120 157 247 70 123 251 121 239 244 30 246 81 246 97 249 52 249 204 248 218 248 174 245 237 241 35 249 133 250 85 251 61 241 215 247 231 251 119 5 192 1 190 1 187 2 30 46 211 90 198 91 214 17 8 2 125 2 119 5 62 10 210 9 90 29 244 99 48 38 56 40 184 38 248 105 136 89 72 126 72 111 40 37 52 54 244 104 232 155 48 215 176 178 176 7 203 117 151 11 151 119 135 75 134 199 132 55 133 207 69 184 71 148 71 140 68 154 68 174 141 188 30 165 24 197 141 234 140 198 70 135 71 55 70 207 174 240 88 177 123 197 120 140 85 76 81 204 157 149 58 43 115 86 94 93 165 184 42 117 213 153 88 201 88 102 236 201 56 116 92 68 220 209 184 15 204 64 102 61 115 54 222 39 126 95 252 12 203 141 181 135 245 156 237 204 174 96 79 113 28 56 229 156 137 4 135 132 242 132 201 68 135 196 93 137 83 73 78 73 149 73 211 92 55 110 53 247 101 178 119 114 109 242 92 74 96 202 225 148 133 212 136 212 214 52 92 90 92 218 41 158 12 47 133 215 147 174 146 158 147 62 152 97 144 81 148 49 178 218 110 245 238 213 51 124 63 126 99 38 148 185 50 179 83 64 21 253 76 245 9 117 133 155 133 163 89 142 89 53 89 111 179 195 179 79 230 72 231 240 114 250 114 245 115 183 229 78 228 121 230 125 187 6 181 134 181 166 59 95 45 127 99 254 232 90 151 181 117 235 160 117 241 235 186 215 107 172 47 92 63 190 193 107 195 145 141 132 141 41 27 127 42 48 45 40 47 120 189 41 98 83 87 161 114 225 134 194 177 205 94 155 155 139 36 138 248 69 195 91 236 183 212 110 69 109 229 110 237 223 102 177 109 239 182 79 197 236 226 107 37 166 37 149 37 31 74 89 165 215 190 49 251 166 234 155 133 237 9 219 251 203 172 203 14 236 192 236 224 237 184 179 211 105 231 145 114 233 242 188 242 177 93 1 187 218 43 232 21 197 21 175 119 199 238 190 90 105 89 89 187 135 176 71 184 103 164 202 191 170 115 175 230 222 29 123 63 84 39 85 223 174 113 173 105 221 167 180 111 219 190 185 253 236 253 67 7 156 15 180 212 42 215 150 212 190 63 200 61 120 183 206 171 174 189 94 187 190 242 16 230 80 214 161 167 13 225 13 189 223 50 190 109 106 84 108 44 105 252 120 152 119 120 228 72 200 145 158 38 155 166 166 163 74 71 203 154 225 102 97 243 212 177 152 99 55 190 115 255 174 179 197 168 165 174 149 214 90 114 28 28 23 30 127 246 125 220 247 119 78 248 157 232 62 201 56 217 242 131 214 15 251 218 40 109 197 237 80 123 110 251 76 71 82 199 72 103 84 231 224 41 223 83 221 93 246 93 109 63 26 255 120 248 180 218 233 154 51 178 103 202 206 18 206 22 158 93 56 151 119 110 246 124 198 249 233 11 137 23 198 186 99 187 31 92 140 188 120 171 39 184 167 255 146 223 165 43 151 61 47 95 236 117 233 61 119 197 225 202 233 171 118 87 79 93 99 92 235 184 110 125 189 189 207 170 175 237 39 171 159 218 250 173 251 219 7 108 6 58 111 216 222 232 26 92 58 120 118 200 105 232 194 77 247 155 151 111 249 220 186 126 123 217 237 193 59 203 239 220 29 142 25 30 185 203 190 59 121 47 245 222 203 251 89 247 231 31 108 120 136 126 88 252 72 234 81 229 99 165 199 245 63 235 253 220 58 98 61 114 102 212 125 180 239 73 232 147 7 99 172 177 231 191 100 254 242 97 188 240 41 249 105 229 132 234 68 211 164 249 228 233 41 207 169 27 207 86 60 27 127 158 241 124 126 186 232 87 233 95 247 189 208 125 241 195 111 206 191 245 205 68 206 140 191 228 191 92 248 189 244 149 194 171 195 175 45 95 119 207 6 205 62 126 147 246 102 126 174 248 173 194 219 35 239 24 239 122 223 71 188 159 152 207 254 128 253 80 245 81 239 99 215 39 191 79 15 23 210 22 22 254 5 3 152 243 252 54 28 15 210 0 0 0 4 103 65 77 65 0 0 177 142 124 251 81 147 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 232 0 0 117 48 0 0 234 96 0 0 58 151 0 0 23 111 151 169 153 212 0 0 39 11 73 68 65 84 120 156 98 252 241 227 199 127 6 70 70 6 24 64 176 144 0 80 30 171 56 146 60 78 189 96 105 188 186 9 202 19 116 31 1 64 208 252 65 4 0 2 136 17 28 33 24 162 120 2 152 64 224 195 212 224 147 167 86 4 145 27 204 131 57 130 0 2 136 241 59 82 132 224 139 0 124 242 3 154 123 136 176 131 176 246 193 19 65 0 1 196 248 253 251 119 68 14 65 115 24 201 17 52 154 123 40 6 0 1 132 26 33 40 50 20 70 14 146 60 78 47 142 214 61 24 0 32 128 112 71 8 134 74 234 68 208 96 200 61 96 38 126 149 228 153 79 5 0 16 64 196 71 8 134 78 34 60 71 73 227 96 132 214 61 0 1 68 126 132 160 152 50 154 123 168 21 57 0 1 68 157 8 193 48 149 184 186 133 144 60 185 17 52 148 115 15 64 0 209 38 66 80 108 160 79 238 193 169 151 97 104 229 30 128 0 98 68 238 169 15 246 142 32 65 119 144 107 254 32 2 0 1 132 217 83 31 77 173 3 10 0 2 8 222 83 39 212 218 193 170 102 52 247 80 29 0 4 16 206 158 250 144 44 235 137 176 131 98 243 105 12 0 2 136 118 61 245 209 220 67 22 0 8 32 178 122 234 163 195 40 180 3 0 1 68 94 179 119 180 35 72 179 8 2 8 32 170 247 212 105 50 140 2 85 51 18 234 30 128 0 162 126 199 112 52 247 80 20 65 0 1 52 168 122 234 180 168 123 32 74 168 19 65 244 24 70 1 8 32 188 83 184 24 194 196 217 78 186 30 42 155 65 105 17 66 149 34 8 139 25 244 108 179 13 134 38 252 112 0 0 1 68 184 163 142 161 131 130 204 131 166 127 32 50 15 196 8 10 138 248 33 158 121 70 51 14 105 0 32 128 16 85 58 158 128 35 41 72 41 137 252 33 158 121 168 161 127 180 230 25 92 0 32 128 136 107 243 14 135 204 67 5 51 6 99 230 1 11 83 102 42 9 214 143 188 204 3 16 64 148 119 10 71 51 15 84 251 224 203 60 163 53 15 229 0 32 128 104 59 106 66 104 180 130 2 115 70 51 15 73 6 96 10 81 102 34 137 214 15 221 204 3 16 64 180 31 86 196 105 51 237 106 30 162 205 160 242 136 219 112 25 48 0 11 83 110 50 145 214 15 238 204 3 16 64 3 151 65 240 1 26 214 60 68 235 31 226 153 135 26 250 71 51 15 3 3 64 0 13 206 12 130 15 12 178 204 51 98 155 109 16 67 48 133 40 55 149 72 171 233 99 19 64 0 13 189 12 66 8 80 163 233 54 90 243 80 164 127 56 213 60 0 1 196 248 232 201 19 204 189 183 200 17 132 43 194 9 136 51 162 179 145 228 177 170 197 98 31 156 77 142 30 60 124 116 247 32 187 23 167 91 113 240 81 220 66 72 61 14 247 227 114 43 33 113 114 51 210 96 104 186 12 21 0 16 64 140 143 30 63 198 57 81 200 136 69 28 107 134 161 98 102 65 86 131 51 225 163 233 67 119 57 177 25 134 17 75 2 71 113 3 190 140 73 68 102 65 225 99 113 11 46 247 227 84 75 72 14 79 230 195 7 70 51 12 110 0 16 64 140 15 161 25 4 87 73 7 231 226 18 39 49 195 224 75 64 20 215 46 88 248 67 173 118 193 149 84 71 107 151 129 1 0 1 4 207 32 24 18 40 28 26 212 46 80 57 106 212 46 40 124 50 107 23 92 230 17 149 97 200 73 252 3 148 97 70 107 23 210 0 64 0 49 62 120 244 8 209 7 33 182 115 69 110 134 193 55 234 65 235 190 11 22 62 49 205 47 116 62 81 77 174 65 210 119 193 41 55 90 187 16 13 0 2 8 37 131 96 72 18 147 97 40 169 93 208 212 225 171 93 208 245 80 92 187 224 83 71 164 28 177 205 49 156 253 39 108 122 177 169 25 173 93 6 12 0 4 16 227 253 135 15 255 19 235 81 188 53 0 66 17 118 53 248 18 28 14 189 148 212 46 48 53 196 102 22 170 54 199 208 253 70 235 218 5 159 56 22 247 18 18 31 205 48 8 0 16 64 224 12 130 34 64 130 39 9 102 152 129 174 93 144 212 144 84 187 144 144 89 112 201 17 91 187 16 106 58 146 84 56 224 19 195 225 118 66 226 35 61 179 0 4 16 70 6 65 145 164 102 102 129 40 194 46 143 39 241 193 213 209 160 118 65 183 15 57 179 160 219 129 43 195 16 51 50 134 205 189 200 110 30 50 181 203 8 236 187 0 4 16 222 12 130 161 152 130 166 24 88 28 85 17 118 121 50 106 23 108 226 184 154 112 20 215 46 88 248 116 169 93 168 193 199 37 134 195 237 132 196 71 66 237 2 16 64 36 101 16 12 205 68 120 20 239 8 11 170 66 236 114 196 100 24 58 215 46 132 18 33 165 125 23 172 25 154 210 218 133 20 49 28 238 197 37 134 108 63 138 27 136 0 131 61 179 0 4 16 69 25 4 197 32 122 212 46 104 114 196 100 22 116 189 248 18 35 54 51 72 170 93 168 208 119 193 231 38 82 107 15 116 179 6 162 118 65 113 3 17 96 176 101 24 128 0 162 90 6 65 49 148 194 190 11 88 28 85 17 118 249 65 82 187 192 249 20 212 46 184 228 136 106 142 17 145 25 176 153 133 83 63 30 247 99 115 55 65 57 50 51 11 33 187 232 1 0 2 136 38 25 4 171 69 20 52 199 176 69 28 86 121 60 137 15 174 14 87 100 17 91 187 96 225 83 163 239 130 206 199 38 71 84 102 33 146 143 238 31 90 212 46 56 229 134 80 237 2 16 64 116 203 32 40 150 82 179 57 6 81 136 41 79 73 237 2 149 163 70 237 130 115 146 144 94 181 11 17 246 18 234 235 96 216 133 203 28 28 110 39 36 142 211 14 2 128 30 25 6 32 128 6 36 131 96 56 130 134 181 11 92 13 190 4 135 67 47 214 68 67 98 115 134 94 181 11 54 127 96 205 224 180 168 93 176 184 137 144 56 78 185 65 214 28 3 8 160 65 145 65 144 193 160 175 93 104 209 119 65 215 135 79 29 145 114 68 213 46 148 246 93 176 241 113 137 225 112 59 94 241 65 144 89 0 2 104 208 101 16 116 64 73 134 33 167 118 161 199 68 37 174 140 64 206 80 50 185 19 149 184 106 71 170 215 46 4 154 79 228 100 24 114 146 62 185 25 6 32 128 6 125 6 65 6 67 177 118 65 151 167 70 237 130 175 182 161 229 68 37 70 184 18 145 161 104 86 187 64 205 2 83 56 204 195 5 72 201 44 0 1 52 164 50 8 50 160 245 80 50 214 132 76 199 218 101 68 244 93 240 36 112 146 251 46 4 204 35 217 44 40 0 8 192 190 125 236 70 8 3 97 0 30 209 123 103 223 255 249 66 239 29 50 70 129 152 200 171 37 81 142 59 151 79 254 127 251 56 71 239 95 110 31 97 8 31 113 12 143 32 184 74 242 40 186 24 161 33 17 251 16 239 209 146 119 81 146 64 232 251 47 141 209 128 152 166 16 120 222 85 170 79 80 159 246 199 125 114 246 137 89 6 190 235 158 166 168 71 155 231 224 57 206 105 134 186 180 69 1 174 109 159 230 168 131 121 142 61 109 65 242 163 167 44 202 18 108 203 250 182 170 192 54 77 40 81 139 101 93 131 101 24 80 161 38 195 186 105 158 230 134 174 63 181 105 91 208 53 237 169 109 215 129 166 170 167 29 170 210 246 61 168 138 178 171 49 242 30 85 88 14 3 40 178 12 3 42 179 28 71 144 37 9 70 84 98 57 77 32 137 226 233 132 138 135 216 31 231 25 21 88 206 51 136 130 176 43 220 113 89 64 224 121 88 80 254 134 235 186 2 199 113 191 118 219 182 125 9 95 250 181 216 180 220 157 119 127 244 61 239 249 143 249 20 128 125 59 86 2 0 132 0 0 218 31 52 246 255 127 90 12 157 162 144 235 90 178 24 48 242 38 248 113 91 4 40 88 14 192 226 1 102 11 139 2 12 157 235 176 56 129 97 176 24 129 1 88 50 64 18 5 70 129 229 22 48 42 44 4 18 177 47 8 204 18 150 9 24 6 75 171 107 192 96 191 21 24 39 44 167 192 184 15 127 26 129 193 229 253 192 252 120 28 85 0 193 43 144 151 192 10 0 27 77 82 197 130 165 130 129 87 44 67 181 231 66 168 130 161 118 207 5 74 147 219 115 1 209 32 117 248 42 30 154 244 92 200 173 96 6 73 207 133 86 21 12 205 43 22 44 21 204 104 207 101 20 208 11 0 4 16 248 196 6 113 49 49 134 151 175 94 49 128 104 108 21 5 89 21 12 182 138 5 95 5 67 97 207 5 94 193 80 171 231 130 86 177 80 165 231 130 163 98 161 74 207 101 144 12 141 225 171 88 168 58 52 70 168 98 25 237 185 140 14 141 141 2 154 3 128 0 194 168 64 112 209 164 86 44 100 85 48 195 105 206 133 196 158 11 190 10 102 88 247 92 112 85 44 3 209 115 33 178 98 65 174 96 136 173 88 6 93 5 195 128 89 177 140 14 141 141 2 82 1 64 0 65 42 16 92 21 0 17 21 11 182 138 134 162 138 101 144 14 141 97 84 44 3 60 231 66 237 73 125 114 231 92 168 85 193 208 173 231 66 98 5 67 78 207 133 164 73 125 58 85 44 131 125 82 127 180 130 25 154 0 32 128 192 203 20 37 128 5 255 11 96 5 128 76 19 170 0 72 173 96 104 218 115 33 113 104 12 89 31 181 38 245 7 99 207 5 219 208 24 174 158 11 37 147 250 131 161 231 66 234 156 11 161 161 49 156 21 12 161 57 23 66 21 204 32 155 115 25 173 88 70 1 37 0 32 128 112 86 32 132 104 106 85 44 131 169 231 130 173 98 34 88 177 80 171 231 66 236 156 11 148 30 240 213 98 67 124 206 133 214 67 99 244 234 185 12 186 73 125 134 209 10 102 36 1 128 0 34 187 2 33 171 98 25 200 158 11 169 21 204 32 239 185 144 60 52 70 97 207 133 22 203 145 201 233 185 12 216 114 100 34 123 46 196 78 234 99 204 189 32 87 56 35 100 82 127 180 98 25 250 0 32 128 168 94 129 12 68 197 66 81 5 67 234 144 24 133 21 12 177 61 23 66 147 250 36 207 185 80 88 193 208 187 231 130 171 130 25 232 141 148 131 113 57 50 73 21 203 16 25 26 3 23 78 163 21 203 160 7 0 1 68 183 10 100 200 246 92 168 180 90 140 230 61 23 34 135 198 232 61 231 66 110 207 133 222 67 99 3 182 145 146 208 156 11 165 171 198 104 60 231 50 186 145 114 100 3 128 0 26 52 21 200 64 84 48 244 24 18 163 214 114 100 106 245 92 6 253 114 228 65 62 231 50 98 143 128 161 211 156 203 96 159 212 31 5 168 0 32 128 134 76 5 50 44 122 46 131 113 206 101 144 246 92 200 93 45 134 175 130 25 142 115 46 180 220 239 50 220 230 92 70 123 46 212 7 0 1 52 228 43 16 122 85 48 244 222 72 73 173 57 23 74 54 82 98 83 63 92 123 46 3 54 231 130 171 130 25 196 115 46 131 113 57 50 217 5 63 195 232 164 62 37 0 32 128 134 125 5 50 44 122 46 248 134 196 168 53 52 70 173 158 11 90 5 67 168 98 33 247 140 49 122 206 185 208 163 231 66 183 161 177 209 158 203 232 208 24 21 1 64 0 141 216 10 132 162 10 134 132 138 101 192 43 152 1 232 185 12 230 57 23 140 10 134 70 115 46 3 125 4 12 205 230 92 40 221 239 66 227 213 98 3 113 4 204 72 174 88 0 2 104 180 2 25 224 10 102 192 246 185 96 169 96 72 169 88 168 117 120 37 185 247 186 12 133 141 148 180 232 185 12 149 73 253 209 195 43 71 198 208 24 64 0 141 86 32 131 116 72 108 196 246 92 134 217 164 254 160 59 2 102 176 30 187 15 173 96 6 234 194 176 209 141 148 228 1 128 0 26 173 64 70 72 5 51 88 14 175 164 228 194 176 129 188 215 101 168 31 1 67 74 197 50 58 231 50 252 123 46 212 2 0 1 52 90 129 12 18 122 72 77 234 15 212 114 228 65 80 177 80 107 181 216 136 57 2 6 74 15 116 207 101 56 85 48 131 169 98 1 8 32 70 96 4 255 31 44 123 20 48 10 135 17 116 46 20 57 173 206 1 27 206 24 189 236 105 116 143 194 40 24 5 64 0 16 64 140 143 30 63 254 79 233 33 131 3 177 163 154 220 83 108 49 42 21 108 149 9 29 238 223 24 232 29 213 35 118 8 3 74 143 244 251 55 70 193 40 160 6 0 8 32 72 15 132 70 19 174 163 183 5 14 175 219 2 7 205 132 43 149 123 38 131 253 120 14 148 12 59 90 153 140 130 65 4 0 2 136 241 33 176 7 50 100 142 65 31 130 149 201 176 57 6 157 198 23 56 81 90 153 140 164 225 174 225 176 122 103 20 12 15 0 16 64 224 30 200 144 156 80 37 97 175 193 112 222 37 77 143 21 59 131 117 151 244 232 220 201 232 220 201 40 24 88 0 16 64 240 30 200 96 63 141 150 216 141 107 132 246 21 144 187 113 141 220 202 100 196 156 233 52 152 78 163 165 225 220 201 96 31 238 26 237 161 140 2 122 2 128 0 98 252 142 54 7 50 24 54 171 81 101 238 100 16 220 87 78 106 101 66 175 99 54 232 125 64 32 169 151 50 13 216 112 215 32 171 76 40 237 153 140 246 80 70 1 173 1 64 0 49 62 120 244 232 255 176 223 249 76 203 59 202 233 124 102 211 136 217 249 76 167 213 93 163 251 7 70 43 147 81 64 62 0 8 32 112 15 132 230 151 47 81 161 50 161 197 112 23 209 115 39 148 86 38 131 112 183 51 61 206 105 162 91 101 66 135 115 154 200 185 124 105 48 87 38 163 61 147 81 64 13 0 16 64 24 61 144 193 114 54 19 177 135 254 141 222 135 65 219 185 147 1 59 85 118 32 238 195 32 163 50 25 178 61 20 6 250 86 38 163 149 202 240 4 0 1 196 8 204 176 255 7 229 65 127 72 149 202 96 184 92 105 176 28 244 55 152 143 203 160 245 17 229 163 149 201 232 112 215 40 24 92 0 32 128 32 61 144 209 179 151 168 55 17 79 233 177 228 68 238 140 31 110 75 133 135 205 112 215 104 101 50 90 153 140 32 0 16 64 136 30 8 161 66 127 32 42 19 42 12 119 13 137 99 86 6 203 220 9 141 206 236 26 46 251 78 134 203 206 248 193 62 119 50 90 169 12 29 0 16 64 24 167 241 210 242 216 113 98 42 147 33 181 84 120 16 204 157 12 244 190 147 17 119 35 31 145 195 93 216 42 149 97 177 239 100 180 135 50 10 144 0 64 0 129 123 32 195 234 168 241 193 184 84 120 16 76 196 143 94 146 52 112 71 141 143 238 140 31 173 76 134 43 0 8 32 146 239 3 25 180 149 9 169 195 93 212 170 76 8 244 80 232 54 17 63 8 135 187 134 84 101 66 104 19 227 96 219 25 63 68 42 147 209 83 133 135 55 0 8 32 162 122 32 20 85 38 36 84 42 131 105 238 132 172 251 193 233 61 220 53 196 142 89 25 44 195 93 112 117 195 229 152 149 33 50 119 50 58 220 53 252 0 64 0 81 253 70 194 161 122 101 235 232 169 194 35 228 136 122 92 61 148 129 216 119 50 132 143 89 161 198 153 93 163 195 93 67 31 0 4 16 69 61 144 193 122 255 247 144 90 42 76 133 125 39 180 92 42 76 202 17 245 244 174 76 6 237 82 97 74 79 21 38 178 50 25 78 195 93 163 61 147 161 9 0 2 136 238 119 162 211 98 238 132 30 61 20 106 13 119 209 250 136 122 146 135 187 232 189 239 100 128 142 89 25 232 165 194 196 94 239 59 58 119 50 58 119 50 148 0 64 0 209 164 7 66 239 158 202 96 170 76 232 181 239 132 38 149 201 32 61 102 101 192 78 21 30 130 71 212 147 212 67 25 189 145 113 180 82 161 16 0 4 16 221 123 32 244 156 136 31 157 59 25 248 185 19 106 31 81 63 88 230 78 232 118 204 10 145 195 93 20 31 0 57 88 123 40 12 163 115 39 131 25 0 4 208 128 246 64 134 220 82 225 193 52 119 66 133 125 39 84 217 25 63 192 215 251 14 169 99 86 40 189 239 100 244 70 70 186 206 157 140 2 194 0 32 128 6 188 7 50 100 151 10 15 84 101 66 239 225 174 193 178 51 126 8 156 217 53 212 134 187 134 227 82 97 112 161 54 90 169 208 13 0 4 208 160 236 129 208 171 50 161 218 125 39 116 158 59 25 61 102 101 96 134 187 200 157 136 31 208 125 39 131 104 184 139 230 149 9 157 43 149 81 192 192 0 16 64 131 190 7 50 172 134 187 70 231 78 70 252 220 9 221 142 89 161 116 169 240 80 31 238 98 24 157 59 161 7 0 8 160 33 213 3 25 178 149 9 169 61 148 129 154 59 25 4 251 78 232 117 68 253 64 239 59 25 9 75 133 7 93 15 133 97 180 50 161 54 0 8 160 33 223 3 25 174 149 9 188 82 193 82 153 12 196 17 245 228 30 179 66 108 101 50 186 84 120 112 158 42 76 173 235 125 7 123 101 50 58 33 79 30 0 8 160 97 213 3 25 136 185 147 209 165 194 35 103 238 100 160 247 157 12 244 49 43 56 43 19 152 252 80 93 42 60 186 186 139 108 0 16 64 195 182 7 50 88 123 40 100 85 38 67 248 152 21 170 204 157 144 112 204 202 96 57 162 126 216 46 21 38 178 50 25 178 115 39 116 174 76 134 122 165 2 16 64 35 162 7 50 88 43 147 65 187 84 152 90 115 39 104 149 10 161 202 132 220 30 202 232 245 190 244 153 59 161 215 190 147 193 62 220 5 46 56 71 43 21 48 0 8 160 17 219 3 25 136 202 100 56 174 238 34 84 153 12 247 225 174 1 59 162 30 215 206 248 129 24 238 26 173 76 70 108 37 2 16 64 163 61 16 74 43 19 50 43 149 193 180 239 132 150 61 20 122 86 38 131 117 34 126 68 85 38 131 104 184 107 32 142 168 31 105 115 39 0 1 52 218 3 25 68 61 148 97 51 220 69 133 185 19 170 172 238 34 225 122 223 17 127 35 35 173 143 168 39 212 67 25 134 71 212 143 132 30 10 64 0 141 246 64 134 99 101 66 106 15 101 0 151 10 83 163 50 25 10 71 212 15 202 165 194 116 218 25 79 206 220 9 57 75 133 71 122 101 50 16 149 10 64 0 141 246 64 232 89 153 140 128 225 174 1 219 119 50 76 134 187 104 94 153 12 212 169 194 244 58 102 133 204 185 19 82 43 21 74 43 147 225 114 204 10 64 0 141 246 64 6 186 82 25 136 30 202 80 25 238 26 4 149 9 41 61 148 145 118 204 10 173 151 10 15 212 245 190 3 113 204 202 80 93 42 12 16 64 163 61 144 209 202 100 240 84 38 116 56 162 126 192 135 187 104 180 84 120 192 42 19 42 239 59 193 88 221 53 220 207 236 26 226 115 39 0 1 52 218 3 25 6 149 201 96 26 238 26 136 35 234 201 61 102 101 184 86 38 131 246 204 174 97 178 84 120 32 134 187 6 235 234 46 128 0 26 237 129 12 18 122 160 230 78 134 218 190 147 129 30 238 26 232 125 39 131 101 184 107 176 157 217 53 84 86 119 13 246 158 9 169 149 10 64 0 118 205 96 7 129 16 6 162 169 198 255 255 96 227 98 167 91 54 128 45 133 96 244 66 79 190 148 25 70 88 216 203 18 31 112 100 147 13 66 97 147 102 25 19 98 195 133 121 83 165 111 49 143 243 252 158 202 120 8 48 174 226 162 63 154 207 211 63 52 143 48 47 86 197 58 158 122 250 30 119 254 31 152 102 253 138 245 155 205 131 3 129 194 161 232 177 235 151 251 129 254 98 62 72 194 124 152 240 60 180 122 48 13 228 1 83 233 55 58 191 193 200 129 60 109 190 138 29 253 203 27 111 49 207 227 233 239 58 222 242 35 163 255 193 129 95 230 163 237 243 133 37 204 151 150 197 135 242 205 211 7 126 158 190 203 188 78 97 30 207 223 233 135 122 143 83 58 89 47 203 164 76 45 235 248 164 122 139 73 253 200 210 79 178 151 207 226 213 249 172 23 197 106 254 217 249 194 130 70 189 46 62 13 135 184 202 195 191 115 130 236 248 255 15 161 191 83 191 254 250 110 215 174 93 187 86 235 45 0 59 118 180 2 32 8 67 1 180 253 255 111 134 127 33 213 192 64 101 219 93 106 16 177 189 93 202 233 147 237 68 123 74 7 95 93 42 28 38 97 194 66 53 251 131 253 180 65 95 26 156 201 113 62 8 147 2 37 107 208 159 129 196 10 152 16 56 159 149 121 80 230 143 49 130 10 130 214 35 88 8 16 168 215 215 25 193 34 131 193 223 13 47 1 78 77 94 1 19 161 159 5 9 18 214 207 194 36 75 208 49 32 192 153 223 71 176 128 48 225 124 237 227 130 73 201 132 250 73 185 31 220 171 231 83 253 60 131 53 255 241 81 96 50 212 207 147 183 173 233 183 188 127 151 181 122 11 42 195 229 128 9 105 251 255 24 38 119 5 80 162 162 162 190 86 167 0 236 153 65 14 192 32 8 4 245 212 255 127 179 126 162 197 104 195 1 1 41 182 73 11 39 55 38 102 111 58 78 222 75 97 13 200 182 0 76 122 118 7 147 148 72 163 129 179 201 160 192 143 126 235 163 61 127 6 12 170 65 114 52 38 112 169 170 64 130 201 143 128 9 206 55 193 100 104 60 12 6 133 2 1 201 120 168 193 134 233 7 253 37 35 34 130 73 235 167 1 147 158 51 181 63 50 40 70 48 185 192 198 106 80 156 192 100 218 192 44 52 38 248 60 23 131 114 174 57 99 162 234 255 67 48 193 89 2 191 47 61 227 3 74 98 98 98 222 156 67 0 246 205 32 5 96 16 6 130 253 255 167 45 138 135 32 100 147 232 70 16 178 167 22 139 245 152 97 199 209 128 244 135 40 72 168 96 98 168 86 148 253 141 255 169 131 251 13 181 11 52 40 91 106 215 28 108 195 231 49 192 130 13 38 16 84 22 48 25 235 36 48 241 236 199 2 147 30 216 128 40 96 226 82 193 130 13 204 22 152 124 162 1 113 126 31 82 187 64 227 145 166 118 121 65 231 162 218 229 2 9 4 42 0 36 178 213 174 215 192 228 232 34 139 212 187 136 96 210 150 247 23 147 113 227 180 82 169 84 100 126 1 216 183 218 21 128 65 16 104 239 255 164 123 138 194 97 67 232 203 92 70 27 10 17 7 117 248 211 195 187 112 209 6 100 118 16 87 101 66 118 11 19 142 71 131 187 112 48 127 37 76 56 54 206 156 168 248 114 63 29 161 118 98 230 132 99 141 208 41 54 30 10 107 23 14 214 183 181 109 210 218 133 239 35 236 201 156 112 124 74 230 228 177 94 25 89 187 68 124 53 126 129 48 137 82 62 88 180 65 49 16 38 136 241 88 11 147 207 100 78 202 6 129 8 155 184 106 237 202 223 233 254 211 6 197 69 137 151 151 215 138 74 2 176 111 6 55 0 128 32 12 148 253 199 52 206 161 15 191 18 11 150 144 24 58 0 188 47 189 74 31 99 55 32 36 117 9 109 16 34 54 39 208 253 4 48 153 218 253 36 181 203 115 207 59 134 103 171 93 16 24 28 26 135 44 48 49 169 89 1 13 10 13 76 140 106 151 122 143 172 118 93 27 14 38 152 180 196 205 201 69 237 138 216 156 32 96 194 26 223 107 112 18 1 38 218 63 83 94 212 46 249 119 115 82 112 82 169 84 208 44 1 196 248 242 213 43 212 61 32 56 26 234 180 238 152 208 99 207 9 94 243 7 96 207 9 222 165 98 67 164 99 66 238 140 9 41 51 40 68 153 79 76 199 130 132 142 9 49 123 88 232 182 180 11 135 249 148 46 237 162 218 158 19 6 204 142 4 213 246 156 80 176 180 139 154 123 78 136 90 218 5 154 161 160 209 158 19 188 13 125 70 220 155 225 71 247 156 12 161 142 9 50 31 102 28 148 30 234 29 149 209 37 93 163 96 20 140 2 116 0 16 128 157 43 71 1 0 132 97 254 255 213 226 100 133 86 123 24 21 108 182 46 162 139 52 228 232 10 136 102 145 23 172 85 237 83 241 182 82 133 21 142 211 10 13 67 84 88 34 129 8 195 115 231 11 153 147 144 21 107 163 181 235 74 230 164 24 137 9 157 21 214 174 215 137 201 240 94 112 230 228 118 157 176 212 218 5 173 19 166 247 213 102 68 118 102 78 180 86 172 5 49 161 179 57 195 178 152 79 101 78 102 75 45 66 65 113 195 72 76 126 179 118 37 49 73 36 254 67 21 64 152 51 32 228 52 220 73 236 152 32 243 169 57 131 194 200 128 191 163 64 137 249 84 93 218 133 103 68 159 30 155 225 97 124 156 51 48 120 248 36 207 16 225 179 159 136 244 65 72 63 217 29 19 180 165 93 164 28 39 12 158 145 33 99 51 60 140 79 215 165 93 195 228 56 97 172 29 11 70 34 238 21 161 231 158 19 232 12 8 73 75 177 64 250 129 141 76 98 78 237 34 56 3 195 128 187 225 63 88 150 118 209 243 158 19 100 62 12 12 135 61 39 200 252 225 222 49 1 129 209 206 201 40 24 5 195 19 0 4 16 227 227 39 79 80 246 128 208 106 243 248 64 46 237 162 101 199 132 210 227 132 49 102 80 40 220 115 66 112 70 134 136 165 93 228 116 76 240 157 218 69 174 121 180 92 218 133 215 253 255 9 223 147 66 171 227 132 233 118 207 9 13 79 237 194 181 180 139 164 142 9 137 75 187 72 217 115 130 109 198 132 214 123 78 168 125 207 201 176 61 78 152 129 1 195 60 170 154 63 18 247 156 48 224 238 168 192 196 134 122 19 127 180 147 50 10 70 193 208 3 0 1 196 248 226 229 75 248 12 8 184 161 136 171 161 74 70 195 154 104 62 150 25 20 188 13 83 74 237 67 226 15 154 61 39 196 30 39 76 237 165 93 32 243 128 149 20 169 230 83 163 99 66 174 121 3 182 231 132 204 25 19 48 31 169 161 140 109 6 133 236 142 197 16 218 115 130 205 126 178 151 118 209 160 99 2 231 227 153 241 160 231 77 241 120 103 60 64 13 75 32 166 213 210 46 188 13 105 98 150 118 209 104 233 213 112 61 78 152 34 0 50 131 6 29 147 161 14 70 59 37 163 96 20 12 110 0 16 64 140 143 96 51 32 132 70 212 7 162 99 130 204 39 162 97 252 159 129 244 134 56 89 75 175 168 49 35 196 138 231 184 98 18 236 35 229 212 46 98 58 110 148 46 237 250 67 78 199 128 20 255 145 184 84 108 56 236 57 33 198 60 106 117 76 136 238 104 80 186 231 4 199 12 12 49 29 137 255 12 120 58 22 67 101 207 9 5 199 9 19 189 20 139 145 240 113 194 68 111 174 39 181 161 63 194 238 57 129 169 25 150 29 19 164 229 101 195 97 182 4 4 70 59 39 163 96 20 12 60 0 8 32 148 25 16 16 160 180 161 78 82 71 101 8 47 237 26 140 155 225 177 233 199 24 193 167 96 70 137 224 210 46 60 29 19 74 150 118 129 59 106 196 186 135 70 29 19 106 239 57 161 231 61 39 32 64 141 165 93 228 116 76 136 186 89 158 72 243 176 45 237 26 76 199 9 227 90 218 197 72 138 126 82 102 80 24 201 216 115 50 200 143 19 198 181 180 139 214 199 9 211 171 227 48 232 102 80 136 88 218 197 136 203 126 198 225 123 156 48 12 140 118 84 70 193 40 160 29 0 8 32 162 102 64 200 230 51 98 57 78 22 137 15 110 248 210 106 6 101 24 239 57 1 1 114 143 19 166 235 169 93 216 226 31 95 71 131 128 60 134 122 28 29 19 156 238 193 231 191 65 210 49 33 106 105 23 3 102 71 133 42 29 19 6 252 29 9 66 124 130 29 13 28 51 30 160 198 12 189 78 237 162 198 158 19 162 150 134 145 187 180 139 216 11 15 137 93 218 69 202 140 7 158 25 19 154 119 76 136 237 168 208 98 79 8 158 25 25 90 238 57 65 6 67 177 99 130 204 31 41 75 187 96 179 92 163 29 147 81 48 10 40 7 0 1 196 248 28 58 3 130 60 130 77 236 136 56 37 124 80 195 11 100 49 27 41 51 38 12 212 111 200 19 58 181 139 218 254 199 219 113 160 231 210 46 98 26 238 4 102 52 72 113 31 37 29 19 16 24 12 75 187 176 165 15 154 238 57 65 230 147 185 180 139 220 142 9 53 143 19 38 167 99 50 232 142 19 38 225 158 19 16 32 105 105 23 49 29 19 98 151 98 209 106 207 201 64 30 39 204 64 100 199 4 180 39 134 154 199 9 67 71 248 233 53 131 130 107 99 60 181 204 39 198 62 130 128 90 29 19 44 51 40 67 189 73 63 218 57 25 5 163 128 52 0 16 64 140 143 30 63 6 151 12 132 150 78 193 103 44 104 220 49 193 216 243 49 192 155 225 241 205 160 80 51 60 40 153 209 160 170 249 196 46 237 26 224 123 78 144 195 159 228 165 88 4 58 122 164 154 71 171 142 9 50 127 176 237 57 193 232 88 208 121 207 9 89 75 187 112 204 192 128 0 197 75 187 136 220 115 130 49 131 66 230 158 19 146 143 19 38 165 99 66 140 121 20 236 57 33 218 60 134 193 187 180 107 160 247 156 208 178 99 66 140 253 88 28 196 0 53 128 104 62 214 165 93 48 227 160 244 112 152 65 65 238 152 140 118 80 70 193 40 64 0 128 0 66 157 1 97 32 174 35 130 147 79 165 17 124 130 124 60 75 187 70 143 19 166 145 249 164 238 57 161 240 56 97 162 58 38 12 248 59 66 36 119 76 160 254 3 85 18 232 167 118 193 103 236 200 232 104 16 53 195 129 163 99 67 183 142 9 142 83 187 6 250 158 19 130 29 13 74 79 249 26 36 123 78 72 189 231 132 106 29 147 1 62 78 120 164 238 57 161 101 71 133 22 155 225 7 251 210 174 161 60 131 50 218 33 25 5 35 21 0 4 16 227 67 232 12 8 206 134 53 149 58 38 244 90 218 5 31 193 166 247 12 10 158 134 59 173 151 118 193 26 218 88 151 94 13 112 199 4 235 140 194 32 88 218 69 246 82 172 225 112 156 48 22 255 144 51 131 2 54 127 128 150 118 13 228 61 39 200 124 122 236 57 193 107 30 174 25 6 106 207 112 144 185 231 132 232 165 88 131 117 207 9 41 199 19 147 192 199 55 131 66 241 210 49 6 194 141 218 65 183 180 11 98 8 77 58 38 200 252 161 6 70 103 77 70 193 112 6 0 1 216 59 99 28 128 65 24 6 38 255 127 50 75 213 1 68 7 84 59 24 16 82 188 101 96 96 203 201 54 52 7 132 142 230 188 139 181 77 130 137 105 23 229 191 185 45 194 17 7 101 210 177 65 163 93 43 238 63 236 132 8 28 161 107 163 93 155 193 132 141 118 65 229 117 224 60 229 160 184 199 59 35 130 104 151 178 115 82 118 69 187 86 117 78 170 3 2 128 12 226 120 160 209 44 250 167 121 230 57 97 39 59 39 3 48 233 103 10 116 12 4 21 7 254 57 81 129 207 161 104 215 137 206 73 63 135 149 96 242 81 130 73 234 102 61 2 136 224 12 8 169 124 170 204 152 48 80 169 225 79 6 31 223 82 31 70 108 254 167 83 199 132 22 51 72 212 216 172 78 177 249 164 44 237 34 97 198 128 228 142 0 45 143 19 198 211 209 163 247 102 248 193 184 180 139 145 144 126 134 129 223 115 2 114 47 168 160 164 116 105 23 161 142 4 81 29 19 6 218 238 57 65 214 79 209 210 46 50 59 38 56 205 195 102 254 96 188 231 132 129 250 29 19 98 151 118 129 212 255 167 208 190 33 211 49 33 97 198 4 111 71 133 113 248 30 39 60 218 65 25 5 131 25 0 4 16 69 51 32 148 240 135 203 158 19 154 119 156 72 220 115 66 211 205 241 84 14 111 74 150 118 145 50 195 65 108 120 16 236 152 224 48 31 153 79 106 199 130 164 83 201 136 48 111 64 151 118 49 82 126 83 252 160 63 78 152 210 142 13 25 51 40 216 220 51 96 199 9 19 187 180 11 173 99 65 215 61 39 120 58 38 100 47 237 34 101 41 22 129 25 20 144 14 154 220 67 130 167 35 53 186 231 4 119 199 132 145 24 247 192 140 163 204 53 131 6 140 118 76 70 193 96 0 0 1 68 245 25 144 193 210 49 25 232 227 132 193 155 153 25 176 116 76 104 60 131 66 143 142 9 172 161 62 24 247 156 80 186 244 138 146 25 50 56 31 199 12 5 35 62 247 17 152 113 160 214 12 10 173 150 118 129 228 65 149 54 173 150 118 13 229 227 132 41 237 152 96 219 12 63 232 150 118 145 210 49 65 51 15 235 12 5 153 123 78 40 89 218 69 141 142 9 222 61 28 196 44 237 162 197 158 16 58 118 76 232 61 131 66 54 160 100 198 4 153 15 51 142 50 215 12 74 48 218 73 25 5 180 6 0 1 52 96 51 32 3 217 81 161 71 199 4 125 6 99 160 55 195 83 99 233 18 49 252 65 115 207 9 150 240 196 59 195 65 173 142 9 25 75 187 200 234 152 252 30 92 75 187 6 242 56 97 108 155 225 233 125 156 48 209 75 179 200 217 76 143 206 199 210 209 33 102 243 250 127 52 62 134 122 34 59 38 32 64 246 158 17 124 75 187 200 89 138 69 194 102 123 130 29 19 6 194 51 40 195 109 105 215 112 62 78 120 64 55 195 15 227 142 202 104 199 100 20 80 11 0 4 208 160 155 1 25 168 142 9 45 26 202 196 240 113 141 168 211 243 2 70 148 240 163 193 140 9 58 31 99 134 131 138 225 141 177 180 137 210 25 148 1 62 78 152 172 11 24 241 248 143 212 205 240 232 230 13 244 158 19 130 75 187 168 176 25 30 198 167 70 199 132 232 142 6 141 142 19 30 10 247 156 32 235 39 245 212 46 148 142 0 35 145 123 78 24 168 191 180 107 184 221 115 66 207 61 39 200 124 100 48 232 150 118 141 118 76 8 2 88 156 141 118 82 70 1 49 0 32 128 134 236 12 200 176 62 181 139 17 255 158 19 70 98 252 75 167 25 20 186 237 9 161 181 249 68 216 135 209 112 39 98 198 128 106 75 187 240 52 244 135 234 210 46 98 59 38 131 109 105 23 222 165 88 116 58 78 24 60 163 1 12 23 90 220 20 143 205 127 148 222 115 66 243 227 132 137 61 101 139 204 165 93 228 222 115 50 28 59 38 112 62 129 165 93 20 47 29 99 32 220 81 24 20 29 19 84 7 49 64 13 36 138 79 108 199 228 63 18 123 40 130 209 14 201 40 192 6 0 2 104 216 205 128 12 72 199 132 142 51 38 240 134 240 32 233 152 208 243 56 97 20 243 169 220 49 193 185 167 133 196 240 163 233 210 46 108 29 83 106 119 76 126 15 157 165 93 68 109 182 39 225 148 45 48 159 136 165 93 132 204 27 232 61 39 200 238 29 42 247 156 144 181 231 4 216 104 163 218 113 194 196 118 76 136 237 232 16 232 152 80 115 207 9 88 158 220 142 10 13 103 52 136 157 65 33 215 124 124 96 36 117 76 144 249 67 17 140 118 78 70 46 0 8 160 17 53 3 66 9 159 226 205 241 116 232 152 16 227 94 188 254 37 183 35 133 163 225 78 76 195 147 166 123 66 168 212 49 36 119 207 9 178 121 24 29 9 2 29 7 82 58 78 40 250 137 216 115 2 227 147 188 20 139 138 29 19 80 67 23 84 185 146 187 180 107 32 247 156 32 243 73 61 78 24 163 99 48 186 180 11 191 60 169 29 19 82 246 156 224 49 143 110 199 9 163 205 184 80 180 231 132 129 250 51 40 3 121 156 48 185 230 99 227 195 0 45 247 156 96 179 143 40 64 194 210 46 16 159 17 155 253 48 163 160 244 112 232 152 128 192 104 231 100 120 3 128 0 236 157 225 10 192 32 8 132 125 255 167 142 69 3 107 238 136 117 42 139 252 39 136 212 63 63 238 172 163 128 36 130 137 169 88 120 130 10 176 118 193 65 90 13 138 91 88 187 136 138 21 125 231 228 182 22 45 128 137 217 223 19 76 192 253 50 118 78 116 30 242 207 137 60 65 133 98 237 2 10 135 167 181 235 181 30 229 108 48 153 84 80 102 94 217 170 249 56 200 127 220 57 9 125 78 88 28 20 20 162 181 75 215 211 192 65 164 235 23 101 237 210 251 38 75 231 79 84 80 76 48 105 86 185 221 192 228 138 3 39 255 143 34 128 70 103 64 6 97 199 4 206 31 204 75 187 6 168 99 66 237 205 241 56 79 237 162 181 249 228 116 76 24 104 176 180 139 192 140 29 53 150 118 129 103 144 136 117 15 145 230 209 189 99 130 165 163 66 112 198 3 203 82 172 129 62 78 152 146 165 89 88 237 31 196 75 187 24 137 209 79 139 205 240 12 116 216 115 66 237 227 132 201 109 232 15 162 142 9 53 237 195 5 6 93 199 4 98 200 232 158 19 52 128 236 207 209 142 202 224 5 0 1 52 58 3 50 0 29 147 255 104 124 16 32 103 51 60 200 28 122 237 185 32 233 20 49 100 62 149 59 38 248 150 118 81 171 99 130 109 196 158 218 123 78 200 78 79 184 102 20 168 221 49 193 53 99 70 197 142 9 156 79 229 165 93 36 219 143 69 255 96 58 78 152 158 75 187 200 61 78 152 168 61 43 68 184 111 208 222 115 66 160 99 65 108 71 135 210 61 39 36 237 17 97 36 124 106 23 85 247 156 48 18 113 207 9 41 247 166 144 200 167 215 210 46 108 13 218 65 215 49 33 98 198 4 157 63 146 102 80 96 51 94 163 157 147 129 5 0 1 216 187 130 20 128 65 24 230 254 255 232 109 56 133 117 88 137 54 149 14 244 22 145 234 193 131 33 73 221 10 72 16 204 34 38 226 159 19 134 66 225 105 237 98 236 15 40 40 76 197 4 182 94 121 215 31 85 80 148 135 179 149 152 9 98 114 223 183 30 81 25 182 98 29 79 151 43 150 130 226 21 134 127 227 149 153 19 164 94 83 241 152 180 118 65 10 8 35 115 210 177 134 153 136 73 242 109 39 172 89 187 16 162 35 136 201 169 135 235 205 86 177 200 255 156 204 212 11 78 76 234 154 80 153 19 121 184 84 10 54 113 62 251 7 103 88 230 52 5 229 207 99 147 147 117 227 18 128 189 115 73 1 0 4 129 40 115 255 75 71 208 54 117 202 2 63 238 220 4 45 31 111 212 54 32 193 250 80 209 46 1 76 190 204 192 192 126 41 222 251 255 47 163 93 148 65 209 64 226 242 206 137 10 38 70 131 114 26 197 138 50 115 34 110 237 2 56 227 225 20 237 18 141 69 181 97 248 5 62 52 152 100 139 118 109 102 78 158 175 19 54 24 20 176 239 41 189 5 76 202 70 187 72 131 146 221 152 204 106 48 241 175 33 128 70 103 64 134 17 31 220 240 162 230 12 10 3 141 58 38 12 131 111 105 23 209 13 213 225 210 49 193 210 17 29 140 75 187 200 93 138 69 205 165 93 180 220 115 66 74 199 4 44 79 165 205 240 244 92 218 69 112 105 22 158 25 25 74 247 172 144 125 47 9 25 123 78 240 154 71 141 123 78 208 204 163 168 99 66 234 210 46 28 29 19 106 46 237 194 186 20 11 164 30 200 198 55 131 194 72 138 121 164 240 241 204 200 208 178 99 66 143 25 19 178 27 210 148 116 76 144 249 48 227 160 244 112 216 115 66 213 125 61 35 4 0 4 208 232 12 200 48 231 83 101 198 132 129 126 29 147 1 93 218 133 101 198 132 158 75 187 24 177 217 79 107 243 177 217 135 103 233 30 74 199 4 168 14 223 12 10 49 225 67 238 102 251 193 178 231 132 210 165 93 48 253 67 126 105 23 157 247 156 144 220 49 25 36 75 187 136 153 241 64 230 83 237 158 19 98 143 19 38 247 248 95 52 253 195 250 56 97 58 118 76 232 189 180 139 106 128 136 142 10 35 54 247 192 180 67 233 225 50 131 50 218 41 193 14 0 2 104 116 6 100 4 243 169 49 99 130 111 68 29 44 63 28 58 38 200 252 1 232 152 32 155 79 147 165 106 216 204 39 49 60 137 221 115 66 181 142 9 52 189 81 251 56 225 193 180 180 139 162 142 9 218 12 10 189 58 38 4 59 22 131 113 207 9 22 243 168 214 49 161 100 105 23 49 29 11 106 239 17 161 243 158 19 154 31 39 140 111 105 23 116 25 22 53 151 118 129 220 7 98 195 26 206 3 113 83 252 160 219 115 2 210 67 228 140 9 206 142 9 129 142 202 80 6 35 185 115 2 16 128 189 115 87 1 0 4 1 96 254 255 71 23 14 65 144 73 249 40 12 219 92 34 183 46 207 204 10 72 198 108 108 50 128 145 121 81 247 232 249 88 129 21 80 249 94 4 147 49 182 204 127 82 175 140 65 240 72 237 82 84 56 36 224 22 181 231 36 148 218 69 128 192 235 57 39 210 30 22 188 216 227 101 198 67 237 162 206 227 242 157 48 3 38 34 181 107 71 197 218 216 175 150 185 226 17 121 206 9 5 38 26 112 128 14 98 31 171 93 226 165 81 187 96 221 115 18 29 84 126 135 147 38 128 70 103 64 70 249 100 243 169 53 131 2 74 128 180 218 12 78 168 161 74 110 195 154 150 123 78 134 210 210 46 162 239 57 193 210 49 194 217 112 199 211 48 167 218 158 19 60 51 40 100 45 237 130 54 116 169 181 180 107 64 142 19 38 181 99 130 204 167 112 198 132 90 247 156 208 125 105 23 17 199 9 227 234 40 225 157 65 33 114 207 9 89 199 9 227 89 138 133 175 163 131 115 233 20 49 75 187 72 49 143 1 75 199 135 6 29 147 255 12 20 118 84 24 135 199 158 19 100 48 232 58 38 16 67 70 247 156 224 1 67 185 147 2 16 64 163 51 32 163 124 154 242 169 50 131 194 64 187 142 9 122 71 97 160 55 195 131 59 118 120 26 198 116 217 19 66 165 25 20 162 246 156 16 176 143 212 61 33 84 217 115 242 31 247 113 194 127 200 153 177 32 198 127 36 46 205 26 78 199 9 51 18 227 158 1 62 78 120 216 236 57 33 98 6 5 220 49 32 97 51 60 178 126 178 102 80 72 237 152 48 82 249 158 19 6 212 142 10 81 29 9 58 223 115 130 239 194 72 100 62 173 246 156 12 186 205 240 16 67 40 95 218 5 51 10 74 15 245 25 19 24 128 205 118 13 246 206 9 64 0 141 206 128 140 242 7 132 63 100 58 38 248 70 208 25 240 44 237 162 164 163 66 68 195 125 64 150 118 145 235 31 52 62 209 29 19 60 246 17 59 99 66 213 61 39 12 152 233 143 162 142 9 204 125 84 92 218 69 204 12 27 46 62 93 55 195 15 194 165 93 3 185 231 132 170 75 187 24 9 239 57 1 53 252 255 163 155 71 70 199 4 231 61 39 248 150 118 17 59 99 130 108 254 80 57 78 152 204 165 93 20 55 228 25 24 80 204 163 231 158 19 100 48 40 102 80 168 209 49 25 198 51 40 131 169 83 2 16 64 163 51 32 163 252 65 197 31 82 29 19 6 34 26 170 180 154 97 24 134 29 19 100 62 209 75 187 8 117 36 168 112 207 9 78 243 177 117 76 9 152 63 144 75 187 104 189 231 4 153 63 24 246 156 128 0 53 54 195 195 248 184 150 82 17 50 159 168 142 13 189 246 156 192 58 6 3 185 180 11 143 121 164 28 39 76 86 199 4 54 67 65 163 142 9 222 134 62 35 233 247 156 80 220 113 128 8 210 109 6 5 23 24 20 29 19 84 7 49 64 13 36 138 63 18 58 38 32 64 239 206 9 64 128 1 0 122 35 215 173 23 3 175 172 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
helpPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 164 73 68 65 84 56 203 165 147 221 79 146 97 24 198 253 91 224 79 104 173 230 106 213 106 157 213 214 56 232 160 3 167 230 39 208 212 114 181 12 150 179 116 152 211 44 41 53 51 157 10 106 234 156 138 2 111 2 34 136 38 202 151 32 240 98 34 95 202 199 68 120 225 85 179 237 234 141 53 156 203 217 90 7 191 147 123 207 245 219 174 103 247 157 3 32 231 127 248 99 160 35 15 217 90 247 129 64 227 220 39 85 235 52 245 197 158 162 20 54 138 156 177 36 4 83 166 61 246 153 2 181 147 230 204 58 210 33 79 248 16 27 59 52 220 65 42 131 103 59 13 71 32 141 145 197 104 104 104 33 204 57 85 64 172 165 56 114 107 50 181 25 217 135 249 219 46 164 115 126 136 198 156 168 31 89 71 23 177 1 131 35 10 87 136 194 39 149 63 213 69 108 113 78 8 100 230 4 107 114 53 30 220 8 211 88 112 198 80 203 132 58 9 15 252 49 26 190 88 26 99 139 1 212 72 173 80 174 134 96 247 37 208 58 73 6 155 199 93 172 172 96 116 41 42 92 243 167 176 226 137 67 56 104 199 227 30 51 210 7 71 12 63 126 115 132 218 65 11 106 250 77 152 183 71 160 93 139 160 78 106 21 102 5 125 115 33 210 197 116 237 38 54 81 221 99 66 85 247 10 252 209 52 166 150 3 80 219 182 65 51 2 181 45 8 110 187 14 173 83 14 88 189 187 120 210 109 36 179 130 14 133 151 118 7 147 16 74 44 168 252 104 4 175 211 192 60 214 99 218 232 195 46 117 128 157 93 26 189 179 46 148 182 105 81 249 65 15 135 111 15 60 177 158 206 10 90 198 93 180 43 144 196 211 1 11 184 29 122 148 191 215 161 236 157 14 73 250 59 182 34 20 248 98 45 138 90 84 184 207 192 23 107 96 247 199 145 223 164 62 22 188 144 218 72 219 86 28 98 153 11 188 118 3 202 196 243 40 121 59 135 226 215 234 12 133 175 8 20 54 42 80 208 40 71 189 116 25 70 79 4 119 235 228 199 21 152 62 66 133 49 0 189 35 130 202 14 67 54 24 75 236 35 178 71 35 79 52 147 161 80 36 3 177 234 131 132 169 115 171 102 226 248 19 249 226 5 86 249 155 249 224 146 51 2 229 138 31 220 54 13 10 154 148 104 27 55 101 248 21 206 111 152 192 176 198 13 141 37 128 155 213 99 193 235 15 71 89 39 22 41 79 52 203 185 215 64 164 116 246 109 168 173 65 188 148 44 161 164 89 142 162 198 105 60 239 213 67 105 244 66 101 246 227 106 197 231 212 229 7 67 156 83 87 249 142 80 198 185 253 108 50 212 71 172 227 171 59 12 139 55 150 97 209 185 131 46 153 13 185 60 105 232 98 185 132 115 230 49 221 120 52 202 190 86 53 34 184 82 49 76 94 226 15 82 185 92 41 117 161 108 128 60 95 218 47 56 87 220 199 254 235 53 254 43 63 1 215 175 5 162 189 77 189 196 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
hpidesignCss
	^ '/* @override http://localhost:8080/seaside/files/ITFileLibrary/hpidesign.css */

/* @group all_components */

/* @group drag_drop_lists */

fieldset.drag_list {
	float: left;
	width: 170px;
}

fieldset.drag_list ul {
	min-height: 50px;
	color: #000;
	
	list-style: none;
	padding: 0;
	margin: 0;
}

fieldset.drag_list ul li {
	margin: 4px;
	padding: 2px;
	cursor: move;
	font-weight: normal;
	text-align: center;
}

fieldset.drag_list ul li.available {
	background-color: #dfe1e1;
  	border: 1px dashed #5e676a;
}
fieldset.drag_list ul li.attached {
	background-color: #cce4ec;
  	border: 1px solid #007a9e;
}



/* @end */

/* @group side_menu */

#tab_menu {
	border: 1px solid black;
	border-color: #aeb3b4 #5e676a #5e676a #aeb3b4;
	margin: 2em 5px;
	padding: .5em 0 0;
	float: left;
	width: 14em;
}

#tab_menu ul { 
	list-style: none;
	margin: 0 0 .5em;
	padding: 0;
}

#tab_menu li {
	margin: 0;
	padding: 0.1em .5em;
	font-weight: bold;
}
#tab_menu li li {
	margin: 0 -0.5em;
	padding-left: 1.5em;
	font-weight: normal
}

#tab_menu li li.active {
	background: #cce4ec;
}
#tab_menu :link, #tab_menu :visited {
	border: none;
	display: block
}
#tab_menu :link:hover, #tab_menu :visited:hover {
	background: transparent
}

.content_with_menu {
	padding: 0.4em 2em;
	margin-left: 14em;
	min-height: 300px;
	width: 400px;
	max-width: 100%
}



/* @end */

body {
	margin: 0px;
	font: 11px Lucida Grande, Verdana, Arial, Helvetica, sans serif;
}

:link, :visited {
	color: #007a9e;
	text-decoration: none;
	border-bottom: 1px dotted #aeb3b4;
}

:link:hover, :visited:hover {
	background-color: #dfe1e1;
	color: #4ca2bb;
}

input {
	font: 11px Lucida Grande, Verdana, Arial, Helvetica, sans serif;
}

.blue input {
	border: 1px dashed #007a9e;
	background-color: #cce4ec;
	width: 100%;
}

fieldset {
	margin: 5px;
	/*color: #aeb3b4;*/
	color: #515151;
	border: 1px solid #aeb3b4;
}

fieldset.add_comment {
	width: 578px;
	min-height: 100px;
}

fieldset ul li {
	color: #000;
	font-weight: normal;
}

.content {
/*	border:1px #000000 dotted;	ONLY DEBUG */
	min-width: 700px;
	max-width: 100%;
	margin: 5px;
}

/* a generic button */

button.submit {
	text-align: center;

	font: 11px Lucida Grande, Verdana, Arial, Helvetica, sans serif;
	
	border: 1px solid #aeb3b4;
	background-color: #dfe1e1;
	
	padding: .1em .3em;
	
	margin-left: 5px;
	margin-right: 5px;
	/*min-width: 5em;*/
}

button.submit:hover {
	border: 1px solid #007a9e;
	background-color: #cce4ec;
	color: #007a9e;
}

/* links/anchors with images */
:link img, :visited img{
	color: #fff;
	background-color: #fff;
	text-decoration: none;
	border: none;
	display: block;
}

:link:hover img, :visited:hover img{
	color: #09c;
	background-color: #09c
}

/* @end */

/* @group add_issue_component */

.add_issue {
	width:600px;
}

.add_issue_textarea {
	max-width:570px;
	min-height:100px;
	width:100%;
	/*height:100%;*/
	border:none;
	resize:vertical;
}

/*.add_issue_textarea {
	width:100%;
	height:100%;
}*/

.add_issue_inPlaceEdit {
}



/* @end */

/* @group header_component */

div#top {
	background: url(seaside/files/ITFileLibrary/headerPng) no-repeat;
}

#project_name {
	float:left;
	margin: 0px;
	padding: 5px;
}

#login_menu {
	text-align: right;
	color: #aeb3b4;
	margin: 5px;
	padding: 5px;
	float:right;
}

#login_menu .text {
	color: #5e676a;
}



/* @end */

/* @group menu_component */

#main_menu {
	margin: 0 .66em 0px 0px;
	padding: .2em 0;
	
	border: none;
	border-right: 1px solid #000;
	
	background-color: #dfe1e1;
}

#main_menu ul {
	margin: 0;
	font-size: 10px;
	text-align: right;
	list-style: none;
}

#main_menu li {
	padding: .25em 0;
	display: inline;
	white-space: nowrap;
}

#main_menu :link, #main_menu :visited {
	color: #000;
	background-color: #fff;
	
	border: 1px solid #000;
	border-right: none;
	
	padding: .2em 20px;
}

#main_menu :link:hover, #main_menu :visited:hover {
	background-color: #aeb3b4;
}

#main_menu .active :link, #main_menu .active :visited {
	font-weight: bold;
	color: #dfe1e1;
	background-color: #000;
}



/* @end */

/* @group footer_component */

.footer {
	border-top: 1px #007a9e dashed;
	margin-top: 25px;
}

.footer_right {
	padding: 0.5em 1em;
	text-align: right;
	color: #9CA0AD;
	float:right;
}



/* @end */

/* @group issue_component */

/* @group list_details */

table#issue.list_details { 
	width:100%;
	text-align: left;
}

table#issue.list_details thead {
	color: #5e676a;
}

table#issue.list_details .tag { 
	width: 20%;
	text-align: center;
	font-weight: bold;
}

#issue tbody tr td div span {
	white-space: nowrap;
}

#issue tbody tr td {
	white-space: nowrap;
}

table#issue.list_details .detail_content {
	width: 100%;
	text-align: left;
	padding-left: 1em;
	min-height: 3ex;
}

/* copied from list_issues */

.list_details tbody td :link, .list_issues tbody td :visited {
	display: block
}

.list_details {
	/*border:1px #000000 dotted;*/	/* ONLY DEBUG */
	border-spacing: 0px;
	/*table-layout: automatic;*/ /* not W3C valid*/
	width:100%;
}

.list_details thead th {
	
	background-color: #fff;
	text-transform: capitalize;
	vertical-align: middle;
	
	border-left: 1px dotted #8e9496;
	border-bottom: 1px solid #8e9496;
	
	padding: .3em 1em;
}

.list_details tbody tr {
	text-align: center;
}

.list_details tbody td {
	padding: .1em .5em;
	border-bottom: 1px solid #8e9496;
	border-left: 1px solid #8e9496;
}

.list_details tbody td :link, .list_issues tbody td :visited {
	display: block;
	/*font-weight: bold;*/
	border: none;
	background: none;
	color: #000;
}

.list_details tbody tr:hover {
	background: #7fbcce;
}

.list_details tr.odd {
	background-color: #dfe1e1;
}

.list_details tr.even {
	background-color: #fff;
}

/* @end */

/* detailed view */

table#issue.list_issues { 
	width: 100%;
	text-align: left;
}

table#issue.list_issues thead {
	color: #5e676a;
}

table#issue.list_issues .tag { 
	width: 20%;
	text-align: center;
	font-weight: bold;
}

table#issue.list_issues .content {
	width: 80%;
	text-align: left;
	padding-left: 1em;
	
	min-height: 3ex;
}

#issue :link, #issue :visited {
	display: block;
	border: none;
}

#issue div.detail table {
	border-spacing: 0;
	vertical-align: middle;
	text-align: center;
}

#issue div.detail table td {
	padding: .15em .3em;
}

#issue div.detail table td.cross {
	background-color: #5e676a;
	border: 4px solid #fff;
	color: #dfe1e1;
}

#issue div.detail table td.tag {
	background-color: #dfe1e1;
	border: 1px solid #5e676a;
	border-right: none;
	
	width: 40%; 
	font-weight: bold;
}

#issue div.detail table td.content {
	border: 1px dashed #5e676a;
	border-left: none;
	width: 60%;
}

.comment {
	background: #cce4ec;
	padding: 0.2em 1em 1em 1em;
	border: 1px dashed #007a9e;
	margin-bottom: 1em;
}

.comment h3 {
 border-bottom: 1px solid #007a9e;
 color: #007a9e;
 font-size: 100%;
 font-weight: normal;
}



/* @end */

/* @group list_issues_component */

/* list issues table */

.list_issues tbody td :link, .list_issues tbody td :visited {
	display: block
}

.list_issues {
	/*border:1px #000000 dotted;*/	/* ONLY DEBUG */
	border-spacing: 0px;
	/*table-layout: automatic;*/ /* not W3C valid*/
	width:100%;
}

.list_issues thead th {
	
	background-color: #fff;
	text-transform: capitalize;
	vertical-align: middle;
	
	border-left: 1px dotted #8e9496;
	border-bottom: 1px solid #8e9496;
	
	padding: .3em 1em;
}

.list_issues tbody tr {
	text-align: center;
}

.list_issues tbody td {
	padding: .1em .5em;
	border-bottom: 1px solid #8e9496;
	border-left: 1px solid #8e9496;
}

.list_issues tbody td :link, .list_issues tbody td :visited {
	display: block;
	/*font-weight: bold;*/
	border: none;
	background: none;
	color: #000;
}

.list_issues tbody tr:hover {
	background: #7fbcce;
}

.list_issues tr.odd {
	background-color: #dfe1e1;
}

.list_issues tr.even {
	background-color: #fff;
}



/* @end */

/* @group message_component */

/* message box */
div.messagebox {
	background-color: white;
	border: 1px solid black;
	color: black;
	padding: 1em;
}

div.messagebox .title {	
	border-bottom: 1px dotted #8e9496;
	font-weight: bold;
	margin-bottom: 1ex;
	padding-bottom: 0.3ex;
}

div.messagebox .message {
	max-width: 20em;
}

div.messagebox .message table {
	width: 100%;
}

div.messagebox .message input {
	border: 1px dashed #007a9e;
	background-color: #cce4ec;
	width: 100%;
}


div.messagebox .buttons {
	margin-top: 2ex;
	text-align: center;
}



/* @end */

/* Debugging for Team 3*/'! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
keyPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 246 73 68 65 84 56 203 141 147 75 79 19 81 24 134 137 191 166 77 89 224 112 145 145 75 66 209 128 11 137 127 192 165 11 87 238 216 32 183 146 98 233 4 82 18 104 72 4 149 184 34 52 134 144 24 13 37 16 8 144 82 160 6 130 211 6 193 82 46 26 90 152 105 59 157 153 150 150 190 158 115 170 8 10 109 39 121 55 51 121 158 239 114 206 148 0 40 201 151 195 169 7 78 18 252 142 243 223 239 69 193 153 228 119 150 155 36 133 97 213 135 84 228 13 210 241 57 242 58 203 36 193 73 243 104 94 193 85 56 249 179 31 169 211 241 63 213 65 159 224 56 15 207 48 63 159 79 192 90 166 176 126 212 129 228 201 16 201 108 78 226 50 35 155 145 176 55 194 49 201 173 2 218 50 133 89 142 187 153 36 29 155 133 178 35 64 221 123 132 139 228 2 118 28 101 184 189 3 253 43 233 160 239 175 132 137 44 136 139 77 136 250 170 201 120 31 32 190 52 222 8 223 161 59 8 47 189 32 64 23 98 155 117 80 191 61 133 22 124 14 197 255 4 242 70 5 210 178 21 98 167 17 51 189 21 129 255 207 123 250 33 194 203 173 56 151 92 164 26 79 224 199 164 250 51 196 125 181 144 87 57 6 251 187 77 112 219 42 183 175 45 145 194 217 76 140 204 233 134 118 212 134 232 150 1 234 246 61 104 251 61 56 156 168 71 70 121 71 218 158 128 223 98 194 188 80 249 229 242 24 175 222 180 84 120 18 74 160 133 84 50 65 17 235 160 135 236 240 218 57 233 115 79 249 174 216 110 68 192 90 138 133 129 170 181 107 23 41 87 57 74 90 158 66 34 208 8 201 99 64 194 223 0 253 160 15 107 2 119 54 109 171 126 159 239 182 82 65 252 92 118 147 101 241 144 150 13 208 118 91 8 108 133 87 224 164 143 2 255 186 208 191 146 59 243 196 10 98 98 35 89 88 51 212 144 5 30 219 221 232 167 126 126 164 16 204 4 33 151 121 67 143 44 65 11 117 66 255 241 22 171 118 78 158 113 220 31 44 6 102 2 239 88 125 219 166 179 234 120 221 81 30 161 51 47 14 215 188 42 22 166 249 5 221 168 255 68 133 204 134 107 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
keygoPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 122 73 68 65 84 56 203 141 147 93 72 147 97 24 134 191 85 36 29 69 7 18 33 36 30 4 142 17 229 154 217 52 131 182 50 61 200 232 32 130 126 41 209 126 132 118 34 130 76 211 109 44 167 195 156 160 115 80 106 203 6 214 86 141 164 8 29 134 162 54 127 65 168 57 44 19 75 139 182 218 159 155 219 220 207 221 251 125 90 45 155 214 7 247 201 251 114 95 207 253 60 207 251 81 0 168 245 244 201 120 68 77 132 21 169 87 223 255 151 57 28 248 192 40 30 228 223 102 223 56 130 246 86 132 60 61 228 56 202 64 102 244 130 187 235 2 98 205 129 47 117 8 126 211 254 172 14 250 155 209 242 97 110 226 191 90 15 192 68 166 205 254 185 155 8 124 109 36 50 45 67 12 2 68 195 14 76 107 184 12 100 77 0 29 153 54 51 154 151 48 144 144 219 132 133 41 37 124 211 121 136 4 122 49 165 218 139 181 19 248 223 146 4 53 191 33 12 72 10 143 229 24 92 227 7 72 123 79 96 41 103 199 53 111 160 103 96 235 23 17 67 21 220 19 217 240 189 59 143 210 142 44 20 182 165 193 57 198 67 200 41 135 165 146 141 174 91 60 235 223 251 126 118 20 182 129 82 44 57 12 164 26 159 152 143 147 234 133 184 209 206 133 164 243 12 78 107 82 49 41 225 160 91 145 254 230 143 33 210 230 104 216 77 250 236 198 226 156 24 34 93 54 68 15 50 112 237 62 15 5 45 123 80 254 244 20 30 141 52 160 68 127 2 57 245 219 163 217 202 109 44 6 16 251 210 130 54 61 22 172 249 112 14 113 80 220 158 1 227 184 26 143 199 26 25 99 199 136 10 42 83 9 180 175 107 113 85 151 131 244 234 132 208 110 25 139 181 82 217 69 34 27 225 181 10 225 48 167 194 59 121 24 5 164 95 218 124 187 75 132 154 151 197 144 191 40 66 85 231 37 72 159 95 129 166 79 134 115 247 14 98 87 37 21 164 1 158 37 103 55 25 22 31 142 129 84 44 190 207 135 255 163 28 103 239 176 241 112 88 5 157 185 14 218 193 90 180 14 40 8 228 58 154 251 164 40 210 229 34 165 130 154 76 42 163 182 44 239 220 59 8 183 69 72 6 150 11 223 172 20 102 69 154 235 100 115 74 52 175 33 41 34 172 79 140 100 41 183 70 46 180 101 66 221 91 133 203 237 57 72 174 160 38 118 148 81 9 204 12 102 13 130 49 191 189 31 139 179 149 240 127 110 195 80 45 215 217 165 202 108 88 189 94 142 140 101 191 168 21 34 89 76 141 18 243 230 95 255 194 112 203 33 241 132 122 255 252 168 106 159 125 68 201 253 222 215 148 85 29 239 113 145 200 246 157 226 141 72 44 163 54 197 158 255 0 96 31 212 153 42 186 174 102 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
logobottomPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 111 0 0 0 69 8 2 0 0 0 203 144 47 103 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 10 79 105 67 67 80 80 104 111 116 111 115 104 111 112 32 73 67 67 32 112 114 111 102 105 108 101 0 0 120 218 157 83 103 84 83 233 22 61 247 222 244 66 75 136 128 148 75 111 82 21 8 32 82 66 139 128 20 145 38 42 33 9 16 74 136 33 161 217 21 81 193 17 69 69 4 27 200 160 136 3 142 142 128 140 21 81 44 12 138 10 216 7 228 33 162 142 131 163 136 138 202 251 225 123 163 107 214 188 247 230 205 254 181 215 62 231 172 243 157 179 207 7 192 8 12 150 72 51 81 53 128 12 169 66 30 17 224 131 199 196 198 225 228 46 64 129 10 36 112 0 16 8 179 100 33 115 253 35 1 0 248 126 60 60 43 34 192 7 190 0 1 120 211 11 8 0 192 77 155 192 48 28 135 255 15 234 66 153 92 1 128 132 1 192 116 145 56 75 8 128 20 0 64 122 142 66 166 0 64 70 1 128 157 152 38 83 0 160 4 0 96 203 99 98 227 0 80 45 0 96 39 127 230 211 0 128 157 248 153 123 1 0 91 148 33 21 1 160 145 0 32 19 101 136 68 0 104 59 0 172 207 86 138 69 0 88 48 0 20 102 75 196 57 0 216 45 0 48 73 87 102 72 0 176 183 0 192 206 16 11 178 0 8 12 0 48 81 136 133 41 0 4 123 0 96 200 35 35 120 0 132 153 0 20 70 242 87 60 241 43 174 16 231 42 0 0 120 153 178 60 185 36 57 69 129 91 8 45 113 7 87 87 46 30 40 206 73 23 43 20 54 97 2 97 154 64 46 194 121 153 25 50 129 52 15 224 243 204 0 0 160 145 21 17 224 131 243 253 120 206 14 174 206 206 54 142 182 14 95 45 234 191 6 255 34 98 98 227 254 229 207 171 112 64 0 0 225 116 126 209 254 44 47 179 26 128 59 6 128 109 254 162 37 238 4 104 94 11 160 117 247 139 102 178 15 64 181 0 160 233 218 87 243 112 248 126 60 60 69 161 144 185 217 217 229 228 228 216 74 196 66 91 97 202 87 125 254 103 194 95 192 87 253 108 249 126 60 252 247 245 224 190 226 36 129 50 93 129 71 4 248 224 194 204 244 76 165 28 207 146 9 132 98 220 230 143 71 252 183 11 255 252 29 211 34 196 73 98 185 88 42 20 227 81 18 113 142 68 154 140 243 50 165 34 137 66 146 41 197 37 210 255 100 226 223 44 251 3 62 223 53 0 176 106 62 1 123 145 45 168 93 99 3 246 75 39 16 88 116 192 226 247 0 0 242 187 111 193 212 40 8 3 128 104 131 225 207 119 255 239 63 253 71 160 37 0 128 102 73 146 113 0 0 94 68 36 46 84 202 179 63 199 8 0 0 68 160 129 42 176 65 27 244 193 24 44 192 6 28 193 5 220 193 11 252 96 54 132 66 36 196 194 66 16 66 10 100 128 28 114 96 41 172 130 66 40 134 205 176 29 42 96 47 212 64 29 52 192 81 104 134 147 112 14 46 194 85 184 14 61 112 15 250 97 8 158 193 40 188 129 9 4 65 200 8 19 97 33 218 136 1 98 138 88 35 142 8 23 153 133 248 33 193 72 4 18 139 36 32 201 136 20 81 34 75 145 53 72 49 82 138 84 32 85 72 29 242 61 114 2 57 135 92 70 186 145 59 200 0 50 130 252 134 188 71 49 148 129 178 81 61 212 12 181 67 185 168 55 26 132 70 162 11 208 100 116 49 154 143 22 160 155 208 114 180 26 61 140 54 161 231 208 171 104 15 218 143 62 67 199 48 192 232 24 7 51 196 108 48 46 198 195 66 177 56 44 9 147 99 203 177 34 172 12 171 198 26 176 86 172 3 187 137 245 99 207 177 119 4 18 129 69 192 9 54 4 119 66 32 97 30 65 72 88 76 88 78 216 72 168 32 28 36 52 17 218 9 55 9 3 132 81 194 39 34 147 168 75 180 38 186 17 249 196 24 98 50 49 135 88 72 44 35 214 18 143 19 47 16 123 136 67 196 55 36 18 137 67 50 39 185 144 2 73 177 164 84 210 18 210 70 210 110 82 35 233 44 169 155 52 72 26 35 147 201 218 100 107 178 7 57 148 44 32 43 200 133 228 157 228 195 228 51 228 27 228 33 242 91 10 157 98 64 113 164 248 83 226 40 82 202 106 74 25 229 16 229 52 229 6 101 152 50 65 85 163 154 82 221 168 161 84 17 53 143 90 66 173 161 182 82 175 81 135 168 19 52 117 154 57 205 131 22 73 75 165 173 162 149 211 26 104 23 104 247 105 175 232 116 186 17 221 149 30 78 151 208 87 210 203 233 71 232 151 232 3 244 119 12 13 134 21 131 199 136 103 40 25 155 24 7 24 103 25 119 24 175 152 76 166 25 211 139 25 199 84 48 55 49 235 152 231 153 15 153 111 85 88 42 182 42 124 21 145 202 10 149 74 149 38 149 27 42 47 84 169 170 166 170 222 170 11 85 243 85 203 84 143 169 94 83 125 174 70 85 51 83 227 169 9 212 150 171 85 170 157 80 235 83 27 83 103 169 59 168 135 170 103 168 111 84 63 164 126 89 253 137 6 89 195 76 195 79 67 164 81 160 177 95 227 188 198 32 11 99 25 179 120 44 33 107 13 171 134 117 129 53 196 38 177 205 217 124 118 42 187 152 253 29 187 139 61 170 169 161 57 67 51 74 51 87 179 82 243 148 102 63 7 227 152 113 248 156 116 78 9 231 40 167 151 243 126 138 222 20 239 41 226 41 27 166 52 76 185 49 101 92 107 170 150 151 150 88 171 72 171 81 171 71 235 189 54 174 237 167 157 166 189 69 187 89 251 129 14 65 199 74 39 92 39 71 103 143 206 5 157 231 83 217 83 221 167 10 167 22 77 61 58 245 174 46 170 107 165 27 161 187 68 119 191 110 167 238 152 158 190 94 128 158 76 111 167 222 121 189 231 250 28 125 47 253 84 253 109 250 167 245 71 12 88 6 179 12 36 6 219 12 206 24 60 197 53 113 111 60 29 47 199 219 241 81 67 93 195 64 67 165 97 149 97 151 225 132 145 185 209 60 163 213 70 141 70 15 140 105 198 92 227 36 227 109 198 109 198 163 38 6 38 33 38 75 77 234 77 238 154 82 77 185 166 41 166 59 76 59 76 199 205 204 205 162 205 214 153 53 155 61 49 215 50 231 155 231 155 215 155 223 183 96 90 120 90 44 182 168 182 184 101 73 178 228 90 166 89 238 182 188 110 133 90 57 89 165 88 85 90 93 179 70 173 157 173 37 214 187 173 187 167 17 167 185 78 147 78 171 158 214 103 195 176 241 182 201 182 169 183 25 176 229 216 6 219 174 182 109 182 125 97 103 98 23 103 183 197 174 195 238 147 189 147 125 186 125 141 253 61 7 13 135 217 14 171 29 90 29 126 115 180 114 20 58 86 58 222 154 206 156 238 63 125 197 244 150 233 47 103 88 207 16 207 216 51 227 182 19 203 41 196 105 157 83 155 211 71 103 23 103 185 115 131 243 136 139 137 75 130 203 46 151 62 46 155 27 198 221 200 189 228 74 116 245 113 93 225 122 210 245 157 155 179 155 194 237 168 219 175 238 54 238 105 238 135 220 159 204 52 159 41 158 89 51 115 208 195 200 67 224 81 229 209 63 11 159 149 48 107 223 172 126 79 67 79 129 103 181 231 35 47 99 47 145 87 173 215 176 183 165 119 170 247 97 239 23 62 246 62 114 159 227 62 227 60 55 222 50 222 89 95 204 55 192 183 200 183 203 79 195 111 158 95 133 223 67 127 35 255 100 255 122 255 209 0 167 128 37 1 103 3 137 129 65 129 91 2 251 248 122 124 33 191 142 63 58 219 101 246 178 217 237 65 140 160 185 65 21 65 143 130 173 130 229 193 173 33 104 200 236 144 173 33 247 231 152 206 145 206 105 14 133 80 126 232 214 208 7 97 230 97 139 195 126 12 39 133 135 133 87 134 63 142 112 136 88 26 209 49 151 53 119 209 220 67 115 223 68 250 68 150 68 222 155 103 49 79 57 175 45 74 53 42 62 170 46 106 60 218 55 186 52 186 63 198 46 102 89 204 213 88 157 88 73 108 75 28 57 46 42 174 54 110 108 190 223 252 237 243 135 226 157 226 11 227 123 23 152 47 200 93 112 121 161 206 194 244 133 167 22 169 46 18 44 58 150 64 76 136 78 56 148 240 65 16 42 168 22 140 37 242 19 119 37 142 10 121 194 29 194 103 34 47 209 54 209 136 216 67 92 42 30 78 242 72 42 77 122 146 236 145 188 53 121 36 197 51 165 44 229 185 132 39 169 144 188 76 13 76 221 155 58 158 22 154 118 32 109 50 61 58 189 49 131 146 145 144 113 66 170 33 77 147 182 103 234 103 230 102 118 203 172 101 133 178 254 197 110 139 183 47 30 149 7 201 107 179 144 172 5 89 45 10 182 66 166 232 84 90 40 215 42 7 178 103 101 87 102 191 205 137 202 57 150 171 158 43 205 237 204 179 202 219 144 55 156 239 159 255 237 18 194 18 225 146 182 165 134 75 87 45 29 88 230 189 172 106 57 178 60 113 121 219 10 227 21 5 43 134 86 6 172 60 184 138 182 42 109 213 79 171 237 87 151 174 126 189 38 122 77 107 129 94 193 202 130 193 181 1 107 235 11 85 10 229 133 125 235 220 215 237 93 79 88 47 89 223 181 97 250 134 157 27 62 21 137 138 174 20 219 23 151 21 127 216 40 220 120 229 27 135 111 202 191 153 220 148 180 169 171 196 185 100 207 102 210 102 233 230 222 45 158 91 14 150 170 151 230 151 14 110 13 217 218 180 13 223 86 180 237 245 246 69 219 47 151 205 40 219 187 131 182 67 185 163 191 60 184 188 101 167 201 206 205 59 63 84 164 84 244 84 250 84 54 238 210 221 181 97 215 248 110 209 238 27 123 188 246 52 236 213 219 91 188 247 253 62 201 190 219 85 1 85 77 213 102 213 101 251 73 251 179 247 63 174 137 170 233 248 150 251 109 93 173 78 109 113 237 199 3 210 3 253 7 35 14 182 215 185 212 213 29 210 61 84 82 143 214 43 235 71 14 199 31 190 254 157 239 119 45 13 54 13 85 141 156 198 226 35 112 68 121 228 233 247 9 223 247 30 13 58 218 118 140 123 172 225 7 211 31 118 29 103 29 47 106 66 154 242 154 70 155 83 154 251 91 98 91 186 79 204 62 209 214 234 222 122 252 71 219 31 15 156 52 60 89 121 74 243 84 201 105 218 233 130 211 147 103 242 207 140 157 149 157 125 126 46 249 220 96 219 162 182 123 231 99 206 223 106 15 111 239 186 16 116 225 210 69 255 139 231 59 188 59 206 92 242 184 116 242 178 219 229 19 87 184 87 154 175 58 95 109 234 116 234 60 254 147 211 79 199 187 156 187 154 174 185 92 107 185 238 122 189 181 123 102 247 233 27 158 55 206 221 244 189 121 241 22 255 214 213 158 57 61 221 189 243 122 111 247 197 247 245 223 22 221 126 114 39 253 206 203 187 217 119 39 238 173 188 79 188 95 244 64 237 65 217 67 221 135 213 63 91 254 220 216 239 220 127 106 192 119 160 243 209 220 71 247 6 133 131 207 254 145 245 143 15 67 5 143 153 143 203 134 13 134 235 158 56 62 57 57 226 63 114 253 233 252 167 67 207 100 207 38 158 23 254 162 254 203 174 23 22 47 126 248 213 235 215 206 209 152 209 161 151 242 151 147 191 109 124 165 253 234 192 235 25 175 219 198 194 198 30 190 201 120 51 49 94 244 86 251 237 193 119 220 119 29 239 163 223 15 79 228 124 32 127 40 255 104 249 177 245 83 208 167 251 147 25 147 147 255 4 3 152 243 252 99 51 45 219 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 233 0 0 117 48 0 0 234 96 0 0 58 152 0 0 23 111 146 95 197 70 0 0 22 160 73 68 65 84 120 218 236 92 121 92 85 213 246 95 251 156 59 112 239 5 46 51 92 153 68 48 21 145 73 52 205 137 44 45 223 79 211 39 168 253 34 43 181 167 57 61 109 146 210 94 22 63 181 87 124 196 71 54 104 243 83 95 35 106 102 250 82 139 16 120 226 128 134 36 56 32 10 200 36 16 112 129 59 157 115 246 222 191 63 182 29 111 112 25 84 234 245 73 214 95 155 189 215 222 251 156 239 89 107 175 97 175 11 162 148 66 31 245 18 161 62 52 251 208 236 67 179 15 205 62 234 53 52 45 34 105 182 138 205 86 177 197 134 205 2 17 41 149 48 33 64 57 10 0 192 115 72 173 224 156 85 10 157 154 119 86 242 206 42 94 167 228 85 10 174 15 205 246 100 149 72 201 79 150 234 22 91 155 77 20 49 38 8 36 12 24 0 19 66 40 5 10 18 166 148 82 129 80 145 82 76 168 2 144 134 231 116 106 228 169 83 246 115 214 244 211 59 25 92 157 208 111 251 38 229 229 229 38 147 41 60 60 252 247 133 166 69 196 121 149 173 245 109 2 135 128 2 161 148 98 74 37 130 8 0 33 132 18 130 41 16 2 24 19 27 33 2 33 0 64 48 21 68 106 195 34 161 132 71 8 113 200 224 170 29 228 169 9 245 212 5 186 105 149 252 175 46 176 159 125 246 217 178 101 203 76 38 211 202 149 43 215 173 91 199 113 220 239 5 205 226 122 115 65 109 171 2 1 5 74 8 197 64 9 65 152 80 12 64 9 166 4 99 130 68 137 56 171 20 253 220 212 158 90 149 78 165 0 160 54 145 180 88 197 218 54 91 69 179 169 222 34 180 88 48 193 162 139 90 233 231 234 20 101 112 27 17 232 238 174 81 253 74 239 96 52 26 227 227 227 11 10 10 0 64 163 209 156 56 113 98 232 208 161 242 104 109 109 109 81 81 81 92 92 156 94 175 255 181 209 84 56 80 115 76 36 66 40 7 0 64 41 16 10 132 82 12 20 83 4 20 176 68 120 142 139 11 208 15 51 184 186 168 29 76 55 139 248 138 209 114 177 193 84 84 219 116 233 39 83 97 141 241 199 26 227 55 23 106 199 245 247 186 59 212 199 77 163 236 245 119 80 171 213 222 222 222 172 237 229 229 229 225 225 113 93 50 138 139 167 77 155 118 233 210 165 152 152 152 3 7 14 200 108 183 66 53 45 150 102 139 56 196 215 181 227 16 255 210 75 47 181 235 106 48 9 245 109 2 80 0 66 9 5 2 148 16 74 8 1 66 36 10 10 158 191 39 204 43 170 159 94 221 137 193 81 242 156 151 86 117 135 151 115 140 191 251 64 47 103 30 113 77 86 169 166 197 86 88 109 60 85 221 236 164 84 4 187 105 81 175 158 169 10 133 98 216 176 97 165 165 165 30 30 30 175 190 250 106 92 92 156 60 180 110 221 186 111 190 249 134 73 168 191 191 255 168 81 163 110 113 175 83 149 77 211 223 251 254 173 195 103 21 74 197 232 254 94 221 203 166 2 33 66 1 81 10 0 132 82 76 41 161 132 16 66 41 37 148 27 19 234 25 230 169 235 201 198 90 37 31 225 167 143 240 211 95 106 52 101 149 54 252 167 236 167 146 250 182 141 217 23 138 174 182 44 24 17 236 164 224 111 226 101 246 238 221 155 145 145 49 96 192 128 39 159 124 210 213 245 186 116 68 71 71 31 58 116 168 35 255 160 65 131 228 246 144 33 67 110 253 203 189 123 172 228 92 89 61 240 220 230 220 243 11 70 133 185 254 82 59 29 200 102 163 69 42 55 90 232 53 28 41 161 0 128 8 161 130 68 6 120 234 198 6 187 223 232 19 184 107 84 49 254 110 209 254 122 10 244 74 179 249 72 69 83 173 201 54 42 208 93 113 131 182 162 168 168 104 234 212 169 199 143 31 207 202 202 210 106 181 19 38 76 232 118 74 68 68 4 198 88 20 197 231 158 123 110 206 156 57 232 214 148 194 44 226 245 135 206 212 180 217 64 193 217 8 189 111 176 33 200 77 215 141 108 170 120 68 9 96 66 41 101 234 14 148 18 140 41 7 40 210 207 229 166 31 37 216 77 187 244 174 176 241 33 222 219 10 42 246 20 215 184 169 249 21 99 6 222 40 154 70 163 145 181 243 243 243 123 50 69 163 209 188 242 202 43 189 117 164 152 5 177 180 161 13 56 4 0 22 65 106 182 8 237 24 28 72 135 74 193 81 10 152 65 73 128 18 74 8 136 152 120 104 85 126 46 234 91 124 160 97 6 253 43 247 69 172 142 191 35 183 188 33 191 178 233 134 230 78 153 50 229 174 187 238 2 0 189 94 191 96 193 130 94 60 121 77 38 83 79 216 68 66 141 86 1 16 2 14 129 77 108 48 217 186 63 55 213 28 34 148 96 66 56 74 8 112 132 2 37 4 19 234 227 162 86 245 134 231 168 224 80 98 132 127 184 143 139 85 36 55 52 209 197 197 101 215 174 93 185 185 185 3 6 12 136 137 137 233 45 40 83 82 82 238 186 235 174 123 239 189 183 123 119 18 128 67 8 51 159 18 99 171 136 123 128 166 130 87 32 176 97 66 17 37 4 99 74 1 40 165 196 91 215 155 14 99 184 143 235 77 204 242 245 245 77 72 72 232 197 199 88 181 106 85 106 106 170 67 11 230 144 174 123 231 10 94 171 84 116 143 166 70 201 41 57 132 9 69 8 97 74 49 165 148 130 68 169 139 147 226 87 114 122 153 161 0 0 132 144 90 173 134 223 138 86 175 94 157 154 154 170 82 169 220 221 123 100 90 121 132 116 106 69 171 85 0 2 72 173 244 114 86 247 192 67 226 144 74 193 91 9 40 16 197 64 49 32 66 8 199 113 58 149 99 52 247 236 217 83 92 92 204 218 19 38 76 96 71 91 71 218 186 117 107 99 99 35 107 47 88 176 192 199 199 7 0 126 248 225 135 157 59 119 158 57 115 166 185 185 25 33 196 243 124 116 116 244 148 41 83 238 185 231 158 142 43 180 182 182 126 248 225 135 236 140 139 143 143 31 61 122 180 60 180 115 231 206 11 23 46 0 128 135 135 71 82 82 146 179 179 51 235 175 168 168 248 215 191 254 5 0 28 199 37 36 36 132 133 133 177 126 65 16 150 47 95 254 206 59 239 0 0 33 100 203 150 45 3 6 12 176 247 4 166 77 155 230 80 107 13 174 154 86 179 13 16 162 0 212 145 232 58 160 253 231 235 254 47 179 228 213 172 146 13 89 37 235 50 75 94 62 116 110 67 230 185 38 139 224 144 249 189 247 222 147 87 139 141 141 149 36 169 35 79 118 118 246 117 227 30 28 220 214 214 38 8 194 154 53 107 220 220 220 28 198 54 79 61 245 148 40 138 237 22 41 47 47 151 37 247 197 23 95 180 31 146 79 61 95 95 223 202 202 74 185 255 224 193 131 242 178 187 118 237 98 157 86 171 117 238 220 185 93 200 224 140 25 51 28 190 105 171 85 140 76 221 7 43 182 193 138 109 218 231 62 253 207 229 186 118 12 142 173 138 94 205 19 44 17 138 137 132 9 150 8 145 16 33 138 78 156 181 199 30 123 76 142 49 126 252 241 199 3 7 14 116 228 121 235 173 183 228 246 218 181 107 117 58 93 122 122 250 250 245 235 155 155 155 59 50 219 108 182 180 180 180 141 27 55 182 247 63 56 78 70 95 163 209 216 15 201 194 168 215 235 237 83 30 74 229 181 64 150 231 121 214 110 106 106 154 53 107 214 246 237 219 187 210 104 158 239 44 30 9 114 215 2 161 0 160 83 242 250 14 81 178 162 19 52 21 0 32 225 107 145 37 198 20 35 210 197 222 79 63 253 244 131 15 62 200 142 191 77 155 54 221 127 255 253 246 175 84 84 84 180 103 207 30 214 142 138 138 154 57 115 38 0 156 57 115 134 245 120 120 120 140 25 51 38 52 52 84 20 197 211 167 79 231 230 230 178 254 215 95 127 253 193 7 31 12 14 14 238 221 179 242 244 233 211 173 173 173 147 39 79 174 171 171 99 137 18 142 227 98 99 99 237 163 251 145 35 71 58 156 203 113 200 224 234 4 132 0 112 174 26 149 159 139 182 71 104 186 58 41 121 14 9 18 166 20 8 0 38 84 100 222 124 39 52 117 234 212 17 35 70 28 61 122 20 0 190 253 246 219 3 7 14 76 153 50 69 30 125 243 205 55 45 22 11 107 207 157 59 151 229 114 216 247 79 74 74 122 246 217 103 163 162 162 174 57 116 162 56 127 254 252 29 59 118 0 64 117 117 245 209 163 71 123 29 205 248 248 248 248 248 120 0 248 238 187 239 216 249 160 86 171 223 122 235 173 17 35 70 244 100 122 160 155 14 8 5 68 221 52 42 87 39 101 247 222 59 0 56 171 21 106 158 19 37 42 17 42 96 98 195 216 42 17 145 118 42 158 78 78 78 79 63 253 180 252 231 63 254 241 15 185 93 86 86 150 145 145 113 237 81 2 3 31 123 236 49 89 109 95 126 249 229 29 59 118 200 80 50 197 92 188 120 177 86 171 189 33 167 250 22 61 118 74 105 91 91 91 15 103 245 115 213 2 2 160 16 236 166 83 114 168 71 104 186 168 21 78 42 222 74 136 72 136 136 169 68 168 32 98 161 75 103 251 129 7 30 144 173 121 78 78 78 94 94 30 107 111 223 190 189 190 190 158 181 23 46 92 232 233 233 201 218 201 201 201 47 190 248 98 199 117 60 61 61 229 195 238 191 149 244 237 130 2 220 180 188 86 13 0 81 253 28 216 79 174 179 112 197 87 171 66 4 35 130 41 145 40 150 68 73 180 137 82 23 219 168 84 170 149 43 87 50 253 181 88 44 233 233 233 0 208 220 220 44 91 124 111 111 239 71 30 121 68 230 151 157 21 123 202 206 206 222 188 121 243 175 42 146 183 72 33 30 58 79 157 26 40 29 234 231 214 163 108 49 35 31 103 53 38 132 32 68 8 37 132 98 73 106 179 137 93 239 52 99 198 140 161 67 135 22 22 22 50 119 228 236 217 179 185 185 185 21 21 21 108 116 222 188 121 65 65 65 14 93 180 179 103 207 126 253 245 215 59 119 238 60 117 234 148 36 73 240 59 166 1 158 206 62 58 167 186 250 214 161 126 250 27 64 211 207 213 9 83 160 148 18 74 36 74 4 137 180 88 187 65 83 169 84 46 91 182 108 225 194 133 204 158 188 240 194 11 213 213 213 215 124 11 173 118 209 162 69 29 167 228 228 228 108 220 184 113 239 222 189 132 16 0 232 223 191 255 157 119 222 185 111 223 190 158 31 100 191 49 41 56 46 170 159 91 109 171 197 199 89 125 3 104 26 92 157 116 106 101 147 89 0 74 37 76 109 152 52 89 108 221 110 54 107 214 172 215 95 127 157 121 63 187 118 237 146 251 31 122 232 161 144 144 144 118 204 219 183 111 95 186 116 105 107 107 43 115 74 150 44 89 50 117 234 212 154 154 26 123 151 251 119 72 143 196 13 8 116 215 185 107 213 61 61 55 1 192 89 165 244 115 113 178 72 88 32 20 83 42 98 82 223 102 233 118 39 55 55 183 101 203 150 117 76 50 46 94 188 184 93 166 54 51 51 115 225 194 133 12 202 228 228 228 220 220 220 71 31 125 212 211 211 211 102 179 49 57 253 45 233 134 178 200 147 7 25 94 249 83 180 195 9 157 162 201 33 8 212 107 176 68 40 161 20 3 193 80 219 108 238 201 102 115 231 206 29 60 120 176 125 207 164 73 147 98 99 99 219 177 165 166 166 90 173 86 22 218 111 216 176 65 182 227 191 125 237 9 33 164 183 190 31 215 181 253 82 114 28 38 148 18 74 129 54 180 90 173 98 247 38 66 171 213 62 255 252 243 246 145 82 114 114 114 71 95 175 188 188 92 14 219 237 61 33 39 39 167 235 199 144 66 241 235 129 40 127 54 65 16 78 156 56 97 63 212 210 210 82 81 81 209 217 119 109 117 148 39 238 30 205 64 55 173 179 147 194 44 17 27 0 1 84 103 178 53 152 172 61 121 80 150 31 186 166 23 147 39 119 188 41 228 56 78 70 48 59 59 251 242 229 203 242 80 86 86 22 147 89 0 168 171 171 3 128 130 130 2 150 175 235 93 10 12 12 148 131 253 180 180 180 143 62 250 168 184 184 56 55 55 55 61 61 125 194 132 9 127 255 251 223 29 170 255 182 252 203 35 54 238 15 91 247 229 234 127 23 88 37 210 83 43 4 0 94 58 181 167 206 169 220 104 86 114 8 35 174 222 44 213 182 88 2 220 156 187 85 156 13 27 54 200 231 209 194 133 11 59 58 225 26 141 38 38 38 166 168 168 136 5 75 211 166 77 155 61 123 182 82 169 204 206 206 102 23 182 140 210 211 211 79 158 60 121 228 200 145 243 231 207 247 58 154 195 134 13 27 55 110 28 179 120 117 117 117 243 230 205 243 246 246 54 153 76 102 179 25 0 198 140 25 211 113 202 217 186 150 5 159 230 73 130 4 28 122 101 111 65 184 175 254 225 216 144 158 202 38 66 48 208 219 89 34 32 82 74 16 152 68 233 98 67 247 142 203 247 223 127 47 7 66 227 199 143 159 62 125 186 67 182 228 228 100 95 95 95 57 45 178 118 237 218 213 171 87 127 243 205 55 225 225 225 178 200 84 84 84 124 252 241 199 102 179 153 169 60 165 84 118 236 5 225 23 55 92 178 56 155 205 102 123 13 197 24 203 141 118 158 172 82 169 124 237 181 215 252 253 253 229 158 250 250 122 6 165 253 68 123 106 178 8 146 32 129 130 7 158 3 74 74 234 91 111 64 54 1 32 194 224 42 21 176 219 116 66 120 84 88 219 252 96 119 89 244 55 222 120 67 126 238 69 139 22 117 102 46 35 34 34 246 237 219 183 102 205 154 211 167 79 219 108 54 141 70 19 24 24 248 240 195 15 39 37 37 189 253 246 219 91 183 110 109 105 105 209 235 245 195 135 15 95 181 106 21 147 110 181 90 61 97 194 4 150 114 110 231 111 69 69 69 177 235 204 160 160 32 251 236 189 187 187 59 75 42 115 28 215 177 208 35 42 42 42 51 51 51 37 37 37 47 47 207 104 52 82 74 21 10 133 193 96 152 56 113 226 242 229 203 29 60 179 175 126 176 191 251 185 178 122 0 112 114 213 76 31 26 208 94 254 186 182 161 70 171 184 108 111 65 101 147 137 3 34 74 196 83 171 252 96 246 104 119 109 167 23 68 199 142 29 27 51 102 12 251 176 49 49 49 217 217 217 114 230 177 51 186 112 225 130 217 108 118 113 113 9 13 13 149 59 43 42 42 234 235 235 253 252 252 236 101 231 215 163 171 87 175 214 214 214 178 248 120 208 160 65 93 228 7 254 83 86 159 158 89 236 164 228 103 15 15 153 26 238 127 99 104 82 74 55 100 157 223 83 92 165 225 57 137 96 66 232 107 127 138 26 55 192 167 51 254 71 31 125 116 219 182 109 242 213 5 139 139 110 31 226 186 117 107 227 252 221 121 196 9 152 82 130 218 108 56 167 236 167 206 152 11 11 11 229 248 39 36 36 36 49 49 17 110 51 234 62 229 21 101 208 187 107 148 86 76 69 0 202 115 71 43 27 27 205 130 67 206 15 62 248 64 142 175 87 172 88 97 159 205 238 67 243 231 244 135 139 83 136 183 174 149 136 102 192 18 135 10 27 90 143 87 53 118 100 187 124 249 178 172 227 253 250 245 235 221 91 239 63 14 154 0 48 190 191 143 72 64 34 64 40 21 8 205 40 174 238 200 243 230 155 111 54 53 53 201 193 101 64 64 128 72 168 72 110 175 159 40 244 40 116 27 21 224 238 161 81 93 53 219 148 8 1 208 236 138 134 194 171 198 72 223 235 9 190 234 234 234 47 190 248 130 181 61 61 61 89 226 227 76 173 81 167 82 220 225 117 221 166 167 164 164 124 254 249 231 127 24 236 148 74 101 102 102 166 125 101 67 143 208 12 208 107 162 251 185 125 94 84 165 83 114 20 104 101 139 229 253 130 242 244 251 34 101 134 45 91 182 200 89 225 164 164 164 128 128 0 0 56 95 103 28 242 203 242 152 242 242 114 22 255 252 97 168 93 68 208 35 52 121 132 38 246 247 222 89 92 99 193 64 16 80 14 125 82 84 181 32 166 127 164 143 43 11 37 93 93 93 151 44 89 130 16 82 169 84 44 43 76 40 61 219 98 11 239 247 139 245 85 170 107 142 234 192 129 3 153 251 205 113 28 165 180 178 178 210 30 101 131 193 48 126 252 120 127 127 255 230 230 230 147 39 79 158 62 125 154 37 83 198 141 27 71 41 205 207 207 111 108 108 84 169 84 241 241 241 132 144 19 39 78 24 141 70 119 119 247 184 184 56 132 208 225 195 135 189 189 189 217 47 51 216 226 0 80 90 90 122 241 226 197 65 131 6 177 27 80 214 95 87 87 247 195 15 63 200 30 200 192 129 3 237 83 115 103 207 158 173 168 168 24 54 108 152 193 96 96 83 4 65 40 46 46 102 110 169 252 58 237 99 19 218 51 170 110 181 6 164 31 128 148 61 176 126 47 172 223 11 107 119 255 57 35 191 11 126 147 40 45 217 127 186 220 104 182 239 124 226 137 39 228 0 220 190 223 104 52 202 161 253 164 73 147 74 74 74 228 33 171 213 250 183 191 253 13 0 252 252 252 154 155 155 41 165 243 230 205 3 128 251 239 191 159 49 36 37 37 1 192 95 254 242 23 198 236 228 228 180 98 197 138 118 15 147 154 154 202 20 200 190 211 98 177 188 251 238 187 44 102 77 73 73 105 55 229 169 167 158 2 128 47 191 252 210 190 179 180 180 116 245 234 213 50 130 42 149 170 190 190 190 251 90 15 7 169 120 103 245 228 16 111 176 137 128 49 96 2 8 246 156 169 252 180 168 170 51 254 42 163 181 222 100 243 211 57 46 209 98 215 235 71 142 28 153 61 123 118 106 106 170 139 139 203 243 207 63 63 124 248 112 79 79 207 29 59 118 132 133 133 189 255 254 251 113 113 113 139 22 45 194 24 167 164 164 60 244 208 67 181 181 181 95 125 245 21 203 86 0 0 187 19 7 128 232 232 104 22 119 1 64 70 70 134 213 106 101 9 39 150 76 153 49 99 70 66 66 194 71 31 125 36 111 122 248 240 225 153 51 103 174 89 179 70 165 82 61 254 248 227 119 223 125 183 28 230 159 60 121 50 33 33 97 250 244 233 137 137 137 172 156 130 133 237 135 14 29 74 72 72 216 176 97 131 191 191 255 250 245 235 87 175 94 125 75 54 157 81 226 16 3 226 17 16 2 4 3 0 193 120 213 129 31 47 53 57 78 33 103 94 174 7 10 93 215 123 94 184 112 225 139 47 190 88 181 106 21 187 34 14 12 12 188 239 190 251 124 124 124 74 74 74 22 45 90 116 242 228 201 119 222 121 103 235 214 173 0 48 103 206 28 0 96 201 148 216 216 88 149 74 21 23 23 199 226 215 240 240 112 119 119 119 86 223 158 147 147 3 63 223 27 99 140 175 94 189 90 87 87 87 93 93 93 90 90 42 111 122 238 220 185 221 187 119 167 165 165 177 180 191 125 10 85 16 132 186 186 186 250 250 250 43 87 174 84 85 85 201 138 127 238 220 185 93 187 118 173 89 179 230 175 127 253 43 0 44 94 188 184 179 154 186 27 64 115 108 160 215 48 95 87 176 137 215 0 69 244 74 157 113 193 151 249 141 29 10 150 9 133 140 226 202 254 110 154 174 23 12 11 11 155 61 123 246 166 77 155 124 124 124 76 38 211 177 99 199 238 188 243 78 22 83 201 41 156 195 135 15 3 64 104 104 168 78 167 203 204 204 52 155 205 145 145 145 145 145 145 49 49 49 135 14 29 202 205 205 29 61 122 116 84 84 212 240 225 195 45 22 139 125 210 55 52 52 244 248 241 227 71 142 28 201 203 203 99 242 203 104 226 196 137 111 188 241 70 86 86 150 94 175 207 202 202 146 11 117 0 96 244 232 209 57 57 57 71 142 28 57 118 236 152 253 79 14 100 196 119 239 222 13 0 94 94 94 17 17 17 55 239 33 49 114 81 43 102 12 50 20 150 53 128 92 227 160 224 178 206 213 252 239 167 199 222 157 57 60 72 127 189 40 231 139 51 149 223 95 168 125 44 170 155 162 151 177 99 199 142 29 59 150 57 88 107 215 174 173 169 169 97 55 10 246 69 85 44 59 39 138 34 207 243 37 37 37 37 37 37 17 17 17 243 231 207 247 240 240 248 238 187 239 120 158 31 59 118 236 204 153 51 245 122 125 126 126 190 189 41 187 114 229 202 51 207 60 163 80 40 40 165 172 24 145 81 80 80 208 210 165 75 1 224 252 249 243 115 230 204 97 18 202 168 160 160 32 45 45 77 146 36 140 177 125 2 91 38 157 78 199 68 216 126 214 77 162 9 0 73 81 193 105 57 23 218 172 18 112 0 148 178 10 240 131 63 86 78 254 169 237 217 9 131 167 12 242 83 112 220 215 231 170 147 255 93 8 4 13 241 238 38 123 148 151 151 183 121 243 102 81 20 143 29 59 118 229 202 21 38 137 43 87 174 28 53 106 148 193 96 168 169 169 1 0 22 83 21 22 22 182 180 180 176 68 125 84 84 212 252 249 243 41 165 121 121 121 76 169 89 53 206 169 83 167 108 182 235 23 12 13 13 13 14 125 219 143 63 254 56 43 43 235 159 255 252 103 88 88 88 98 98 162 125 241 94 121 121 185 195 218 57 89 81 152 166 95 186 116 73 174 87 189 37 52 239 240 114 254 159 59 124 63 59 126 9 88 101 44 179 110 10 116 190 166 233 241 207 142 250 186 106 17 7 181 45 86 144 72 136 159 62 204 211 185 139 100 10 123 172 79 62 249 228 23 167 109 102 230 241 227 199 71 142 28 249 213 87 95 125 248 225 135 35 71 142 76 76 76 52 153 76 239 191 255 62 99 56 120 240 224 242 229 203 213 106 117 93 93 221 169 83 167 92 93 93 155 155 155 89 25 162 156 180 103 139 7 7 7 111 218 180 137 231 121 158 231 247 237 219 183 127 255 126 214 79 8 217 182 109 91 116 116 244 147 79 62 153 150 150 118 241 226 197 131 7 15 178 161 152 152 152 205 155 55 19 66 148 74 229 39 159 124 146 147 147 195 250 227 227 227 83 83 83 71 142 28 57 126 252 120 179 217 188 106 213 170 118 185 234 155 68 19 0 146 162 131 50 242 47 99 17 3 143 126 174 2 71 0 20 48 185 218 216 10 28 2 14 1 33 195 124 92 92 212 157 254 8 80 16 4 139 197 210 49 191 221 210 210 146 152 152 184 113 227 198 89 179 102 177 159 164 21 22 22 38 39 39 203 197 180 103 206 156 169 170 170 242 240 240 248 246 219 111 45 22 139 40 138 185 185 185 247 220 115 143 201 100 98 21 122 236 88 176 88 44 58 157 110 229 202 149 172 167 177 177 113 255 254 253 108 83 118 152 188 240 194 11 209 209 209 99 198 140 97 183 64 109 109 109 22 139 197 96 48 200 215 215 197 197 197 57 57 57 86 171 213 98 177 12 30 60 56 34 34 162 165 165 37 35 35 99 243 230 205 246 149 189 55 150 223 236 72 109 54 41 254 141 3 39 75 235 64 197 255 12 37 0 80 0 4 8 128 227 0 33 176 138 107 31 136 121 105 114 100 187 185 139 23 47 222 178 101 11 59 13 213 106 181 32 8 242 205 65 59 234 223 191 191 193 96 48 26 141 165 165 165 246 250 11 0 206 206 206 10 133 194 98 177 176 126 182 20 33 132 29 5 44 69 223 174 86 214 106 181 90 173 214 118 155 42 149 74 157 78 167 84 42 89 145 184 92 152 39 251 112 54 155 77 171 213 202 17 7 198 184 221 113 169 82 169 170 170 170 188 188 188 110 94 54 157 213 138 121 35 195 78 94 168 5 204 1 32 64 20 16 2 64 64 41 32 0 32 64 17 167 226 71 4 120 118 177 136 197 98 145 43 58 29 82 89 89 89 89 89 153 227 207 249 203 162 154 142 75 217 108 182 118 31 192 33 167 40 138 246 149 205 14 149 215 108 54 119 246 189 111 213 67 146 41 33 58 56 192 75 15 34 6 66 228 95 187 1 1 192 24 48 6 65 236 231 172 113 88 144 39 223 133 253 49 72 16 132 118 154 125 147 255 215 227 185 221 249 175 238 61 5 106 5 32 244 179 113 231 0 81 32 0 22 225 129 225 33 123 150 78 114 40 113 87 175 94 253 195 160 137 16 138 141 141 181 247 255 111 18 205 179 53 205 163 95 249 202 104 182 129 146 7 66 1 1 240 60 251 143 10 96 178 189 253 248 196 39 238 14 135 219 143 110 178 120 119 136 193 237 161 17 161 96 19 175 169 185 132 65 176 129 96 3 139 205 199 77 247 167 200 32 184 45 233 230 75 161 159 156 28 225 229 166 1 139 13 36 12 146 4 86 1 172 34 152 108 127 142 11 9 242 116 238 67 243 198 104 160 175 254 153 41 209 32 136 32 138 32 97 144 8 8 216 213 77 187 104 226 80 184 93 233 150 202 244 87 76 138 252 243 168 129 96 52 131 77 4 9 131 85 88 126 95 84 76 176 247 109 139 230 173 254 175 174 198 54 235 146 119 191 251 44 167 88 193 243 243 238 141 76 159 23 175 81 41 250 208 188 133 187 17 76 10 202 235 21 28 138 10 246 70 8 193 109 76 125 255 71 174 15 205 62 52 251 208 236 163 62 52 255 123 244 255 3 0 9 33 97 176 122 213 141 185 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
mainCss
	^ '/* general settings for all components */
body {
	margin: 0px;
	font: 11px Lucida Grande, Verdana, Arial, Helvetica, sans serif;
}

:link, :visited {
	color: #c00;
	text-decoration: none;
	border-bottom: 1px dotted #bbb;
}

:link:hover, :visited:hover {
	background-color: #eee;
	color: #555;
}

.content {
	width: 700px;
	max-width: 100%;
	margin: 5px;
}

fieldset {
	margin: 5px;
	color: #bbb;
	border: 1px solid #bbb;
	font-weight: bold;
}

fieldset ul li {
	color: #000;
	font-weight: normal;
}

/* links/anchors with images */
:link img, :visited img{
	color: #fff;
	background-color: #fff
	
	text-decoration: none;
	border: none;
	
	display: block;
}

:link:hover img, :visited:hover img{
	color: #09c;
	background-color: #09c
}

/* a div button */
.button {
	margin: 10px auto;
	background-color: #eee;
	border: 1px dotted #bbb;
	width: 100px;
	padding: 5px;
	text-align: center;
}

/* login menu */
#login_menu {
	text-align: right;
	color: #bbb;
	margin: 5px;
	padding: 5px;
}

#login_menu .text {
	color: #000;
}

/* main menu */
#main_menu {
	margin: .66em 5px;
	padding: .2em 0;
	border: 1px solid #000;
	background-color: #f7f7f7;
}

#main_menu ul {
	margin: 0;
	font-size: 10px;
	text-align: right;
	
	list-style: none;
}

#main_menu li {
	border-right: none;
	padding: .25em 0;
	
	display: inline;
	white-space: nowrap;
}

#main_menu :link, #main_menu :visited {
	color: #000;
	background-color: #fff;
	border-left: 1px solid #555;
	border-bottom: none;
	border-right: 1px solid #fff;
	padding: .2em 20px;
}

#main_menu :link:hover, #main_menu :visited:hover {
	background-color: #ccc;
	border-right: 1px solid #ddd;
}

#main_menu .active :link, #main_menu .active :visited {
	font-weight: bold;
	color: #eee;
	border-right: 1px solid #000;
	border-top: none;
	background-color: #333;
}

#main_menu .active :link:hover, #main_menu .active :visited:hover {
	border-right: 1px solid #000;
}

/* side menu (left) */
#tab_menu {
	border: 1px solid black;
	border-color: gray black black gray;
	margin: 2em 5px;
	padding: .5em 0 0;
	float: left;
	width: 12em;
}

#tab_menu ul { 
	list-style: none;
	margin: 0 0 .5em;
	padding: 0;
}

#tab_menu li {
	margin: 0;
	padding: 0.1em .5em;
	font-weight: bold;
}
#tab_menu li li { margin: 0 -0.5em; padding-left: 1.5em; font-weight: normal }
#tab_menu li li.active { background: #ccb; }
#tab_menu :link, .tabs :visited { border: none; display: block }
#tab_menu :link:hover, .tabs :visited:hover { background: transparent }

.content_with_menu {
	padding: 0.4em 2em;
	margin-left: 12em;
	min-height: 300px;
	width: 400px;
	max-width: 100%
}

/* detailed view */
#issue {
 background: #ffd;
 border: 1px outset #996;
 margin-top: 1em;
 padding: .5em 1em;
 position: relative;
}

#issue h2 {
	margin: 0 0 .8em 0;
}

#issue table.details {
 border-top: 1px solid #dd9;
 border-collapse: collapse;
 table-layout: fixed;
 width: 100%;
}

#issue table.details tr { 
	border-bottom: 1px dotted #eed;
}

#issue table.details td, #issue table.details th {
 font-size: 80%;
 padding: .5em 1em;
 vertical-align: top;
}

#issue table.details th {
 color: #663;
 font-weight: normal;
 text-align: left;
 width: 20%;
}

#issue table.details td {
	width: 30%;
}

#issue table.details .description {
	border-top: 1px solid #dd9;
}

#issue .description h3 {
 border-bottom: 1px solid #dd9;
 color: #663;
 font-size: 100%;
 font-weight: normal;
}

.comment {
	background: #ffc;
	padding: 1em;
	border: 1px outset #996;
	margin-bottom: 1em;
}

.comment h3 {
 border-bottom: 1px solid #d7d7d7;
 color: #999;
 font-size: 100%;
 font-weight: normal;
}

/* drag & droppable lists */

fieldset.drag_list {
	float: left;
	width: 170px;
}

fieldset.drag_list ul {
	min-height: 50px;
	color: #000;
	
	list-style: none;
	padding: 0;
	margin: 0;
}

fieldset.drag_list ul li {
	margin: 4px;
	padding: 2px;
	cursor: move;
	font-weight: normal;
	text-align: center;
}

fieldset.drag_list ul li.attached {
	background-color: #6c6;
  	border: 1px solid #060;
}
fieldset.drag_list ul li.available {
	background-color: #ddd;
  	border: 1px dashed #000;
}

/* list issues table */

.list_issues tbody td :link, .list_issues tbody td :visited {
	display: block
}

.list_issues {
	border-spacing: 0px;
}

.list_issues thead th {
	background-color: #eee;
	text-transform: capitalize;
	vertical-align: middle;
	
	border-top: 1px solid #ddd;
	border-left: 1px solid #ddd;
	border-bottom: 1px solid #5F9EA0;
	
	padding: .3em 1em;
}

.list_issues tbody tr {
	text-align: center;
}

.list_issues tbody td {
	padding: .1em .5em;
	border-bottom: 1px solid #5F9EA0;
	border-left: 1px solid #5F9EA0;
}

.list_issues tbody td :link, .list_issues tbody td :visited {
	display: block;
	font-weight: bold;
	border: none;
	background: none;
}

.list_issues tbody tr:hover {
	background: #ff0;
}

.list_issues tr.odd {
	background-color: #B0E0E6;
}

.list_issues tr.even {
	background-color: #ADD8E6;
}


/*
#main_menu a.active {
	color: #fff;
	background-color: #000;
}

#main_menu a:hover {
	color: #fff;
	background-color: #000;
}
*/
/* a simple button */
/* a.button, a.button:hover {
	text-decoration: none;		
	color: #000
}
	
a.button:hover {
	color: #fff;
}
	
span.button, span.button:hover {
		margin: 5px;
		padding: 5px;
		font-size: 10px;
		font-family: Verdana,Arial;
		border-width: 1px;
		border-style: none, none, solid, none;
		border-color: #000;
		text-align: center;
		background-color: #fff;		
	}
	
div span.button:hover {
	background-color: #000;
}*/

/* older stuff */

	
	div.security_container {
		padding: 10px;
		border-width: 1px;
		border-style: solid;
		width: 500px;
	}
	
	div.security_menu {
		margin: 0px;
	}
	'! !

!ITFileLibrary methodsFor: 'accessing' stamp: 'afk 1/8/2008 22:20'!
selectorsToInclude
	"Appends a unique style to all used components."
	
	"^#(#mainCss)."
	^ #(#hpidesignCss).! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
tagbluePng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 220 73 68 65 84 56 203 133 147 217 138 26 65 20 134 235 37 230 54 193 183 16 154 60 93 32 100 146 76 46 51 67 18 242 0 174 40 238 40 40 42 110 211 61 110 61 46 136 152 142 162 198 220 53 169 102 108 123 254 212 41 177 29 113 201 197 185 169 58 255 247 255 167 250 52 211 117 29 221 110 23 237 118 27 173 86 11 205 102 19 154 166 65 85 85 52 26 141 43 0 236 82 177 78 167 3 199 113 142 106 58 157 74 72 181 90 189 8 97 228 76 2 195 48 164 243 100 50 113 33 116 38 82 160 84 42 157 133 48 138 76 205 228 70 7 245 122 93 53 77 19 182 109 203 115 2 86 42 21 228 243 249 147 16 70 174 187 70 33 126 18 145 13 203 178 176 48 109 228 126 109 147 140 199 99 20 139 69 228 114 185 35 8 35 231 93 228 205 102 3 18 255 254 235 224 195 195 51 222 222 3 89 99 123 55 26 141 40 5 210 233 244 1 132 137 25 249 106 181 146 77 235 245 26 115 225 124 173 61 227 75 23 248 170 3 31 31 246 144 225 112 72 41 16 143 199 93 8 171 213 106 111 196 140 124 185 92 202 166 206 202 150 206 119 2 240 163 7 124 123 60 132 12 6 3 74 129 72 36 34 33 146 82 46 151 21 49 35 95 44 22 178 233 241 143 141 247 170 131 59 125 15 185 214 128 204 207 45 164 215 235 81 10 132 66 161 43 119 150 66 161 160 136 25 249 124 62 63 130 124 23 128 207 77 224 230 222 130 189 217 166 72 38 147 8 4 2 175 15 94 84 204 167 100 179 89 62 155 205 92 200 59 1 249 36 70 184 109 173 97 90 79 47 197 30 119 132 151 149 201 100 148 84 42 197 105 19 9 162 11 200 109 219 129 101 239 157 253 126 191 199 125 196 83 203 145 72 36 148 88 44 198 105 19 119 159 152 196 226 252 64 124 22 64 21 141 70 189 226 165 57 45 88 191 223 63 41 190 8 160 10 135 195 222 96 48 200 73 236 243 249 60 39 87 249 127 191 171 112 245 10 241 171 115 247 255 0 90 204 58 236 180 244 123 41 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
userPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 119 73 68 65 84 56 203 165 147 109 72 211 81 20 135 13 42 233 197 214 182 86 4 194 106 41 164 162 95 212 79 213 208 165 178 57 93 3 141 172 149 109 42 230 140 89 138 225 235 150 169 101 45 172 168 104 150 182 141 185 181 45 205 208 81 154 74 22 105 154 76 203 26 70 159 164 50 8 193 133 228 166 76 189 191 254 74 140 4 153 69 31 30 56 92 206 239 57 247 114 184 126 0 252 254 135 21 15 115 163 183 41 107 249 140 233 27 194 173 80 243 105 80 242 232 46 73 36 171 250 175 4 167 34 25 69 119 68 116 82 159 198 192 93 138 91 41 116 212 36 210 144 207 221 66 132 17 204 11 171 10 52 66 26 116 84 80 77 133 106 132 139 208 113 49 137 133 115 177 12 28 143 164 77 173 42 168 225 110 70 135 166 28 227 237 106 12 52 228 35 47 134 133 130 184 157 56 29 207 65 246 62 186 199 80 156 236 239 83 80 159 206 158 156 249 254 9 83 125 26 124 235 186 134 17 235 121 140 61 185 2 231 216 16 204 57 123 166 222 235 115 215 251 20 220 148 132 246 58 7 31 96 194 222 4 247 59 51 133 101 169 118 14 153 113 47 35 220 238 243 9 59 132 166 117 193 251 115 13 102 69 20 102 237 13 152 113 60 94 194 253 214 132 150 194 40 132 115 115 30 177 226 235 55 172 40 96 37 154 253 153 124 163 60 144 223 13 199 151 105 88 203 68 176 148 37 47 177 88 191 26 253 137 192 132 231 96 240 238 159 97 242 234 54 46 19 48 5 214 53 1 9 86 41 59 169 23 163 227 110 24 199 129 2 7 32 127 3 100 190 0 78 62 5 42 94 2 175 63 186 193 22 244 97 19 87 43 167 197 106 214 122 5 97 41 90 113 92 170 26 77 3 243 168 27 1 20 253 4 57 61 4 89 207 8 210 91 9 142 90 8 82 245 11 40 109 167 68 86 2 158 248 50 194 146 174 74 189 130 225 242 189 243 19 182 98 220 214 218 32 165 154 50 169 137 178 54 130 19 205 84 216 68 144 162 93 128 72 67 112 176 22 168 188 222 141 201 14 21 250 11 67 60 94 193 136 50 132 56 59 43 145 150 173 67 117 151 11 146 102 224 24 53 245 136 129 10 55 16 28 110 4 196 58 160 184 101 22 137 233 70 252 232 174 194 96 81 40 241 10 62 24 20 37 67 37 193 115 50 217 165 185 104 105 15 100 212 213 51 254 64 246 155 32 65 43 20 89 165 174 97 85 196 44 149 169 90 182 133 49 139 60 64 85 90 205 220 37 182 213 177 99 244 95 57 241 15 93 156 132 102 15 39 174 201 179 59 214 236 102 31 208 77 4 29 106 235 60 155 87 177 253 179 85 30 224 243 55 254 11 191 0 70 223 169 245 178 218 220 44 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
usereditPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 211 73 68 65 84 56 203 165 147 125 44 20 96 28 199 245 71 243 79 186 142 212 106 109 26 105 94 198 218 228 175 98 72 186 115 136 97 89 151 183 152 156 58 139 209 113 156 200 251 69 67 94 35 54 47 157 75 75 185 114 134 92 53 118 200 201 189 36 173 116 136 49 229 188 228 238 188 222 183 235 214 108 214 70 91 127 124 182 103 207 246 249 236 217 243 60 63 61 0 122 255 195 95 27 81 246 7 89 249 36 195 229 2 202 1 176 73 4 176 92 136 74 170 157 113 198 63 5 174 217 25 50 74 189 136 154 202 0 67 84 104 185 239 75 68 182 59 1 49 142 251 53 20 91 163 180 93 3 101 20 2 106 180 34 91 43 101 83 126 67 68 166 135 49 226 157 13 113 197 142 176 184 107 32 219 113 31 218 202 146 49 201 103 163 183 42 6 209 78 198 136 117 61 130 235 231 77 17 113 134 184 86 155 224 169 191 99 160 50 200 100 78 61 243 9 139 61 101 152 234 184 7 49 247 54 228 47 115 161 144 139 192 137 52 91 28 109 182 191 52 210 16 128 207 207 51 49 244 208 13 226 18 235 160 109 129 34 170 85 183 162 255 17 102 7 154 160 26 226 104 105 212 173 21 34 14 248 25 22 95 167 187 24 88 144 188 0 126 78 97 94 242 4 194 60 210 204 150 124 152 210 176 215 252 108 84 45 135 126 26 43 3 85 80 203 154 117 168 222 55 64 192 182 197 119 105 202 230 234 183 22 76 191 190 11 149 244 41 54 198 186 33 173 137 80 235 100 99 119 142 190 17 169 158 118 140 212 9 217 196 50 184 73 94 104 76 242 212 209 158 231 2 133 140 133 77 85 55 84 163 55 48 215 79 199 199 186 48 72 42 3 127 244 23 249 156 212 51 34 115 247 24 184 113 67 76 60 186 49 60 169 66 253 36 16 43 3 104 125 64 86 135 0 10 105 34 54 213 66 40 191 4 67 61 65 197 210 48 19 242 10 167 25 81 190 235 41 221 37 90 251 86 123 187 250 177 209 212 187 129 114 49 64 23 106 16 41 208 32 149 243 12 211 239 24 127 228 32 168 199 47 99 81 150 4 89 161 27 98 35 34 83 183 94 97 48 217 98 99 150 151 128 226 106 30 66 248 64 88 43 192 170 227 65 218 118 75 123 236 183 80 203 195 176 50 17 136 5 237 73 70 74 46 96 182 37 30 194 56 203 181 173 128 152 101 169 81 180 223 65 64 68 13 50 58 148 136 43 226 224 67 43 19 98 126 1 38 251 252 161 148 83 49 55 148 0 73 169 15 2 195 75 49 223 153 142 126 134 149 102 43 32 173 165 39 138 18 205 215 67 67 179 214 237 67 4 120 156 123 14 88 26 195 200 3 63 116 49 143 162 167 156 12 126 14 25 46 126 85 160 135 51 149 131 41 182 43 90 39 125 219 71 146 55 210 12 82 152 25 70 199 189 121 229 105 225 54 88 21 22 99 181 175 16 125 153 206 168 191 106 6 7 114 174 226 196 197 150 246 155 209 169 135 198 185 52 131 29 167 209 223 129 168 206 14 182 88 127 149 67 94 239 204 119 121 195 99 216 152 238 52 206 191 0 229 209 183 51 123 199 52 119 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

TestCase subclass: #ITTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTest class methodsFor: 'testing' stamp: 'mig 1/12/2008 12:37'!
isAbstract
	"ITTest is the test base class without any tests"
	
	^self name = #ITTest! !

!ITTest methodsFor: 'private' stamp: 'mig 1/13/2008 22:07'!
checkAccessor: aSelector on: aTargetObject with: anObject
	"test the getter/setter aSelector of aTargetObject with anObject"
	
	aTargetObject perform: (aSelector numArgs: 1) with: anObject.
	self 
		assert: (aTargetObject perform: aSelector) == anObject
		description: 'The getter/setter pair ''#', aSelector asString, ''' does not return the object just set before.'! !

ITTest subclass: #ITTestCondition
	instanceVariableNames: 'condition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestCondition commentStamp: 'afk 1/20/2008 17:54' prior: 0!
I'm testing basic functionality of the ITCondition class.!

!ITTestCondition methodsFor: 'running' stamp: 'afk 1/20/2008 17:56'!
setUp

	super setUp.
	condition := ITCondition new.! !

!ITTestCondition methodsFor: 'running' stamp: 'afk 1/20/2008 17:59'!
tearDown
	super tearDown.
	condition := nil.! !

!ITTestCondition methodsFor: 'tests' stamp: 'afk 1/20/2008 17:56'!
testCondtionArguments
	"Tests whether the accessors for #tagDescription do what they should do - set and get."
	
	self checkAccessor: #arguments on: condition with: 'tmp_testConditionArguments'.! !

!ITTestCondition methodsFor: 'tests' stamp: 'afk 1/20/2008 17:57'!
testCondtionOperation
	"Tests whether the accessors for #tagDescription do what they should do - set and get."
	
	self checkAccessor: #operation on: condition with: 'tmp_testConditionOperation'.! !

!ITTestCondition methodsFor: 'tests' stamp: 'afk 1/20/2008 17:57'!
testCondtionTag
	"Tests whether the accessors for #tagDescription do what they should do - set and get."
	
	self checkAccessor: #tag on: condition with: 'tmp_testConditionTag'.! !

ITTest subclass: #ITTestFilter
	instanceVariableNames: 'filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestFilter commentStamp: 'afk 1/20/2008 17:54' prior: 0!
I'm testing basic functionality of the ITFilter class.!

!ITTestFilter methodsFor: 'running' stamp: 'afk 1/20/2008 17:55'!
setUp

	super setUp.
	filter := ITFilter new.! !

ITTest subclass: #ITTestGlorp
	instanceVariableNames: 'session user comment issue detail role tag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestGlorp methodsFor: 'testing' stamp: 'mt 1/25/2008 16:46'!
setUp

	super setUp.
	session := ITGlorpSession getInstance.
	
	user := (ITUser new
		username: 'tmp_ITTestGlorp_username';
		yourself).
		
	tag := (ITTag new
		tagName: 'title';
		tagDescription: 'tmp_ITTestGlorp';
		yourself).
		
	comment := (ITIssueComment new
		content: 'tmp_ITTestGlorp_comment';
		user: user;
		yourself).
		
	role := (ITRole new
		rolename: 'tmp_ITTestGlorp_rolename';
		yourself).
		
	detail := (ITIssueDetail new
		tag: tag;
		content: 'tmp_ITTestGlorp_title';
		yourself).
		
	issue := (ITIssue new
		setDetail: detail;
		yourself).
	
	comment user: user.
	issue addComment: comment.
	! !

!ITTestGlorp methodsFor: 'testing' stamp: 'mt 1/25/2008 17:00'!
testUserIdentity
	"Tests whether the user objects at comments and roles are unique."
	
	session inUnitOfWorkDo: [
		session
			register: tag;
			register: role;
			register: user;
			register: issue.

		role addUser: user.
		user roles: {role}.
	].	
	
	session logout. "Clear cache manager."
	session := ITGlorpSession getInstance.
	
	user := session readOneOf: ITUser where: [:each | each id = user id].
	issue := session readOneOf: ITIssue where: [:each | each id = issue id].
	role := session readOneOf: ITRole where: [:each | each id = role id].
	tag := session readOneOf: ITTag where: [:each | each id = tag id].
	
	"If one of the following assertions fails then you have 2 dummy issues, 2 dummy roles and 2 dummy users in the database. I don't know why."
	
	self
		assert: user == issue comments first user
		description: 'Object identity is not given in ITIssueComment>>user.'.
		
	self
		assert: user == role users anyOne
		description: 'Object identity is not given in ITRole>>users.'.
		
	self
		assert: tag == (issue getDetail: tag) tag
		description: 'Object identity is not given in ITIssueDetail>>tag'.
	
	"Clean up"	
	session delete: issue.
	session inUnitOfWorkDo: [
		session register: role.
		role users: Set new.
	].
	session delete: role.
	session delete: user.
	session delete: tag.! !

ITTest subclass: #ITTestGlorpDB
	instanceVariableNames: 'login accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestGlorpDB methodsFor: 'testing' stamp: 'afk 12/13/2007 14:52'!
setUp
	
	super setUp.
	login := ITGlorpDB login.
	accessor := DatabaseAccessor forLogin: login.! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 1/18/2008 20:19'!
testConnect
	"Tests whether ITGlorpDB>>#connect answers with a valid GlorpSession which could be used for operating with the SQL database."
	
	| session |
	session := ITGlorpDB connect.
	
	self
		assert: (login username = session currentLogin username)
		description: 'Username of session does not equal login username'.
		
	self
		assert: (login connectString = session currentLogin connectString)
		description: 'ConnectString of session does not equal login ConnectString'.
		
	self 
		assert: (login password = session currentLogin password) 
		description: 'Password of session does not equal login password '.
	
	self 
		assert: session isLoggedIn
		description: 'You are not logged in. Maybe your login data is corrupt.'! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 1/18/2008 20:21'!
testLogin
	"Tests whether it is possible to use the default login for connecting to a vaild SQL database which is used in the whole application."
	
	self
		assert: accessor isLoggedIn not
		description: 'A new database accessor should not be logged in.'.
		
	accessor loginIfError: [:exception |
		self
			assert: false
			description: 'Login failed.'
	].
		
	self
		assert: accessor isLoggedIn
		description: 'You should be logged in.'.
		
	accessor logout.
	
	self
		deny: accessor isLoggedIn
		description: 'You should be logged out after the accessor logged out.'! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 1/18/2008 20:22'!
testUniqueSession
	"Tests whether a glorp session really is a singleton."

	self
		assert: ITGlorpSession getInstance notNil
		description: 'Glorp session is nil.'.
		
	self
		assert: (ITGlorpSession getInstance == ITGlorpSession getInstance)
		description: 'Glorp session is not unique.'.! !

ITTest subclass: #ITTestIssue
	instanceVariableNames: 'issue tag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestIssue commentStamp: 'mt 1/18/2008 19:09' prior: 0!
I'm testing the basic functionality of an issue object.!

!ITTestIssue methodsFor: 'testing' stamp: 'mt 1/24/2008 17:29'!
setUp

	super setUp.
	issue := ITIssue new.
	tag := ITTag new
		tagName: 'title';
		yourself.! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 1/18/2008 19:13'!
testAddComment
	"Tests whether it is possible to add a new ITIssueComment instance to an issue and retrieve this comment."
	
	| comment |
	
	comment := (ITIssueComment new
		content: 'tmp_testAddIssue';
		user: ITUser new;
		yourself).

	issue addComment: comment.
	
	self 
		assert: issue comments last == comment
		description: 'A just added comment could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mig 1/13/2008 22:08'!
testComments
	"Tests whether the accessors for #comments do what they should do - set and get."
	
	self checkAccessor: #comments on: issue with: OrderedCollection new
! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 1/24/2008 17:31'!
testDeleteDetail
	"Tests whether it is possible to delete a detail from an issue."
	
	|detail|
	detail := (ITIssueDetail new
		tag: tag;
		content: 'tmp_testDeleteDetail_title';
		yourself).
		
	issue setDetail: detail.
	issue deleteDetail: detail tag.
	
	self 
		assert: (issue getDetail: tag) isNil
		description: 'A detail could not be deleted from an issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mig 1/13/2008 22:08'!
testDetails
	"Tests whether the accessors for #details do what they should do - set and get."
	
	self checkAccessor: #details on: issue with: OrderedCollection new! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 1/24/2008 17:31'!
testGetDetail
	"Tests whether it is possible to retrieve a detail which was just added to an issue. Does not make use of #setDetail: due to better testing."
	
	| detail |
	detail := (ITIssueDetail new
		tag: tag;
		content: 'tmp_testGetDetail_title';
		yourself).
		
	issue details: detail asOrderedCollection.
	
	self 
		assert: (issue getDetail: tag) == detail 
		description: 'A just added detail could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mig 1/13/2008 22:08'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	self checkAccessor: #id on: issue with: 1! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 1/24/2008 17:32'!
testSetDetail
	"Tests whether it is possible to add a new detail to an issue."
	
	issue setDetail: (ITIssueDetail new
		tag: tag;
		content: 'tmp_testSetDetail_title';
		yourself).
	
	self 
		assert: (issue getDetail: tag) content = 'tmp_testSetDetail_title' 
		description: 'A new detail was set and could not be retrieved from the issue.'.
		
	issue setDetail: (ITIssueDetail new
		tag: tag;
		content: 'tmp_testSetDetail_titleModified';
		yourself).
		
	self 
		assert: (issue getDetail: tag) content = 'tmp_testSetDetail_titleModified' 
		description: 'The content of a present detail could not be modified.'.! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 1/24/2008 17:32'!
testSetDetailWithTo
	"Tests whether it is possible to add a new detail to an issue."
	
	issue
		setDetailWith: tag
		to: 'tmp_testSetDetailWithTo_title'.
	
	self 
		assert: (issue getDetail: tag) content = 'tmp_testSetDetailWithTo_title' 
		description: 'A new detail was set and could not be retrieved from the issue.'! !

ITTest subclass: #ITTestIssueComment
	instanceVariableNames: 'comment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestIssueComment commentStamp: 'mt 1/18/2008 20:14' prior: 0!
I'm testing the basic functionality of comments object which could be attached to an issue object.!

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 12/30/2007 11:39'!
setUp

	super setUp.
	comment := ITIssueComment new.! !

!ITTestIssueComment methodsFor: 'testing' stamp: 'mig 1/13/2008 22:11'!
testContent
	"Tests whether the accessors for #content do what they should do - set and get."
	
	self checkAccessor: #content on: comment with: 'tmp_testContent'
	! !

!ITTestIssueComment methodsFor: 'testing' stamp: 'mig 1/13/2008 22:09'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	self checkAccessor: #id on: comment with: 1! !

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 1/18/2008 19:45'!
testInitialize
	"Tests whether a new comment object has a (current) timestamp set."
	
	self
		assert: (comment timestamp isKindOf: TimeStamp)
		description: 'A new comment object does not have a timestamp set!!'.
		
	self
		assert: (TimeStamp current - comment timestamp) seconds < 10
		description: 'The difference between the timestamp of a new created comment and the current time is more than 10 seconds.'! !

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 1/18/2008 19:35'!
testTimestamp
	"Tests whether the accessors for #timestamp do what they should do - set and get."
	
	self checkAccessor: #timestamp on: comment with: TimeStamp current
	! !

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 1/18/2008 19:36'!
testUser
	"Tests whether the accessors for #user do what they should do - set and get."
	
	self checkAccessor: #user on: comment with: ITUser new
	! !

ITTest subclass: #ITTestIssueDetail
	instanceVariableNames: 'detail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestIssueDetail commentStamp: 'mt 1/18/2008 20:14' prior: 0!
I'm testing the basic functionality of details which can be added to issues.!

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/30/2007 11:39'!
setUp

	super setUp.
	detail := ITIssueDetail new.! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mig 1/13/2008 22:13'!
testContent
	"Tests whether the accessors for #content do what they should do - set and get."

	self checkAccessor: #content on: detail with: 'tmp_testContent'! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mig 1/13/2008 22:14'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	self checkAccessor: #id on: detail with: 1! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mfp 1/22/2008 15:34'!
testTag
	"Tests whether the accessors for #tag do what they should do - set and get."

	self checkAccessor: #tag on: detail with: (ITTag new).! !

ITTest subclass: #ITTestIssueManager
	instanceVariableNames: 'issueManager securityManager testUser testIssue testComment testTag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestIssueManager methodsFor: 'running' stamp: 'mt 1/24/2008 18:06'!
setUp
	"Creates necessary test-data for the test-issue. You must not modify that test-data of the issue!! Create local tags and comments to do further testing on the issue."
	
	super setUp.
	
	issueManager := ITIssueManager new.
	securityManager := ITSecurityManager new.
	
	testTag := ITTag new
		tagName: 'tmp_ITTestIssueManager_title';
		yourself.
	
	testUser := ITUser new 
		username: 'tmp_ITTestIssueManager';
		yourself.
	
	[securityManager addUser: testUser] 
		on: ITUserAlreadyExists 
		do: [testUser := securityManager getUser: testUser username].
	
	testIssue := (ITIssue new
		setDetail: (ITIssueDetail new 
			tag: testTag;
			content: 'tmp_ITTestIssueManager_testtitle';
			yourself);
		yourself).
		
	testComment := (ITIssueComment new
		content: 'tmp_ITTestIssueManager_initial';
		user: testUser;
		yourself).
	
	testIssue addComment: testComment.	
	issueManager addTag: testTag.
	issueManager addIssue: testIssue.
	! !

!ITTestIssueManager methodsFor: 'running' stamp: 'mt 1/24/2008 17:39'!
tearDown

	super tearDown.
	
	issueManager deleteIssue: testIssue.
	issueManager deleteTag: testTag.
	securityManager deleteUser: testUser.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/24/2008 17:40'!
testAddIssue
	"Tests whether it is possible to add an issue through the issue manager. (see setUp)"
		
	self
		deny: (testIssue id) isNil
		description: 'You tried to add an issue through the issue manager but that issue got no ID.'.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/24/2008 17:41'!
testAddTag
	"Tests whether it is possible to add a tag through the issue manager."
	
	self
		deny: (testTag id) isNil
		description: 'You tried to add a new tag through the issue manager but that tag got no ID.'.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/24/2008 17:42'!
testDeleteIssue
	"Tests whether it is possible to remove an issue with a given ID."
	
	issueManager deleteIssue: testIssue.
	
	self
		assert: (issueManager getIssueWithID: testIssue id) isNil
		description: 'A new issue was just added and could not be deleted through an issue manager.'! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/24/2008 17:51'!
testDeleteTag
	"Tests whether it is possible to delete a tag through the issue manager."
	
	| tag |
	tag := ITTag new
		tagName: 'tmp_testDeleteTag';
		yourself.
	
	issueManager addTag: tag.
	issueManager deleteTag: tag.
	
	self
		deny: (issueManager getTags anySatisfy: [:each | each == tag])
		description: 'ITIssueManager>>#deleteTag: the tag has not been deleted successfully'.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/24/2008 17:44'!
testGetIssueWithID
	"Tests whether it is possible to retrieve an issue by its ID from the issue manager."
	
	self
		assert: (issueManager getIssueWithID: testIssue id) == testIssue
		description: 'A new issue was just set and could not be retrieved through an issue manager.'.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/24/2008 17:45'!
testGetIssues
	"Tests whether the issue manager returns only objects with the type ITIssue which all have an ID and at least one comment."
	
	issueManager getIssues do: [ :issue |
		self 
			assert: (issue isKindOf: ITIssue)
			description: 'ITIssueManager>>#getIssues returns a collection in where not all objects are issues.'.
		
		self
			deny: (issue id) isNil
			description: 'ITIssueManager>>#getIssues returns a collection in where not all issues have an ID.'.
			
		self
			deny: issue comments isEmpty
			description: 'One issue found with no comments. Every Issue needs at least one comment.'
	].
	
	
	! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mfp 1/19/2008 16:33'!
testGetTags
	"Tests whether the issue manager returns only objects with the type ITTag which all have an ID."
	
	issueManager getTags do: [ :tag |
		self 
			assert: (tag isKindOf: ITTag)
			description: 'ITIssueManager>>#getTags returns a collection in where not all objects are tags.'.
		
		self
			deny: (tag id) isNil
			description: 'ITIssueManager>>#getTags returns a collection in where not all tags have an ID.'.
	].
	
	
	! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mfp 1/19/2008 14:20'!
testUpdateIssueAddComment
	"Tests whether it is possible to add a comment to an issue through the issue manager to the database."

	| comment |

	comment := ITIssueComment new
		content: 'tmp_testUpdateIssueAddComment';
		user: testUser;
		yourself.
		
	issueManager updateIssue: testIssue addComment: comment.

	self 
		assert: (issueManager getIssueWithID: testIssue id) comments last == comment
		description: 'A comment to a new issue was just set and could not be retrieved through an issue manager.'.
! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/24/2008 17:54'!
testUpdateIssueDeleteDetail
	"Tests whether it is possible to delete a detail from an issue through the issue manager to the database."

	| detail tag |
	
	tag := ITTag new
		tagName: 'tmp_testUpdateIssueDeleteDetail';
		yourself.

	detail := ITIssueDetail new
		tag:		tag;
		content: 'tmp_testUpdateIssueDeleteDetail';
		yourself.
	
	issueManager addTag: tag.	
	issueManager updateIssue: testIssue setDetail: detail.
	issueManager updateIssue: testIssue deleteDetail: detail.
	issueManager deleteTag: tag.

	self 
		deny: ((issueManager getIssueWithID: testIssue id) details 
			anySatisfy: [:each | each == detail])
		description: 'A detail could not be deleted succesfully from an Issue'.
! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/24/2008 18:04'!
testUpdateIssueSetDetail
	"Tests whether it is possible to set/modifiy a detail through the issue manager."
		
	| detail anotherDetail tag result |
	
	tag := ITTag new
		tagName: 'tmp_testUpdateIssueSetDetail';
		yourself.

	detail := ITIssueDetail new
		tag:		tag;
		content: 'tmp_testUpdateIssueSetDetail';
		yourself.
	
	issueManager addTag: tag.
	issueManager updateIssue: testIssue setDetail: detail.
	
	self
		deny: (detail id) isNil
		description: 'ITIssueManager>>#updateIssue:setDetail: returns an ITIssueDetail object which has no ID.'.
		
	anotherDetail := ITIssueDetail new
		tag:		tag;
		content: 'tmp_testUpdateIssueSetDetail_Modified';
		yourself.
		
	issueManager updateIssue: testIssue setDetail: anotherDetail.
	
	result := (issueManager getIssueWithID: testIssue id) details
		detect: [:each | each content = anotherDetail content]
		ifNone: [].
	
	self
		deny: result isNil
		description: 'ITIssueManager>>#updateIssue:setDetail: an existing detail could not be changed successfully.'.
		
	self
		assert: result == detail
		description: 'Object indentity is inconsistent!!'.
		
	issueManager updateIssue: testIssue deleteDetail: detail.
	issueManager deleteTag: tag.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/24/2008 18:08'!
testUpdateTagDescription
	"Tests whether it is possible to update a tags description through the issue manager."
		
	| description |
	description := 'tmp_testUpdateTagDescription'.
	
	issueManager updateTag: testTag description: description.
	
	self
		assert: (testTag tagDescription = description)
		description: 'ITIssueManager>>#updateTag:description:: the tag has not been renamed successfully'.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/24/2008 18:09'!
testUpdateTagName
	"Tests whether it is possible to update a tags name through the issue manager."
		
	| tag |
	
	tag := ITTag new
		tagName: 'tmp_testUpdateTagName';
		yourself.
		
	issueManager addTag: tag.
	issueManager updateTag: tag name: 'tmp_testUpdateTagName_testTagRenamed'.
	
	self
		assert: (tag tagName = 'tmp_testUpdateTagName_testTagRenamed')
		description: 'ITIssueManager>>#updateTag:name:: the tag has not been renamed successfully'.
		
	issueManager deleteTag: tag.! !

ITTest subclass: #ITTestRole
	instanceVariableNames: 'role'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestRole commentStamp: 'mt 1/18/2008 20:14' prior: 0!
I'm testing the basic functionality of roles which can be attached to users.!

!ITTestRole methodsFor: 'testing' stamp: 'mt 12/30/2007 11:39'!
setUp

	super setUp.
	role := ITRole new! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 1/18/2008 20:11'!
testAddUser
	"Tests whether it is possible to attach a user to a role."
	
	|user|
	user := ITUser new.
	
	role addUser: user.
	
	self
		assert: (role users includes: user)
		description: 'A user was just attached to a role and could not be retrieved.'! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 1/18/2008 20:12'!
testDeleteUser
	"Tests whether it is possible to detach a user from a role."
	
	|user|
	user := ITUser new.
	
	role users: {user}.
	role deleteUser: user.
	
	self
		assert: role users isEmpty
		description: 'A user could not be detached from a role.'! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 1/18/2008 20:13'!
testHasPrivilege
	"Tests whether it is possible to set a privilege to a role and ask this role whether it has this privilege."
	
	role privileges: #(read write).
		
	self
		assert: (role hasPrivilege: #write)
		description: 'A privilege was just added and could not be retrieved from a role.'! !

!ITTestRole methodsFor: 'testing' stamp: 'mig 1/13/2008 22:15'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	self checkAccessor: #id on: role with: 1! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 1/18/2008 19:56'!
testInitialize
	"Tests whether a new role has an empty set of users and privileges attached."
	
	self
		assert: (role users isKindOf: Set)
		description: 'A new role object does not have an empty set of users.'.
		
	self
		assert: (role privileges isKindOf: Set)
		description: 'A new role object does not have an empty set of privileges.'! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 12/30/2007 11:23'!
testPrivileges
	"Tests whether the accessors for #privileges do what they should do - set and get."

	| somePrivileges |
	somePrivileges := #('read' 'read' 'write' 'write').
	role privileges: somePrivileges.
	
	self
		assert: (role privileges isKindOf: Set) 
		description: 'The privileges of a role are not a set.'.
		
	self
		assert: role privileges = #('read' 'write') asSet
		description: 'Some privileges where just set to a role and could not be retrieved.' ! !

!ITTestRole methodsFor: 'testing' stamp: 'mig 1/13/2008 22:15'!
testRolename
	"Tests whether the accessors for #rolename do what they should do - set and get."
	
	self checkAccessor: #rolename on: role with: 'tmp_testRolename'
! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 1/18/2008 20:01'!
testUsers
	"Tests whether the accessors for #users do what they should do - set and get."
	
	self checkAccessor: #users on: role with: {ITUser new} asSet! !

ITTest subclass: #ITTestSecurityManager
	instanceVariableNames: 'securityManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 11:58'!
setUp

	super setUp.
	securityManager := ITSecurityManager new.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mfp 1/19/2008 15:36'!
testAddRole
	"Tests whether it is possible to add a new role to the database."
	
	|role|
	role := ITRole new.
	role rolename: 'tmp_testAddRole'.
	
	self 
		assert: (securityManager addRole: role) == role
		description: 'addRole does not return the just added object.'.
	
	self
		deny: role id isNil 
		description: 'New role could not be added to database through security manager because it got no ID.'.
		
	securityManager deleteRole: role.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mfp 1/19/2008 15:37'!
testAddUser
	"Tests whether it is possible to add a new user to the database."
	
	|user|
	user := ITUser new.
	user username: 'tmp_testAddUser'.
	
	self
		assert: (securityManager addUser: user) == user
		description: 'addUser does not return the just added object.'.
	
	self
		deny: user id isNil 
		description: 'New user could not be added to database through security manager because it got no ID.'.
		
	securityManager deleteUser: user.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mig 1/9/2008 15:26'!
testDefaultUser
	"tests whether the default user is an ITUser object and has the correct name"

	self 
		assert: (securityManager defaultUser isKindOf: ITUser)
		description: 'Returned default user is not an ITUser object.'.
		
	self 
		assert: securityManager defaultUser username = 'Guest'
		description: 'Default username should be "Guest".'.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mfp 1/19/2008 16:41'!
testDeleteRole
	"Tests whether it is possible to remove a role from the database."
	
	|role|
	role := ITRole new.
	role rolename: 'tmp_testDeleteRoleWithID'.
	securityManager 
		addRole: role;
		deleteRole: role.
		
	self
		deny: (securityManager getRoles anySatisfy: [:each | each == role])
		description: 'A role could not be deleted through a security manager.'
	! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 1/24/2008 18:13'!
testDeleteUser
	"Tests whether it is possible to remove a user from the database."
	
	|user|
	user := ITUser new.
	user username: 'tmp_testDeleteUserWithID'.
	securityManager 
		addUser: user;
		deleteUser: user.
		
	self
		deny: (securityManager getUsers anySatisfy: [:each | each == user])
		description: 'A user could not be deleted through a security manager.'
	! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 1/24/2008 18:19'!
testGetRandomRole
	"Tests whether it is possible retrieve a random role object."
	
	|role|
	role := ITRole new
		rolename: 'tmp_testGetRandomRole';
		yourself.
		
	securityManager addRole: role.
		
	self
		assert: (securityManager getRandomRole isKindOf: ITRole)
		description: 'A random role could not be retrieved through the security manager.'.
		
	securityManager deleteRole: role.
	! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 1/24/2008 18:16'!
testGetRoles
	"Tests whether the security manager returns only objects with the type ITRole which all have an ID."
	
	|tmpRole|
	tmpRole := ITRole new
		rolename: 'tmp_testGetRoles';
		yourself.
		
	securityManager addRole: tmpRole.
	
	securityManager getRoles do: [ :role |
		self 
			assert: (role isKindOf: ITRole)
			description: '#getRoles returns a collection in where not all objects are roles.'.
		
		self
			deny: (role id) isNil
			description: '#getRole returns a collection in where not all roles have an ID.'.
	].

	securityManager deleteRole: tmpRole.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 1/24/2008 18:16'!
testGetUser
	"Tests whether it is possible to retrieve a user instance with a given username from the database."
	
	|user|
	user := ITUser new
		username: 'tmp_testGetUser';
		yourself.
	
	securityManager addUser: user.
	
	self
		assert: ((securityManager getUser: user username) isKindOf: ITUser)
		description: 'A new user was just added to the database through a security manager and could not be retrieved by its name.'.
	
	securityManager deleteUser: user.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 1/24/2008 18:15'!
testGetUsers
	"Tests whether the security manager returns only objects with the type ITUser which all have an ID."
	
	|tmpUser|
	tmpUser := ITUser new
		username: 'tmp_testGetUsers';
		yourself.
		
	securityManager addUser: tmpUser.
	
	securityManager getUsers do: [ :user |
		self 
			assert: (user isKindOf: ITUser)
			description: '#getUsers returns a collection in where not all objects are users.'.
		
		self
			deny: (user id) isNil
			description: '#getUsers returns a collection in where not all users have an ID.'.
	].

	securityManager deleteUser: tmpUser.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 13:46'!
testHashAlgorithm
	"Tests whether Message Digest 5 Hash is used by the security manager."
	
	self
		assert: ITSecurityManager hashAlgorithm == MD5
		description: 'ITSecurityManager uses wrong hash algorithm. MD5 expected.'! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 13:50'!
testHashPassword
	"Tests whether a given password will be modified correctly through the security manager."
	
	self
		deny: (ITSecurityManager hashPassword: 'somepassword') = 'somepassword'
		description: 'A password was not hashed through security manager. The plain text was returned.'.
		
	self
		assert: (ITSecurityManager hashPassword: 'somepassword') = (ITSecurityManager hashPassword: 'somepassword')
		description: 'The security manager returns different hashes for the same plain text.'! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mfp 1/22/2008 14:30'!
testUpdateUserPassword
	"Tests whether it is possible to change a users password in the database."
	
	|user hash|
	user := ITUser new
		username: 'tmp_testChPwTo_User';
		setPassword: 'tmp_testChPwTo_Password';
		yourself.
	
	securityManager addUser: user.
	securityManager
		updateUser: user
		password: 'tmp_testChPwTo_NewPassword'.
	
	hash := ITSecurityManager hashPassword: 'tmp_testChPwTo_NewPassword'.

	self
		assert: (securityManager getUser: 'tmp_testChPwTo_User') passwordHash = hash
		description: 'Password could not be changed successfully'.
		
	securityManager deleteUser: user.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mfp 1/19/2008 17:50'!
testUpdateUserUsername
	"Tests whether it is possible to change a username in the database."
	
	| user |
	user := ITUser new
		username: 'tmp_testChUserTo_User';
		yourself.
	securityManager addUser: user.

	[securityManager updateUser: user username: 'tmp_testChUserTo_User'.
	self
		assert: false
		description: 'Username has been changed to a username that allready exists.'.] 
	on: ITUserAlreadyExists
	do: [self assert: true].
		
	securityManager updateUser: user username: 'tmp_testChUserTo_NewUser'.
	
	self
		assert: (user username = 'tmp_testChUserTo_NewUser')
		description: 'Username could not be changed successfully'.
		
	securityManager deleteUser: user.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mfp 1/22/2008 14:33'!
testUser
	"Tests whether the accessors for #user do what they should: get and set."
	
	self checkAccessor: #user on: securityManager with: ITUser new! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mfp 1/19/2008 16:40'!
testValidatePasswordFor
	"Tests whether security manager verifies the password for a user correctly."
	
	|user plaintext|
	plaintext := 'tmp_testValidatePasswordFor'.
	user := ITUser new
		username: 'tmp_testValidatePasswordFor_username';
		setPassword: plaintext;
		yourself.
	
	securityManager addUser: user.
	
	self
		assert: (securityManager validatePassword: plaintext for: user username)
		description: 'A correct password for a user could not be validated through the security manager.'.
		
	self
		deny: (securityManager validatePassword: (plaintext,'invalid_garbage') for: user username)
		description: 'An invalid password for a user was validated through the security manager. False expected.'.
		
	securityManager deleteUser: user.
! !

ITTest subclass: #ITTestSession
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestSession methodsFor: 'testing' stamp: 'mt 12/30/2007 11:53'!
setUp

	super setUp.
	session := ITSession new.! !

!ITTestSession methodsFor: 'tests' stamp: 'mig 1/15/2008 16:50'!
testIsLoggedIn
	"tests whether isLoggedIn returns the correct values"
	
	self 
		deny: session isLoggedIn
		description: '#isLoggedIn must be false directly after its creation'.
	
	session user: ITUser new.
	
	self
		assert: session isLoggedIn
		description: '#isLoggedIn must be true directly after user was set'.
	
	session user: nil.
	
	self
		deny: session isLoggedIn
		description: '#isLoggedIn must be false directly after logout'.! !

!ITTestSession methodsFor: 'testing' stamp: 'mt 1/18/2008 20:25'!
testIssueManager
	"Tests whether there is only one issue manager per session."
	
	self
		deny: session issueManager isNil
		description: 'A new ITSession object does not have an issue manager.'.
	
	self
		assert: session issueManager == session issueManager
		description: 'An ITSession instance returns different ITIssueManager instances. There should only be one.'! !

!ITTestSession methodsFor: 'testing' stamp: 'mt 12/30/2007 11:44'!
testLanguage
	"Tests whether it is possible to set and get a language for translation issues."
	
	session language: #english.
	
	self
		assert: session language = #english
		description: 'A language was just set and could not be retrieved from a session.'! !

!ITTestSession methodsFor: 'testing' stamp: 'mt 1/18/2008 20:25'!
testSecurityManager
	"Tests whether there is only one security manager per session."
	
	self
		deny: session securityManager isNil
		description: 'A new ITSession object does not have a security manager.'.
	
	self
		assert: session securityManager == session securityManager
		description: 'An ITSession instance returns different ITSecurityManager instances. There should only be one.'! !

!ITTestSession methodsFor: 'testing' stamp: 'mig 1/15/2008 16:34'!
testUser
	"Tests whether it is possible to set and get a user for a session. Also tests the default user functionality."
	
	self checkAccessor: #user on: session with: ITUser new.

	session user: nil.
	self 
		assert: session user == session securityManager defaultUser
		description: 'The default user ist not returned after user was set to nil.'! !

ITTest subclass: #ITTestTag
	instanceVariableNames: 'tag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestTag commentStamp: 'afk 1/20/2008 17:54' prior: 0!
I'm testing basic functionality of the ITTag-class like it's accessors.!

!ITTestTag methodsFor: 'testing' stamp: 'afk 1/9/2008 18:40'!
setUp

	super setUp.
	tag := ITTag new.! !

!ITTestTag methodsFor: 'testing' stamp: 'mig 1/12/2008 12:05'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."
	
	self checkAccessor: #id on: tag with: 1.
! !

!ITTestTag methodsFor: 'testing' stamp: 'mt 1/18/2008 20:26'!
testTagDescription
	"Tests whether the accessors for #tagDescription do what they should do - set and get."
	
	self checkAccessor: #tagDescription on: tag with: 'tmp_testTagDescription'.
! !

!ITTestTag methodsFor: 'testing' stamp: 'mt 1/18/2008 20:26'!
testTagName
	"Tests whether the accessors for #tagName do what they should do - set and get."
	
	self checkAccessor: #tagName on: tag with: 'tmp_testTagName'.
! !

ITTest subclass: #ITTestTranslator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 1/18/2008 20:31'!
setUp
	"Loads the test phrasebook."
	
	super setUp.
	ITTranslator getInstance loadFromLibrary: #exampleTs forLanguage: #example.	
	! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 1/18/2008 20:32'!
tearDown
	"Removes the test phrasebook."
	
	super tearDown.
	ITTranslator getInstance discardTranslationsForLanguage: #example.
	! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mig 1/10/2008 21:40'!
testAllLanguages
	"Tests whether it is possible to iterate through the results - #do: should be understood."
	
	self
		assert: (ITTranslator getInstance allLanguages isKindOf: Collection)
		description: '#allLangauges should be sequencable.'! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 1/18/2008 20:36'!
testCanTranslateInto
	"Tests whether it is possible to ask the translator whether a certain string could be translated."
	
	self
		assert: (ITTranslator getInstance canTranslate: 'test' into: #example)
		description: 'Should be able to translate the test string.'! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 1/18/2008 20:32'!
testDiscardTranslationsForLanguage
	"Tests whether all translations for a given language will be discarded."

	ITTranslator getInstance discardTranslationsForLanguage: #example.
	
	self
		assert: (ITTranslator getInstance translate: 'test' into: #example) = 'test'
		description: 'The test string should not be translated but was modified.'.
		
	! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 1/18/2008 20:40'!
testInitialize
	"Tests whether the translator has the default (maybe empty) phrasebook for #en."
	
	self
		assert: ((ITTranslator getInstance phraseBookFor: #en) isKindOf: Dictionary)
		description: 'A default phrasebook #en should always be present.'! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 1/18/2008 20:37'!
testLoadFromLibraryForLanguage
	"Tests whether it is possible to load a translation file (a lookup dictionary or so) from the file library. See #setUp for initialization."
		
	self
		assert: (ITTranslator getInstance translate: 'test' into: #example) = 'test_translated'
		description: 'Could not translate test string.'! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 1/18/2008 20:33'!
testPhraseBookFor
	"Tests whether it is possible to get a phrasebook for a language that should be present."
	
	self
		assert: ((ITTranslator getInstance phraseBookFor: #example) isKindOf: Dictionary)
		description: 'A phrasebook which should be present could not be retrieved.'! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 1/18/2008 20:34'!
testSingleton
	"Tests whether ITTranslator is a singleton."
	
	self
		assert: ITTranslator getInstance == ITTranslator getInstance
		description: ITTranslator name, '(class)>>#getInstance returns variing objects. Should be a Singleton.'.
		
	[ITTranslator new.
	self
		assert: false
		description: ITTranslator name, '(class)>>#new does not throws an Exception!! Should be a Singleton.']
		on: Exception
		do: [self assert: true].
		
	! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 12/26/2007 12:00'!
testTr
	"Tests whether ByteString>>#tr does return a byte string as well."
	
	self
		assert: ('test' tr isKindOf: ByteString)
		description: 'ByteString>>#tr does not return an instance of ByteString.'! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 12/26/2007 12:15'!
testTranslateInto
	"Tests whether the same text is returned if no translation is used."
	
	self
		assert: (ITTranslator getInstance translate: 'No Translation' into: #default) = 'No Translation'
		description: 'Text was modified while no translation was performed!!'! !

ITTest subclass: #ITTestUser
	instanceVariableNames: 'user adminRole guestRole'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Tests'!

!ITTestUser methodsFor: 'testing' stamp: 'mt 12/30/2007 11:40'!
setUp

	super setUp.
	user := ITUser new.
	
	adminRole := ITRole new.
	adminRole rolename: 'admin'.
	
	guestRole := ITRole new.
	guestRole rolename: 'guest'.! !

!ITTestUser methodsFor: 'testing' stamp: 'mig 1/13/2008 22:17'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	self checkAccessor: #id on: user with: 1! !

!ITTestUser methodsFor: 'testing' stamp: 'mt 1/18/2008 20:53'!
testInitialize
	"Tests whether a new user has an empty set of roles attached."
			
	self
		assert: user roles isEmpty
		description: 'A new user should not have any roles attached.'! !

!ITTestUser methodsFor: 'testing' stamp: 'mig 1/13/2008 22:20'!
testPassword
	"Tests whether a password for a user can be set and retrieved as hash for security reasons."

	|password hash|
	password := 'tmp_testPassword'.
	hash := ITSecurityManager hashPassword: password.
	
	user setPassword: password.
	
	self
		assert: user passwordHash = hash
		description: 'The user got another password hash as from ITSecurityManager(class)>>#hashPassword:.'
! !

!ITTestUser methodsFor: 'testing' stamp: 'mt 12/30/2007 11:22'!
testRoles
	"Tests whether the accessors for #roles do what they should do - set and get."

	| someRoles |
	someRoles := {adminRole. adminRole. guestRole. guestRole}.
	user roles: someRoles.
	
	self
		assert: (user roles isKindOf: Set) 
		description: 'The roles of an user are not a set.'.
		
	self
		assert: user roles = {adminRole. guestRole} asSet
		description: 'Some roles where just set to a user and could not be retrieved.' ! !

!ITTestUser methodsFor: 'testing' stamp: 'mig 1/13/2008 22:19'!
testUsername
	"Tests whether the accessors for #username do what they should do - set and get."

	self checkAccessor: #username on: user with: 'tmp_testUsername'! !

Object subclass: #ITCondition
	instanceVariableNames: 'tag operation arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITCondition commentStamp: 'afk 1/20/2008 17:51' prior: 0!
I'm a condition that is used to filter issues through an ITFilter. I hold arguments (an OrderedCollection) that are matching the tag and the operation on this tag (in particular the number of the arguments).!

!ITCondition methodsFor: 'accessing' stamp: 'amb 1/19/2008 01:10'!
arguments
	^ arguments! !

!ITCondition methodsFor: 'accessing' stamp: 'amb 1/19/2008 01:10'!
arguments: anObject
	arguments := anObject! !

!ITCondition methodsFor: 'accessing' stamp: 'amb 1/19/2008 01:10'!
operation
	^ operation! !

!ITCondition methodsFor: 'accessing' stamp: 'amb 1/19/2008 01:10'!
operation: anObject
	operation := anObject! !

!ITCondition methodsFor: 'accessing' stamp: 'amb 1/19/2008 01:10'!
tag
	^ tag! !

!ITCondition methodsFor: 'accessing' stamp: 'amb 1/19/2008 01:10'!
tag: anObject
	tag := anObject! !

Object subclass: #ITFilter
	instanceVariableNames: 'conditions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITFilter commentStamp: 'afk 1/20/2008 17:53' prior: 0!
I'm able to filter issues via using conditions (ITCondition) and combining them with logical and. By using my apply-method I can receive a number of issues (OrderedCollection) and return only those that match my conditions.!

!ITFilter class methodsFor: 'as yet unclassified' stamp: 'mt 1/25/2008 17:48'!
detailAccessors
	"Returns a dictionary whose keys are possible 'filter tags' and whose values are mappings for the specified values of the issues."

	| accessors session |
	accessors := (Dictionary new
		at: 'Created at'           put: [:issue | issue firstComment timestamp];
		at: 'Owner'                put: [:issue | issue firstComment user username];
		at: 'Number of answers'    put: [:issue | issue comments size];
		at: 'Last comment date'    put: [:issue | issue latestComment timestamp];
		at: 'User of last comment'	put: [:issue | issue latestComment user username];
		yourself).
				
	(session := WACurrentSession value)
		ifNil: [^accessors].
		
	session issueManager getTags do: [:tag |
		accessors at: tag tagName put: [:issue |
			| detail |
			detail := (issue getDetail: tag).
			detail
				ifNotNil:  [detail content]
				ifNil: [String new]
		]
	].

	^accessors! !

!ITFilter class methodsFor: 'as yet unclassified' stamp: 'mt 1/25/2008 17:48'!
detailTypes
	"Returns a dictionary whose keys are possible 'filter tags' and whose values specify the class of these information. See #detailAccessors."
	
	| types session |
	types := (Dictionary new
		at: 'Created at'           put: #TimeStamp;
		at: 'Owner'                put: #String;
		at: 'Number of answers'    put: #Integer;
		at: 'Last comment date'    put: #TimeStamp;
		at: 'User of last comment'	put: #String;
		yourself).
				
	(session := WACurrentSession value)
		ifNil: [^types].
		
	session issueManager getTags do: [:tag |
		types at: tag tagName put: #String
	].

	^types! !

!ITFilter class methodsFor: 'as yet unclassified' stamp: 'mig 1/20/2008 10:26'!
numArgsOfTag: aTag with: anOperation

	^	( self operators at: (Array
				braceWith: (self detailTypes at: aTag)
				with: anOperation)
			ifAbsent: [ [:x | ] ]
		) numArgs - 1! !

!ITFilter class methodsFor: 'initialize-release' stamp: 'mig 1/20/2008 20:00'!
operators
	"assignment of available filter operator"

	^(Dictionary new
			at: #(#String 'equals') put:
				[:x :str | str = x asString];
			at: #(#String 'matches') put:
				[:x :str | str matches: x asString];
			at: #(#TimeStamp 'equals') put:
				[:x :date | date = x asTimeStamp];
			at: #(#TimeStamp 'between') put:
				[:start :end :date | (start asTimeStamp < date) & (date < end asTimeStamp)];
			at: #(#TimeStamp 'until') put:
				[:until :date | date < until asTimeStamp];
			at: #(#TimeStamp 'since') put:
				[:since :date | since asTimeStamp < date];
			at: #(#Integer 'equals') put:
				[:x :int | int = x asInteger];
			at: #(#Integer 'between') put:
				[:start :end :int | (start asInteger <= int) & (int <= end asInteger)];
			at: #(#Integer 'at most') put:
				[:until :int | int <= until asInteger];
			at: #(#Integer 'at least') put:
				[:since :int | since asInteger <= int];
			yourself
		).! !

!ITFilter methodsFor: 'validation' stamp: 'mt 1/25/2008 17:18'!
addCondition: aCondition
	"Adds an ITCondition object which contains specific parameters to this filter."
	
	conditions add: aCondition.! !

!ITFilter methodsFor: 'private' stamp: 'mt 1/25/2008 17:20'!
apply: someIssues
	"Answers with a collection of all issues which match the filter."
	
	^ someIssues select: [:issue | self matches: issue ]
! !

!ITFilter methodsFor: 'accessing' stamp: 'amb 1/14/2008 01:11'!
conditions
	^ conditions! !

!ITFilter methodsFor: 'accessing' stamp: 'amb 1/14/2008 01:11'!
conditions: anObject
	conditions := anObject! !

!ITFilter methodsFor: 'initialize-release' stamp: 'amb 1/14/2008 11:25'!
initialize
	conditions := OrderedCollection new.! !

!ITFilter methodsFor: 'private' stamp: 'mt 1/25/2008 17:50'!
matches: anIssue
	"Answers whether the given issue matches all conditions in this filter. That means that conditions are connected with a logical AND."
	
	^self conditions allSatisfy: [:condition | | args |
		args := condition arguments copyWith: ((self class detailAccessors at: condition tag) value: anIssue).
		(self class operators at: {self class detailTypes at: condition tag. condition operation}) valueWithArguments: args asArray.
	]
	! !

Object subclass: #ITGlorpDB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITGlorpDB commentStamp: 'mt 12/14/2007 10:22' prior: 0!
I encapsulate the connection to a SQL database, at the moment with PostGreSQL. Feel free to use my class side method #connect to get a new ITGlorpSession instance. But be aware of the connection. You'll have to check whether ITGlorpSession>>#isLoggedIn.!

!ITGlorpDB class methodsFor: 'debug' stamp: 'mt 12/14/2007 10:37'!
closeAllConnections
	ITGlorpSession allInstances do: [:aSession |
		aSession logout.
		Transcript show: 'DEBUG: You closed a Session.']! !

!ITGlorpDB class methodsFor: 'connection' stamp: 'mig 1/24/2008 21:43'!
connect
	"Connects to the database with the given information from login and returns the current (new) session."	
	
	| accessor session system |
	accessor := DatabaseAccessor forLogin: self login.
	accessor login.
	accessor connection clientEncoding: #'utf-8'.

	session := GlorpSession new.
	session accessor: accessor.

	system := ITGlorpDescriptorSystem new.
	session system: system.
	system platform: session platform.
	
	^ session
! !

!ITGlorpDB class methodsFor: 'debug' stamp: 'mig 1/10/2008 13:25'!
createTables
	"I'm creating alle relevant tables but only if there are none of them."
	
	| session errorBlock |
	
	errorBlock := [:error | Transcript show: error messageText].
	
	Transcript show: 'IT: Trying to create all tables.'; cr.
	
	session := self connect.
	
	session inTransactionDo: [
		session system allTables do: [:each |
			Transcript show: ('IT: Creating table ''{1}''' format: {each name}); cr.
			session accessor createTable: each ifError: errorBlock.
		].
	].

	session inTransactionDo: [
		session system allTables do: [:each |
			Transcript show: ('IT: Creating foreign keys for table ''{1}''' format: {each name}); cr.
			session accessor createTableFKConstraints: each ifError: errorBlock.
		].
	].
! !

!ITGlorpDB class methodsFor: 'connection' stamp: 'mig 1/25/2008 00:24'!
login
	"The default login can be found here. It is used across the whole issue tracker application. It is also used for unit tests."
	
	^ Login new 
		database: PostgreSQLPlatform new;
		username: 'issuetracker';
		password: 'weloveseasidedontwe';
		connectString: '87.230.86.137:5432_issuetracker'.
		
	"^ Login new database: PostgreSQLPlatform new;
		username: 'postgres';
		password: '1234';
		connectString: 'localhost:5432_postgres'."! !

Object subclass: #ITIssue
	instanceVariableNames: 'id details comments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssue commentStamp: 'mt 12/30/2007 14:33' prior: 0!
I am an issue, a task, a ticket ... or however you'll call me.

I encapsulate several details and comments.!

!ITIssue methodsFor: 'accessing' stamp: 'mt 1/18/2008 19:11'!
addComment: aComment
	"Adds an ITIssueComment object to this issue."
	
	comments add: aComment.! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 1/25/2008 00:13'!
comments
	"Returns all comments sorted after their timestamps."
	
	^comments asArray sort: [:c1 :c2 | c1 timestamp < c2 timestamp].! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 12/11/2007 15:23'!
comments: aCollection
	comments := aCollection.! !

!ITIssue methodsFor: 'accessing' stamp: 'mfp 1/22/2008 15:30'!
deleteDetail: aTag
	"Removes the detail with the given tag."
	
	details removeAllSuchThat: [:detail| detail tag == aTag]! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 17:03'!
details
	^ details! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 17:03'!
details: anObject
	details := anObject! !

!ITIssue methodsFor: 'comments' stamp: 'mt 1/6/2008 14:13'!
firstComment
	"Returns the first added comment when the issue was created."
	
	^self comments first.! !

!ITIssue methodsFor: 'accessing' stamp: 'mfp 1/22/2008 15:31'!
getDetail: aTag
	"Returns a detail of an issue by a given tag"

	^details detect: [:each | each tag == aTag ] ifNone: [nil]! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 16:20'!
id
	^ id! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 16:20'!
id: anObject
	id := anObject! !

!ITIssue methodsFor: 'initialization' stamp: 'mt 12/15/2007 16:20'!
initialize

	super initialize.
	details := OrderedCollection new.
	comments := OrderedCollection new.

	! !

!ITIssue methodsFor: 'comments' stamp: 'mt 1/6/2008 14:13'!
latestComment
	"Returns the last added comment."
	
	^self comments last.! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 1/19/2008 15:44'!
setDetail: aDetail
	"Changes a detail --- creates a new detail if not present."
	
	|detail|
	(detail := self getDetail: aDetail tag)
		ifNil: [ self details add: aDetail ]
		ifNotNil: [ detail content: aDetail content ]! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 1/18/2008 19:28'!
setDetailWith: aTag to: aValue
	"Changes a detail with the given tag to the value --- creates a new detail if not present. This is just a convenience method for #setDetail:."
	
	self setDetail: (ITIssueDetail new
		tag: aTag;
		content: aValue;
		yourself)! !

Object subclass: #ITIssueComment
	instanceVariableNames: 'id content user timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueComment commentStamp: 'mt 12/30/2007 14:30' prior: 0!
I am just an annotation to an issue.

You should be able to see which user created me, what's the content and when that user created me.!

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
content
	^ content! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
content: anObject
	content := anObject! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
id
	^ id! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
id: anObject
	id := anObject! !

!ITIssueComment methodsFor: 'initialize-release' stamp: 'mfp 1/17/2008 16:19'!
initialize
	
	super initialize.
	
	content := ''.
	timestamp := TimeStamp current.! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mt 1/5/2008 10:53'!
timestamp
	^ timestamp! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mt 1/5/2008 10:53'!
timestamp: anObject
	timestamp := anObject! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mt 1/19/2008 11:39'!
user
	"send yourself to get rid of possibly existing Proxy"
	^ user yourself! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 1/16/2008 23:43'!
user: anObject
	user := anObject! !

Object subclass: #ITIssueDetail
	instanceVariableNames: 'id tag content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueDetail commentStamp: 'mt 12/30/2007 14:28' prior: 0!
I represent an issue detail.

What's a detail? Well... it's a tag-value-combination. You have some tag like 'title' and the proper value like 'This is the title'. These two things form a 'detail'. There are other well-definied tags out there but you should be able to create your own one in the application (e.g. as an administrator).!

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:54'!
content
	^ content! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:54'!
content: anObject
	content := anObject! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
id
	^ id! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
id: anObject
	id := anObject! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'mfp 1/22/2008 15:25'!
tag
	^ tag yourself.! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
tag: anObject
	tag := anObject! !

Object subclass: #ITIssueManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueManager commentStamp: 'mt 12/30/2007 14:22' prior: 0!
I am responsible for any "issue-related" operations.

Every issue operation has to go through my methods. I determine which privilege is needed to perfom several actions. 

DO NOT talk to the ITSecurityManager about security-related things assigned to issues in any ITComponent - I will do that for you.!

!ITIssueManager class methodsFor: 'testing' stamp: 'mt 1/26/2008 21:31'!
checkIssue: anIssue
	"Tests whether the given ITIssue object needs to be created in the next unit of work with GLORP. In that case an error will be raised. This is helpful for catching inconsistent browser states for many users, e.g. one user did remove an issue and the other one has its detail view open and tries to modify a detail."
	
	| session |
	
	session := ITGlorpSession getInstance.
	
	session beginUnitOfWork.
	
	(session isNew: anIssue)
		ifTrue: [session rollbackUnitOfWork. ITInvalidIssueError signal].
	
	session rollbackUnitOfWork.
	^true.	
	! !

!ITIssueManager methodsFor: 'changing' stamp: 'mt 1/20/2008 09:54'!
addIssue: anIssue
	"Adds a new issue to the database."
	
	ITSecurityManager checkPrivilege: #'add_issue'.
	
	ITGlorpSession getInstance save: anIssue.
	
	^anIssue.! !

!ITIssueManager methodsFor: 'changing' stamp: 'mt 1/20/2008 09:54'!
addTag: aTag
	"Adds a new tag to the database."
	
	ITSecurityManager checkPrivilege: #'add_tag'.
	
	ITGlorpSession getInstance save: aTag.
	
	^aTag.! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mfp 1/17/2008 16:39'!
deleteIssue: anIssue
	"Tries to delete an issue from the database."

	ITSecurityManager checkPrivilege: #'delete_issue'.
	
	ITGlorpSession getInstance delete: anIssue.
	! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mfp 1/17/2008 17:33'!
deleteTag: aTag
	"Tries to delete a tag from the database."

	ITSecurityManager checkPrivilege: #'delete_tag'.
	
	ITGlorpSession getInstance delete: aTag.
	! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mt 1/20/2008 09:55'!
getIssueWithID: aNumber
	"Returns an ITIssue object where the 'id' equals 'aNumber'."
	
	ITSecurityManager checkPrivilege: #'read_issue'.

	^ITGlorpSession getInstance
		readOneOf: ITIssue
		where: [:each | each id = aNumber]
! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mig 1/16/2008 23:05'!
getIssues
	"Returns all ITIssue objects in a collection that are in the database"
	
	ITSecurityManager checkPrivilege: #'list_issues'.

	^ ITGlorpSession getInstance readManyOf: ITIssue
	! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mfp 1/22/2008 16:15'!
getTag: aString

	^self getTags detect: [:tag | tag tagName asLowercase = aString]! !

!ITIssueManager methodsFor: 'as yet unclassified' stamp: 'mfp 1/17/2008 17:00'!
getTags
	"Reads and returns all tags"
	
	^ITGlorpSession getInstance read: ITTag.! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mt 1/24/2008 23:09'!
updateComment: aComment content: aString
	"Changes the content of a comment."

	| session |
	session := ITGlorpSession getInstance.

	session inUnitOfWorkDo: [
		session register: aComment.
		aComment content: aString.
	].! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mt 1/26/2008 21:48'!
updateIssue: anIssue addComment: aComment
	"Adds a new ITIssueComment object to an issue."

	| session |
	self class checkIssue: anIssue.
	ITSecurityManager checkPrivilege: #'add_comment'.

	session := ITGlorpSession getInstance.

	session inUnitOfWorkDo: [
		session register: anIssue.
		session register: aComment.
		anIssue addComment: aComment.
	].! !

!ITIssueManager methodsFor: 'as yet unclassified' stamp: 'mt 1/26/2008 21:48'!
updateIssue: anIssue deleteDetail: aDetail
	"Deletes one detail of anIssue"
	
	| session |
	self class checkIssue: anIssue.
	session := ITGlorpSession getInstance.
	
	session inUnitOfWorkDo: [
		session register: anIssue.
		anIssue deleteDetail: aDetail tag].! !

!ITIssueManager methodsFor: 'as yet unclassified' stamp: 'mt 1/26/2008 21:39'!
updateIssue: anIssue setDetail: aDetail
	"Updates one detail of anIssue with the content of aDetail."
	
	|session|
	self class checkIssue: anIssue.
	ITSecurityManager checkPrivilege: #'set_detail'.
	
	session := ITGlorpSession getInstance.
	
	session inUnitOfWorkDo: [
		session register: anIssue.
		anIssue setDetail: aDetail
	]	! !

!ITIssueManager methodsFor: 'as yet unclassified' stamp: 'mfp 1/17/2008 17:57'!
updateTag: aTag description: aString
	"Updates the description of a tag."
	
	|session|	
	session := ITGlorpSession getInstance.
	
	session inUnitOfWorkDo: [
		session register: aTag.
		aTag tagDescription: aString].! !

!ITIssueManager methodsFor: 'as yet unclassified' stamp: 'mfp 1/17/2008 17:56'!
updateTag: aTag name: aString
	"Updates the name of a tag."
	
	|session|	
	session := ITGlorpSession getInstance.
	
	session inUnitOfWorkDo: [
		session register: aTag.
		aTag tagName: aString].! !

Object subclass: #ITRole
	instanceVariableNames: 'id rolename privileges users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITRole commentStamp: 'mt 12/30/2007 14:37' prior: 0!
I am a user role and encapsulate a group of privileges.!

!ITRole methodsFor: 'accessing' stamp: 'mt 1/7/2008 22:51'!
addUser: aUser
	users add: aUser.! !

!ITRole methodsFor: 'accessing' stamp: 'mt 1/18/2008 20:08'!
deleteUser: aUser
	"Tries to remove a user from the role and does nothing if that user is absent."

	users remove: aUser ifAbsent: [].! !

!ITRole methodsFor: 'testing' stamp: 'mt 12/30/2007 11:24'!
hasPrivilege: aSymbol
	^ privileges includes: aSymbol! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:23'!
id
	^ id! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:23'!
id: anObject
	id := anObject! !

!ITRole methodsFor: 'initialization' stamp: 'mt 1/5/2008 23:31'!
initialize
	super initialize.
	
	privileges := Set new.
	users := Set new.
	rolename := 'unnamed'.! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:49'!
privileges
	^ privileges! !

!ITRole methodsFor: 'accessing' stamp: 'mt 12/30/2007 11:24'!
privileges: somePrivileges
	"Ensure that privileges are a set."
	
	privileges := somePrivileges asSet! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 15:53'!
rolename
	^ rolename! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 15:53'!
rolename: anObject
	rolename := anObject! !

!ITRole methodsFor: 'accessing' stamp: 'mt 1/5/2008 23:27'!
users
	^ users! !

!ITRole methodsFor: 'accessing' stamp: 'mt 1/7/2008 22:47'!
users: someUsers
	"Ensure that someUsers are a set."
	
	users := someUsers asSet.! !

Object subclass: #ITSecurityManager
	instanceVariableNames: 'user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITSecurityManager commentStamp: 'mt 12/30/2007 14:23' prior: 0!
I am responsible for application security operations.

You ask me to add and retrieve users and roles from/to the database. I'm able to hash a password which is in plaintext - that is more secure than saving them without.

Please DO NOT let me add any issue related stuff to the database - that's the job of the ITIssueManager. Although I'm able to verify passwords to usernames and given privileges for a user, it's also the job of the ITIssueManager to assign safety critical operations on issues to needed privileges.!

!ITSecurityManager class methodsFor: 'testing' stamp: 'mt 1/23/2008 23:32'!
checkPrivilege: aSymbol 
	"check whether the current user has the 'aSymbol'-privilege; always allow if no current session (unit testing)"
	| currentSession |
	
	WACurrentSession value ifNil: [^true].
	currentSession := WACurrentSession value.
	
	(currentSession user roles anySatisfy: [:role | 
		role hasPrivilege: aSymbol 
	]) ifFalse: [
		ITUnauthorizedError signal.
	].

	^ true! !

!ITSecurityManager class methodsFor: 'hashing' stamp: 'mig 12/22/2007 17:13'!
hashAlgorithm
	^MD5! !

!ITSecurityManager class methodsFor: 'hashing' stamp: 'mig 12/22/2007 17:13'!
hashPassword: aString
	^ (self hashAlgorithm hashMessage: aString) hex.
	! !

!ITSecurityManager class methodsFor: 'hashing' stamp: 'mig 12/22/2007 17:15'!
hashSize
	"hash size is algorithms size * 2 because hashs are stored as hex"
	^ self hashAlgorithm hashSize * 2
	! !

!ITSecurityManager class methodsFor: 'testing' stamp: 'mt 1/24/2008 16:54'!
privileges
	"Returns all available privileges that are used by the issue manager or the security manager. Please note that privileges that are not listed here cannot be attached to a role with the web frontend."

	^#(#'read_issue' #'add_issue' #'delete_issue' #'list_issues' #'view_home' #'add_comment' #'add_tag' #'delete_tag' #'add_role' #'add_user' #'rename_user' #'delete_role' #'delete_user' #'list_users' #'view_admin_panel' #'set_detail')! !

!ITSecurityManager methodsFor: 'traits' stamp: 'mt 1/20/2008 09:51'!
addRole: aRole
	"Adds an ITRole object to the database."
	
	self class checkPrivilege: #'add_role'.
	
	ITGlorpSession getInstance	save: aRole.
	
	^aRole! !

!ITSecurityManager methodsFor: 'traits' stamp: 'mt 1/23/2008 08:36'!
addUser: aUser
	"Adds an ITUser object to the database, but only if there is not already a user with the given name present. Applies the roles of the default user to the new created one."
	
	self user ifNotNil: [self class checkPrivilege: #'add_user'].
		
	(self getUser: aUser username)
		ifNotNil: [	ITUserAlreadyExists signal ].
	
	ITGlorpSession getInstance save: aUser.

	self updateUser: aUser roles: self defaultUser roles.

	^aUser! !

!ITSecurityManager methodsFor: 'defaults' stamp: 'mig 1/9/2008 15:19'!
defaultUser
	"return the default user (the unauthenticated user); create default user if it does not exist"
	
	^ (self getUser: 'Guest') ifNil: [
		self addUser: (ITUser new username: 'Guest')
	].! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mfp 1/17/2008 18:22'!
deleteRole: aRole
	"Removes a role from the database."
	
	self class checkPrivilege: #'delete_role'.
		
	ITGlorpSession getInstance delete: aRole.! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mfp 1/17/2008 18:25'!
deleteUser: aUser
	"Removes a user from the database."
	
	self class checkPrivilege: #'delete_user'.
		
	ITGlorpSession getInstance delete: aUser.! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mfp 1/17/2008 19:28'!
getRandomRole
	"Returns a random role"
	
	^ ITGlorpSession getInstance readOneOf: ITRole! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mfp 1/17/2008 18:47'!
getRoles
	"Returns all ITUser objects in a collection that are in the database"
	
	"ITSecurityManager checkPrivilege: #'list_roles'."

	^ ITGlorpSession getInstance read: ITRole.
	! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mt 1/20/2008 09:46'!
getUser: aUsername
	"Answers with an ITUser object whose username equals aUsername. Comparison is case-insensitive."
		
	^ITGlorpSession getInstance
		readOneOf: ITUser
		where: [:each | each username asLowercase = aUsername asLowercase].
	! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mfp 1/17/2008 18:47'!
getUsers
	"Returns all ITUser objects in a collection that are in the database"
	
	"ITSecurityManager checkPrivilege: #'list_users'."

	^ ITGlorpSession getInstance read: ITUser.
	! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mig 1/15/2008 16:42'!
loginUser: aUserName withPassword: aUserPassword
	"Logic for logging in a user"

	| valid |
	
	valid := self validatePassword: aUserPassword for: aUserName.
	valid ifTrue: [ 
		self user: (self getUser: aUserName) 
	].
	^valid

! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mfp 1/17/2008 19:42'!
updateRole: aRole addUser: aUser
	"Updates a Role by adding aUser"
	
	|session|
	session := ITGlorpSession getInstance.
	session inUnitOfWorkDo: [
		session register: aRole.
		aRole addUser: aUser.
	]! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mt 1/18/2008 20:08'!
updateRole: aRole deleteUser: aUser
	"Updates a Role by deleting aUser"
	
	|session|
	session := ITGlorpSession getInstance.
	session inUnitOfWorkDo: [
		session register: aRole.
		aRole deleteUser: aUser.
	]! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mfp 1/17/2008 19:24'!
updateRole: aRole privileges: somePrivileges
	"Updates the privileges of aRole to somePrivileges"
	
	|session|
	session := ITGlorpSession getInstance.
	session inUnitOfWorkDo: [
		session register: aRole.
		aRole privileges: somePrivileges.
	]! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'amb 1/29/2008 16:30'!
updateUser: aUser isActive: aBoolean
	"activates or deactivates a user."

	| glorpSession |
	glorpSession := ITGlorpSession getInstance.
	
	glorpSession inUnitOfWorkDo: [ 
		glorpSession register: aUser.
		aUser isActive: aBoolean.
		aBoolean
			ifTrue: [aUser setPassword: '']
			ifFalse: [aUser passwordHash: nil]
	]! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mt 1/19/2008 16:26'!
updateUser: aUser password: aPassword
	"change the password of a user"

	| session |
	session := ITGlorpSession getInstance.
	session inUnitOfWorkDo: [
		session register: aUser.
		aUser setPassword: aPassword.	
	]! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mfp 1/17/2008 19:50'!
updateUser: aUser roles: someRoles
	"Updates the roles of aUser to someRoles"
	
	|session|
	session := ITGlorpSession getInstance.
	session inUnitOfWorkDo: [
		session register: aUser.
		aUser roles: someRoles.
	]! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mt 1/19/2008 16:56'!
updateUser: aUser username: aString
	"change the name of a user and signals an error if this name is already taken"

	| glorpSession |
	self class checkPrivilege: #'rename_user'.
	
	(self getUser: aString) 
		ifNotNil: [ ITUserAlreadyExists signal ].
	
	glorpSession := ITGlorpSession getInstance.
	
	glorpSession inUnitOfWorkDo: [ 
		glorpSession register: aUser.
		aUser username: aString.
	]! !

!ITSecurityManager methodsFor: 'accessing' stamp: 'mig 1/15/2008 16:23'!
user
	^ user! !

!ITSecurityManager methodsFor: 'accessing' stamp: 'mig 1/15/2008 16:23'!
user: anObject
	user := anObject! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mig 1/17/2008 16:28'!
validatePassword: aPassword for: aUsername
	"return whether aUsername and aPassword match"
	| anUser |
	
	anUser := self getUser: aUsername.
	anUser ifNil: [^false].
	
	^ anUser passwordHash = (self class hashPassword: aPassword).! !

Object subclass: #ITTag
	instanceVariableNames: 'id tagName tagDescription'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTag commentStamp: 'afk 1/14/2008 01:09' prior: 0!
I'm a tag that users can select as an detail and add it to issues. My tagname and my tagdescription can be set via the admin-menu.!

!ITTag methodsFor: 'accessing' stamp: 'afk 1/6/2008 12:22'!
id
	^ id! !

!ITTag methodsFor: 'accessing' stamp: 'afk 1/6/2008 12:22'!
id: anObject
	id := anObject! !

!ITTag methodsFor: 'accessing' stamp: 'mig 1/12/2008 12:09'!
tagDescription
	^ tagDescription! !

!ITTag methodsFor: 'accessing' stamp: 'mig 1/12/2008 12:09'!
tagDescription: anObject 
	tagDescription := anObject! !

!ITTag methodsFor: 'accessing' stamp: 'mig 1/12/2008 12:10'!
tagName
	^ tagName! !

!ITTag methodsFor: 'accessing' stamp: 'mig 1/12/2008 12:10'!
tagName: anObject 
	tagName := anObject! !

Object subclass: #ITTranslator
	instanceVariableNames: 'phraseBooks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

ITTranslator class
	instanceVariableNames: 'instance'!

!ITTranslator commentStamp: 'mt 12/26/2007 10:42' prior: 0!
I am responsible for managing all localization/internationalization issues in this application. You have to load a valid translation file and then any ByteString can be translated using ByteString>>#tr.!

ITTranslator class
	instanceVariableNames: 'instance'!

!ITTranslator class methodsFor: 'as yet unclassified' stamp: 'mig 1/10/2008 22:41'!
findTexts
	"return all literals with tr message (i.e. find all texts to translate)"
	
	| texts pts |
	pts := ParseTreeSearcher new.
	texts := Set new.
	
	pts matches: '`#literal tr' do: [:aNode :answer | texts add: aNode children first value].
	
	(SystemNavigation default allCallsOn: #tr) do: [:methodref |
		pts executeTree: (methodref actualClass parseTreeFor: methodref methodSymbol).
	] displayingProgress: 'Searching for texts...'.

	^ texts.! !

!ITTranslator class methodsFor: 'accesssing' stamp: 'afk 1/16/2008 21:59'!
getInstance
	"Using singleton pattern to ensure that there is only one instance of ITTranslator"
	
	^instance ifNil:
		[ instance := self basicNew initialize.].! !

!ITTranslator class methodsFor: 'instance creation' stamp: 'mt 12/26/2007 10:50'!
new
	Error signal: 'Use getInstance because ', self name, ' is a singleton'.! !

!ITTranslator methodsFor: 'translation' stamp: 'mig 1/10/2008 19:30'!
allLanguages
	^ phraseBooks keys! !

!ITTranslator methodsFor: 'qtlinguist' stamp: 'mig 1/11/2008 12:18'!
buildTSFile: filename fromLanguage: language
	"builds a .ts file for use with Qt Linguist"
	
	| context tsroot document stream |
	
	context := XMLElement named: 'context'.
	context addElement: ((XMLElement named: 'name') addContent: (XMLStringNode string: 'SWE-IssueTracker')).
	
	(self class findTexts union: ITSecurityManager privileges) do: [:each | | message source translation |
		message := XMLElement named: 'message'.
		source := XMLElement named: 'source'.
		translation := XMLElement named: 'translation'.
	
		source addContent: (XMLStringNode string: each).
		(self canTranslate: each into: language)
			ifTrue: [ translation addContent: (XMLStringNode string: (self translate: each into: language)). ]
			ifFalse: [ translation attributeAt: 'type' put: 'unfinished' ].
		
		message
			addElement: source;
			addElement: translation.
		context
			addElement: message.
	].

	tsroot := XMLElement named: 'TS'.
	tsroot addElement: context.

	document := XMLDocument new version: '1.0'; encoding: 'utf-8'.
	document addElement: tsroot.
	
	stream := FileStream fileNamed: filename.
	stream ensureOpen; truncate.

	document printXMLOn: ((XMLWriter on: stream) initialize; yourself).
	
	stream close.
! !

!ITTranslator methodsFor: 'private' stamp: 'mig 1/10/2008 21:24'!
canTranslate: aByteString into: aLanguage
	"Checks whether there is a phrasebook for the given language present and that phrasebook has the desired translation."
	
	| phraseBook |	
	[phraseBook := self phraseBookFor: aLanguage]
		on: Error "key not found"
		do: [^false].
	
	^phraseBook includesKey: aByteString.
! !

!ITTranslator methodsFor: 'translation' stamp: 'mt 12/26/2007 12:19'!
discardTranslationsForLanguage: aSymbol
	"Discards the given phrase book."
	
	phraseBooks
		removeKey: aSymbol
		ifAbsent: ["No such phrase book found."].! !

!ITTranslator methodsFor: 'initialization' stamp: 'mig 1/10/2008 19:36'!
initialize
	"creates a new phraseBooks holder with an empty english phrasebook"

	super initialize.
	
	phraseBooks := Dictionary new.
	phraseBooks at: #en put: Dictionary new.! !

!ITTranslator methodsFor: 'translation' stamp: 'mig 1/10/2008 19:19'!
loadFromDisk: aFilename forLanguage: aSymbol
	"Creats a new phrase book for lookup."
	
	phraseBooks 
		at: aSymbol 
		put: (self parseTSDocument: 
			(XMLDOMParser parseDocumentFromFileNamed: aFilename)
		).
! !

!ITTranslator methodsFor: 'translation' stamp: 'mig 1/10/2008 21:56'!
loadFromLibrary: aFileSelector forLanguage: aSymbol
	"Creats a new phrase book for lookup."
	
	| xmltext |
	
	xmltext := (ITFileLibrary new perform: aFileSelector).
	
	phraseBooks 
		at: aSymbol 
		put: (self parseTSDocument: 
			(XMLDOMParser parseDocumentFrom:
				(xmltext convertFromWithConverter: UTF8TextConverter new) readStream
			)
		).
! !

!ITTranslator methodsFor: 'qtlinguist' stamp: 'mig 1/10/2008 19:13'!
parseTSDocument: aDOMDocument
	"reads a ts xml document used with Qt Linguist"	

	| context phraseBook |
	
	phraseBook := Dictionary new.

	context := aDOMDocument topElement elementAt: 'context'.
	
	context tagsNamed: #message do: [:each | | source translation |
		source := (each elementAt: 'source') contentString.
		translation := (each elementAt: 'translation') contentString.
		translation ifNotEmpty: [
			phraseBook at: source put: translation.
		].
	].

	^phraseBook! !

!ITTranslator methodsFor: 'private' stamp: 'mt 12/26/2007 11:50'!
phraseBookFor: aLanguage
	^phraseBooks at: aLanguage.! !

!ITTranslator methodsFor: 'translation' stamp: 'mig 1/10/2008 18:51'!
translate: aByteString into: aLanguage
	"translate aByteString into aLanguage, return aByteString if no translation was found"
	^ (self canTranslate: aByteString into: aLanguage)
		ifFalse: [aByteString]
		ifTrue: [(self phraseBookFor: aLanguage) at: aByteString]! !

Object subclass: #ITUser
	instanceVariableNames: 'id username passwordHash roles isActive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITUser commentStamp: 'mt 12/30/2007 14:36' prior: 0!
I represent a user which is utilizing this application.

I have a username, a password and several roles that are attached to me. These roles enable me to do some operations through the ITIssueManager on the database.!

!ITUser methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:26'!
id
	^ id! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:26'!
id: anObject
	id := anObject! !

!ITUser methodsFor: 'initialization' stamp: 'amb 1/29/2008 15:37'!
initialize
	super initialize.
	
	roles := Set new.
	isActive := true.! !

!ITUser methodsFor: 'accessing' stamp: 'amb 1/29/2008 15:49'!
isActive
	^ (isActive = '1')! !

!ITUser methodsFor: 'accessing' stamp: 'amb 1/29/2008 15:49'!
isActive: anObject
	isActive := (anObject ifTrue: ['1'] ifFalse: ['0'])! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
passwordHash
	^ passwordHash! !

!ITUser methodsFor: 'accessing' stamp: 'amb 1/29/2008 16:26'!
passwordHash: aHash
	"Sets explicitly the hash of the password. Should only be used from the security manager to clear the hash."
	
	passwordHash := aHash! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
roles
	^ roles! !

!ITUser methodsFor: 'accessing' stamp: 'mt 12/30/2007 11:21'!
roles: someRoles
	"Ensure that roles are a set."

	roles := someRoles asSet! !

!ITUser methodsFor: 'passwords' stamp: 'mt 1/18/2008 20:50'!
setPassword: aString
	"Saves the hash of the password aString."
	
	passwordHash := ITSecurityManager hashPassword: aString.! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
username
	^ username! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
username: anObject
	username := anObject! !

WASession subclass: #ITSession
	instanceVariableNames: 'language issueManager securityManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITSession commentStamp: 'mt 12/30/2007 14:42' prior: 0!
I am a normal Seaside session. You can get to your personal security manager and issue manager through my methods.

I have exactly one current language and one active user.!

!ITSession methodsFor: 'testing' stamp: 'mig 1/15/2008 16:31'!
isLoggedIn
	"returns whether the current user is logged in (or the guest)"

	^self user ~~ self securityManager defaultUser! !

!ITSession methodsFor: 'accessing' stamp: 'mig 1/15/2008 16:31'!
issueManager
	"returns the current issue manager for this session and creates one if not already done"
	
	^ issueManager ifNil: [ issueManager := ITIssueManager new ]! !

!ITSession methodsFor: 'accessing' stamp: 'mt 12/26/2007 11:36'!
language
	^ language! !

!ITSession methodsFor: 'accessing' stamp: 'mt 12/26/2007 11:36'!
language: anObject
	language := anObject! !

!ITSession methodsFor: 'accessing' stamp: 'mig 1/15/2008 16:30'!
securityManager
	"returns the current security manager for this session and creates one if not already done"

	^ securityManager ifNil: [ securityManager := ITSecurityManager new ]
! !

!ITSession methodsFor: 'accessing' stamp: 'mig 1/15/2008 16:30'!
user
	"returns the current or if absent the default user in the security manager"
	
	^ self securityManager user ifNil: [ self securityManager defaultUser ]! !

!ITSession methodsFor: 'accessing' stamp: 'mig 1/15/2008 16:30'!
user: aUser
	"sets the current user in the security manager"
	
	self securityManager user: aUser! !

!ByteString methodsFor: '*SWE-IssueTracker' stamp: 'mig 1/24/2008 22:48'!
tr
	"Uses ITTranslator for translating this string into another language"	
	
	| lang |
	
	[lang := WACurrentSession value language]
		on: MessageNotUnderstood "for testing purpose only"
		do: [lang := #default].
	
	^ITTranslator getInstance translate: self into: lang.! !

WAComponent subclass: #ITComponent
	instanceVariableNames: 'inLightbox director'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponent commentStamp: 'mig 1/9/2008 11:56' prior: 0!
I am the base class for all components (ITComponent*) in this project.!

!ITComponent methodsFor: 'accessing' stamp: 'mfp 1/11/2008 15:32'!
director
	^ director! !

!ITComponent methodsFor: 'accessing' stamp: 'mfp 1/11/2008 15:32'!
director: anObject
	director := anObject! !

!ITComponent methodsFor: 'messagebox' stamp: 'mig 1/10/2008 23:07'!
inLightbox
	^ inLightbox! !

!ITComponent methodsFor: 'messagebox' stamp: 'mig 1/10/2008 23:07'!
inLightbox: anObject
	inLightbox := anObject! !

!ITComponent methodsFor: 'messagebox' stamp: 'mt 1/10/2008 22:58'!
inform: aString
	^self inform: aString title: 'Information' tr! !

!ITComponent methodsFor: 'messagebox' stamp: 'mig 1/9/2008 17:37'!
inform: message title: title 
	
	^self inform: message title: title buttons: {'OK' tr}.! !

!ITComponent methodsFor: 'messagebox' stamp: 'mig 1/10/2008 23:10'!
inform: message title: title buttons: buttons
	"show a message (light-)box with title, message and buttons"
	| component |
	component := ITComponentMessage new title: title; message: message; buttons: buttons; yourself.

	^self inLightbox
		ifTrue: [self show: component]
		ifFalse: [self lightbox: component]
! !

!ITComponent methodsFor: 'initialization' stamp: 'mig 1/10/2008 23:07'!
initialize
	super initialize.
	inLightbox := false.! !

!ITComponent methodsFor: 'messagebox' stamp: 'mig 1/10/2008 23:17'!
lightbox: aComponent
	"call aComponent in a lightbox; workaround to allow recursive lightboxes"
	| returned |
	aComponent inLightbox: true.
	returned := super lightbox: aComponent.
	aComponent inLightbox: false.
	^returned! !

!ITComponent methodsFor: 'convenience' stamp: 'mt 1/12/2008 20:06'!
renderButtonOn: html callback: aBlock text: aString
	"Renders a push button with text. Remember to put this button into a form tag or the callback will not work!!"
	
	html button
		callback: aBlock;
		with: [html text: aString]! !

!ITComponent methodsFor: 'convenience' stamp: 'mt 1/12/2008 20:05'!
renderButtonOn: html callback: aBlock text: aString image: anUrl
	"Renders a push button with an icon and text. Remember to put this button into a form tag or the callback will not work!!"
	
	html button
		callback: aBlock;
		with: [
			html image url: anUrl.
			html break.		
			html text: aString]! !

!ITComponent methodsFor: 'inputbox' stamp: 'mt 1/13/2008 19:13'!
request: aMessage
	self request: aMessage value: '' title: 'Request' tr.
		
! !

!ITComponent methodsFor: 'inputbox' stamp: 'mt 1/13/2008 19:13'!
request: aMessage value: aValue
	self request: aMessage value: aValue title: 'Request' tr.
		
! !

!ITComponent methodsFor: 'inputbox' stamp: 'mt 1/13/2008 19:12'!
request: aMessage value: aValue title: aTitle
	"show a message (light-)box with title, message and buttons"
	| component |
	component := ITComponentInput new title: aTitle; message: aMessage; currentValue: aValue; yourself.

	^self inLightbox
		ifTrue: [self show: component]
		ifFalse: [self lightbox: component]
! !

ITComponent subclass: #ITComponentAddComment
	instanceVariableNames: 'comment currentIssue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentAddComment commentStamp: 'mig 1/9/2008 11:59' prior: 0!
I am responsible to provide an interface for adding comments to an issue.!

!ITComponentAddComment methodsFor: 'private' stamp: 'mig 1/26/2008 23:02'!
addComment
	"Appends a comment to the issue."
	
	[self session issueManager
		updateIssue: self currentIssue
		addComment: (ITIssueComment new 
			user: self session user; 
			content: self comment; yourself)]
		on: ITUnauthorizedError 
		do: [self inform: 'You are not authorized to add a comment.']
		on: ITInvalidIssueError
		do: [self inform: 'You cannot modify an invalid issue. Maybe another user just deleted this one...' tr title: 'Invalid Issue'. self director signal: #'invalid_issue_detected']! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:11'!
comment
	^ comment! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:11'!
comment: anObject
	comment := anObject! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mt 1/4/2008 23:16'!
currentIssue
	^ currentIssue! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mt 1/4/2008 23:16'!
currentIssue: anObject
	currentIssue := anObject! !

!ITComponentAddComment methodsFor: 'rendering' stamp: 'mt 1/25/2008 00:20'!
renderContentOn: html
	"Renders the anchor and the form (that is hidden by default) for adding a comment to an issue"

	html div: [
		html anchor
			id: 'comment_link';
			"onClick: (html effect id: 'comment_link'; perform: #toggleAppear);"
			onClick: (html effect id: 'new_comment_form'; perform: #toggleAppear);
			with: 'Click to add a comment...' tr.
				
		html form
			id: 'new_comment_form';
			style: 'display: none; text-align: right;';
			with: [
				html fieldSet
					class: 'add_comment';
					legend: 'Enter a new comment here:' tr;
					with: [
						html textArea
							class: 'add_issue_textarea'; 
							on: #comment of: self.
						html break.
					].
				
				html button
					callback: [ self addComment ];
					with: [
						html image url: ITFileLibrary / #commentaddPng.
						html break; text: 'Add Comment' tr.]
			]
	]! !

ITComponent subclass: #ITComponentAddIssue
	instanceVariableNames: 'details content comment tags selectedTag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentAddIssue commentStamp: 'mig 1/9/2008 11:59' prior: 0!
I am responsible to provide an interface for creating new issues.!

!ITComponentAddIssue methodsFor: 'private' stamp: 'mig 1/24/2008 11:53'!
addDetail
	
	self content isEmpty
		ifTrue: [self inform: 'Please enter a value for the selected tag.' tr. ^nil].
	
	(details detect: [:detail | detail tag == self selectedTag] ifNone: [details add: (ITIssueDetail new tag: self selectedTag; yourself)])
		content: self content.! !

!ITComponentAddIssue methodsFor: 'private' stamp: 'mt 1/23/2008 18:26'!
addIssue
	"Creates a new issue with an initial comment."
	
	| issue |
	
	(self details anySatisfy: [:detail | detail tag tagName asLowercase = 'title'])
		ifFalse: [self inform: 'You have to add a title to the issue.' tr title: 'No Title' tr. ^nil].
		
	self comment isEmpty
		ifTrue: [self inform: 'You have to add a comment to the issue.' tr title: 'Empty Comment' tr. ^nil].
	
	issue := ITIssue new. 
		
	self details do: [:detail | issue setDetail: detail].
	
	issue addComment: (ITIssueComment new 
		user: self session user; 
		content: self comment; yourself).
			
	[self session issueManager addIssue: issue]
		on: ITUnauthorizedError 
		do: [self inform: 'You are not authorized to add an issue!!' tr. ^nil].
		
	^ issue.
		! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'mt 1/22/2008 22:45'!
comment
	^ comment contents! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'mt 1/22/2008 22:45'!
comment: anObject
	comment contents: anObject! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'mt 1/22/2008 22:34'!
content
	^ content! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'mt 1/22/2008 22:34'!
content: anObject
	content := anObject! !

!ITComponentAddIssue methodsFor: 'private' stamp: 'mt 1/22/2008 22:58'!
deleteDetail: aDetail

	details remove: aDetail.! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'mt 1/22/2008 22:34'!
details
	^ details! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'mt 1/22/2008 22:34'!
details: anObject
	details := anObject! !

!ITComponentAddIssue methodsFor: 'initialize-release' stamp: 'mt 1/23/2008 18:14'!
initialize
	
	super initialize.
		
	details := OrderedCollection new.
	content := ''.
	
	comment := WAValueHolder with: ''.
	
	tags := self session issueManager getTags.
	selectedTag := tags first.
	! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mt 1/23/2008 18:22'!
renderButtonsOn: html
	"Displays a button to submit ne newly created issue."
	
	html form style: 'text-align: right;'; with: [
		html button	
			callback: [ self submit ];
			with: 'Create Issue' tr.	
	]
! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mt 1/23/2008 21:59'!
renderCommentInputOn: html
	"Enables the user to input an initial comment to the issue."
	
	html fieldSet
		class: 'add_comment';
		legend: 'Comment:' tr;
		with: [
			html textArea 
				id: 'add_comment_input';
				class: 'add_issue_textarea';
				on: #comment of: self;
				onChange: (html updater
					id: 'comment_changed';
					triggerFormElement: 'add_comment_input';
					callback: [:r | ]).
			html div id: 'comment_changed'. "dummy div ... due to html updater..."
		]! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mt 1/23/2008 18:37'!
renderContentOn: html
		
	[ITSecurityManager checkPrivilege: #'add_issue']
		on: ITUnauthorizedError 
		do: [html text: 'You are not authorized to add issues!!' tr. ^self].

	html div	class: 'add_issue'; with: [
		self renderHeadingOn: html.
		self renderInstructionsOn: html.
		html form class: 'blue'; with: [
			self renderDetailsOn: html.
			self renderCommentInputOn: html.
		].
		self renderButtonsOn: html.					
	]! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mt 1/26/2008 20:30'!
renderDetailOn: html for: aDetail class: aSymbol
	"Shows one detail in a table row. Creates an in-place-editor for each detail's content."
	
	html tableRow
		class: aSymbol asString;
		with: [
			html tableData class: 'tag'; with: aDetail tag tagName.
			
			html tableData class: 'detail_content'; with: [
				html span
				script: (html inPlaceEditor
					okControl: 'false';
					cancelControl: 'false';
					submitOnBlur: true;
					highlightColor: 'none';
					highlightEndColor: 'none';
					savingText: 'Saving...' tr;
					triggerInPlaceEditor: [ :value | aDetail content: value ];
					callback: [ :r | r render: aDetail content]);
				with: aDetail content	
			].
		
			html tableData with: [ aDetail tag == (self session issueManager getTag: 'title') ifFalse: [
				html anchor
					callback: [self deleteDetail: aDetail];
					with: [html image style: 'background: none;'; url: ITFileLibrary / #deletePng]]
			]
		]! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mt 1/26/2008 20:29'!
renderDetailsOn: html
	"Shows all details of the current issue. #odd is used for CSS."

	| odd |
	odd := true.
	
	html table
		id: 'issue';
		class: 'list_details';
		with: [
			html tableHead with: [ html tableRow with: [
				html tableHeading class: 'tag'; with: 'Tag' tr.
				html tableHeading class: 'detail_content'; with: 'Content' tr.
				html tableHeading.
			]].
	
			html tableBody with: [
				self details do: [:detail | 
					self
						renderDetailOn: html
						for: detail
						class: (odd ifTrue: [odd := false. #odd] ifFalse: [odd := true. #even])
				].
			
				self updateTags.
				self tags size > 0
					ifTrue: [self renderNewDetailRowOn: html].
			]		
		]! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mt 1/23/2008 18:16'!
renderHeadingOn: html
	"Renders a headline."
	
	html heading
		level: 2;
		with: 'Create a new Issue' tr! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mt 1/23/2008 18:37'!
renderInstructionsOn: html
	html paragraph
		with: 'Here you are able to create a new issue. Any detail can be attached to this issue. You have to insert an initial comment so that this issue can be referenced to you.' tr! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mt 1/26/2008 20:28'!
renderNewDetailRowOn: html
	"Shows a combobox with all available tags and some editor for the selected tag. Enables the user to create new details."

	html tableRow style: 'height: 20px; background-color: #fff;'; with: [
		html tableData style: 'border-left: none;'.	
		html tableData style: 'border-left: none;'.
		html tableData style: 'border-left: none;'.
	].
		
	html tableRow with: [	
		html tableData 
			with: [ 
				html div 
					class: 'tag_choosing';
					"style: 'white-space: nowrap;'; "
					with: [
						html span with: [
							html select
								id: 'tag_selection';
								list: (self tags);
								labels: [:tag| tag tagName];
								on: #selectedTag of: self;
								onChange: (html updater
									id: 'tag_description';
									triggerFormElement: 'tag_selection';
									callback: [:r |  ])
						].
						html span class: 'tag_choosing'; style: 'padding: 5px'; with: [
							html anchor
								style: 'display: table-cell;';
								callback: [self inform: self selectedTag tagDescription title: self selectedTag tagName];
								with: [html image url: ITFileLibrary / #helpPng].
				]]		
		].
		html tableData class: 'detail_content'; with: [
			html div id: 'tag_description'; style: 'display: none;'.
			html textInput
				id: 'content_input';
				style: 'width: 100%;';
				on: #content of: self;
				onChange: (html updater
					id: 'tag_description';
					triggerFormElement: 'content_input';
					callback: [:r |  ])
		].

		html tableData with: [
			html anchor
				callback: [self addDetail];
				with: [html image url: ITFileLibrary / #addPng]	
		]
	]! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'mt 1/22/2008 22:49'!
selectedTag
	^ selectedTag! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'mt 1/22/2008 22:49'!
selectedTag: anObject
	selectedTag := anObject! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'mt 1/23/2008 18:15'!
states

	^ {comment. details}! !

!ITComponentAddIssue methodsFor: 'private' stamp: 'mt 1/22/2008 22:41'!
submit
	"Tries to create a new issue and signals the result to the director."
	
	| newIssue |
	newIssue := self addIssue.
	newIssue ifNotNil: [self director signal: #'issue_created' with: {newIssue}].! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 1/6/2008 23:43'!
tags
	^ tags! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 1/6/2008 23:43'!
tags: anObject
	tags := anObject! !

!ITComponentAddIssue methodsFor: 'private' stamp: 'mt 1/26/2008 20:29'!
updateTags
	"Updates the collection with all available tags."
	
	self tags: (
		self session issueManager getTags copyWithoutAll:
			(self details collect: [:detail | detail tag])
	).
	
	self tags isEmpty
		ifFalse: [self selectedTag: self tags first]
		ifTrue: [self selectedTag: nil].! !

ITComponent subclass: #ITComponentAdmin
	instanceVariableNames: 'userTabs tagTabs languageTabs tabsRoot selectedTabKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentAdmin commentStamp: 'mt 1/6/2008 18:28' prior: 0!
I am rendering the frontend for the administration area. Here you can manage custom tags and attach roles to users. You can also create new roles and modify the privileges each role has.

If you want to add more (sub-)sections just modifiy #refreshTabs.!

!ITComponentAdmin methodsFor: 'rendering' stamp: 'mt 1/6/2008 17:51'!
children
	^Array with: self currentChild.! !

!ITComponentAdmin methodsFor: 'private' stamp: 'mt 1/6/2008 17:50'!
currentChild
	"Returns the component which is the current child - otherwise return nil."
	
	self tabsRoot valuesDo: [:subsections |
		(subsections includesKey: self selectedTabKey)
			ifTrue: [^(subsections at: self selectedTabKey)]
		].
	^nil! !

!ITComponentAdmin methodsFor: 'initialization' stamp: 'mt 1/6/2008 18:21'!
initialize
	"Creates all tabs once but does not select one to show the introduction for this component to the user. See #renderIntroductionOn: for more details."
	
	super initialize.
	self refreshTabs.
	self selectedTabKey: ''.! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mig 1/10/2008 19:46'!
languageTabs
	^ languageTabs! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mig 1/10/2008 19:46'!
languageTabs: anObject
	languageTabs := anObject! !

!ITComponentAdmin methodsFor: 'private' stamp: 'amb 1/29/2008 16:02'!
refreshTabs
	"Creates new component objects for the children but keeps the current selected tab."
	
	self userTabs: (Dictionary new
		at: 'User -> Roles' tr put: ITComponentAdminUsers new;
		at: 'Roles -> Privileges' tr put: ITComponentAdminRoles new;
		at: 'Activation' tr put: ITComponentAdminActivation new;
		yourself).

	self tagTabs: (Dictionary new
		at: 'Tags' tr put: ITComponentAdminTags new;
		yourself).
		
	self languageTabs: (Dictionary new
		at: 'Install Languages' tr put: ITComponentAdminLanguages new;
		yourself).

	self tabsRoot: (Dictionary new
		at: 'User Control' tr put: self userTabs;
		at: 'Custom Tags' tr put: self tagTabs;
		at: 'Translation' tr put: self languageTabs;
		yourself).
	
				
	! !

!ITComponentAdmin methodsFor: 'rendering' stamp: 'afk 1/16/2008 00:34'!
renderChildrenOn: html
	"renders the contents of the current active section of the admin-area"

	html div
		class: 'content_with_menu';
		with: [
			self selectedTabKey = ''
				ifTrue:  [ self renderIntroductionOn: html ]
				ifFalse: [ html render: self currentChild ]
		]! !

!ITComponentAdmin methodsFor: 'rendering' stamp: 'mt 1/23/2008 23:33'!
renderContentOn: html
	"checks the privilege of the current user for accessing this area and rendering main div-structure"

	[ITSecurityManager checkPrivilege: #'view_admin_panel'.]
		on: ITUnauthorizedError
		do: [html text: 'You are not authorized to administrate this application!!' tr. ^nil].

	html div
		with: [
			self renderSideMenuOn: html.
			self renderChildrenOn: html.
		]! !

!ITComponentAdmin methodsFor: 'rendering' stamp: 'mt 1/23/2008 22:49'!
renderIntroductionOn: html
	"This shows a brief introduction to the admin area."
	
	html heading
		level: 2;
		with: 'Welcome to the Administration Area!!' tr.
		
	html paragraph	with: 'In this area you can modify several security settings:' tr.
	html unorderedList with: [
		html listItem with: 'attach roles to users' tr.
		html listItem with: 'detach roles from users' tr.
		html listItem with: 'create new roles' tr.
		html listItem with: 'modify privileges for each role' tr.
		html listItem with: 'delete roles' tr.].
	html paragraph with: 'Additionally, you can create your own new tags. These custom tags can be added to any issue by all users which are allowed to add or modify them. Every custom tag contains:' tr.
	html unorderedList with: [
		html listItem with: 'a descriptive title' tr.
		html listItem with: 'a clear and brief description of the tag' tr.
		html listItem with: 'the required input field with default values (not yet)' tr].
	html paragraph with: 'Please apply any changes with caution!!' tr.
			! !

!ITComponentAdmin methodsFor: 'rendering' stamp: 'afk 1/16/2008 00:33'!
renderSideMenuOn: html
	"renders the side-menu of the admin-area and highlights the current selected section"

	html div	id: 'tab_menu'; with: [
		html unorderedList with: [
			self tabsRoot keysAndValuesDo: [:section :subsections | 
				html listItem with: [ html text: section. html unorderedList with:[
					subsections keysDo: [:subsection |
						html listItem
							class: (self selectedTabKey = subsection ifTrue: ['active']);
							with: [html anchor callback: [self selectedTabKey: subsection; refreshTabs]; with: subsection]
					]
				]]
			]
		]
	]! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 11:54'!
selectedTabKey
	^ selectedTabKey! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 11:54'!
selectedTabKey: anObject 
	^ selectedTabKey := anObject! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 17:23'!
tabsRoot
	^ tabsRoot! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 17:23'!
tabsRoot: anObject
	tabsRoot := anObject! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'afk 1/6/2008 15:35'!
tagTabs
	^ tagTabs! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'afk 1/6/2008 15:35'!
tagTabs: anObject
	tagTabs := anObject! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 17:23'!
userTabs
	^ userTabs! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 17:23'!
userTabs: anObject
	userTabs := anObject! !

ITComponent subclass: #ITComponentAdminActivation
	instanceVariableNames: 'users selectedUser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentAdminActivation methodsFor: 'private' stamp: 'amb 1/29/2008 16:50'!
activateSelectedUser

	[self session securityManager updateUser: self selectedUser isActive: true]
		on: ITUnauthorizedError
		do: [self inform: 'You are not authorized to activate a user.' tr]! !

!ITComponentAdminActivation methodsFor: 'rendering' stamp: 'amb 1/29/2008 17:05'!
renderButtonOn: html
	
	html button
		callback: [self activateSelectedUser];
		with: 'Activate ' tr.! !

!ITComponentAdminActivation methodsFor: 'rendering' stamp: 'amb 1/29/2008 17:01'!
renderContentOn: html
	html heading
		level: 2;
		with: 'User Activation' tr.
		
	[self users: (self session securityManager getUsers select: [:user |user isActive not])]
		on: ITUnauthorizedError 
		do: [html text: 'You are not authorized to list all users!!.'. ^self].
	
	self users isEmpty
		ifTrue: [html paragraph with: 'No user needs to be activated.' tr. ^self].
	
	self
		renderIntroductionOn: html;
		selectedUser: self users first.
		
	html div
		style: 'border: 1px solid #dfe1e1; text-align: center; padding: 5px;';
		with: [self renderUserSelectionOn: html]
	
! !

!ITComponentAdminActivation methodsFor: 'rendering' stamp: 'amb 1/29/2008 16:57'!
renderIntroductionOn: html
	html paragraph
		with: 'Deactivated users are not able to login. Created issues are not affected by this status. An activated user has an empty default password.' tr.! !

!ITComponentAdminActivation methodsFor: 'rendering' stamp: 'amb 1/29/2008 17:01'!
renderUserSelectionOn: html

	html form
		id: 'user_drop_down';
		with: [
			html text: 'Select a user:' tr; space.
			html select
				list: self users;
				selected: self selectedUser;
				labels: [:user|user username];
				callback: [ :value | self selectedUser: value ];
				onChange: (html updater
					id: 'activation_button';
					triggerForm: 'user_drop_down';
					callback: [:r | self renderButtonOn: r]).
					
			html span
				id: 'activation_button';
				with: [ self renderButtonOn: html ]
		]! !

!ITComponentAdminActivation methodsFor: 'accessing' stamp: 'amb 1/29/2008 16:47'!
selectedUser
	^ selectedUser! !

!ITComponentAdminActivation methodsFor: 'accessing' stamp: 'amb 1/29/2008 16:47'!
selectedUser: anObject
	selectedUser := anObject! !

!ITComponentAdminActivation methodsFor: 'accessing' stamp: 'amb 1/29/2008 16:47'!
users
	^ users! !

!ITComponentAdminActivation methodsFor: 'accessing' stamp: 'amb 1/29/2008 16:47'!
users: anObject
	users := anObject! !

ITComponent subclass: #ITComponentAdminLanguages
	instanceVariableNames: 'languageId selectedFile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentAdminLanguages commentStamp: 'afk 1/14/2008 01:07' prior: 0!
I offer the ability to add / delete dictionarys to translate the issuetracker to other languages.!

!ITComponentAdminLanguages methodsFor: 'accessing' stamp: 'mig 1/10/2008 20:44'!
languageId
	^ languageId! !

!ITComponentAdminLanguages methodsFor: 'accessing' stamp: 'mig 1/10/2008 20:44'!
languageId: anObject
	languageId := anObject! !

!ITComponentAdminLanguages methodsFor: 'rendering' stamp: 'mig 1/10/2008 21:08'!
renderContentOn: html
	"You can only uninstall a language if there are at least two."

	self renderInstallOn: html.
	
	ITTranslator getInstance allLanguages size > 1
		ifTrue: [ self renderUninstallOn: html ]! !

!ITComponentAdminLanguages methodsFor: 'rendering' stamp: 'afk 1/21/2008 11:12'!
renderInstallOn: html
	"renders the form where the user can select what language (from the ITFileLibary) to add to the application"

	html heading
		level: 2;
		with: 'Install a new language' tr.
	
	html paragraph
		with: 'You can install any new language to the internal translator which is present in the application''s file library.' tr.
		
	html paragraph
		with: 'Enter a proper language ID and press the Install-Button. Note, that present IDs will be overwritten.' tr.

	html form with: [
		html table with: [ html tableRow with: [
			html tableData
				with: [html image title: 'Select a phrasebook'; url: ITFileLibrary / #bookopenPng].
	
			html tableData with: [
				html select
					on: #selectedFile of: self;
					list: (ITFileLibrary selectors select: [:item | item endsWith: 'Ts']);
					labels: [:item | item asString]].
		
			html tableData style: 'width: 10px;'.
		
			html tableData
				with: [html image title: 'ID'; url: ITFileLibrary / #tagbluePng].
				
			html tableData
				with: [html textInput on: #languageId of: self; maxLength: 2; size: 2].
				
			html tableData style: 'width: 10px;'.
	
			html tableData with: [
				html button
					callback: [ITTranslator getInstance
						loadFromLibrary: self selectedFile
						forLanguage: self languageId asLowercase asSymbol];
					with: 'Install' tr]
		]]
	]! !

!ITComponentAdminLanguages methodsFor: 'rendering' stamp: 'mt 1/20/2008 21:56'!
renderUninstallOn: html
	"renders the form where the user can uninstall a language except english"

	html heading
		level: 2;
		with: 'Uninstall a language' tr.
		
	html paragraph
		with: 'Select the language ID of an installed phrasebook and press ''Uninstall'' to remove all translations which are in that book from the application.' tr.
		
	html paragraph
		with: 'Note, that you cannot remove #en because this is the default language.' tr.
	
	html form with: [
		html table with: [ html tableRow with: [ 
			html tableData 
				with: [html image title: 'Phrasebook'; url: ITFileLibrary / #bookopenPng].
					
			html tableData with: [
				html select
					on: #languageId of: self;
					list: (ITTranslator getInstance allLanguages copyWithout: #en) asSortedCollection;
					labels: [:item | item asString]].
			
			html tableData style: 'width: 10px'.
				
			html tableData with: [
				html button
					callback: [ITTranslator getInstance discardTranslationsForLanguage: self languageId asSymbol];
					with: 'Uninstall' tr].
		]]
	]! !

!ITComponentAdminLanguages methodsFor: 'accessing' stamp: 'mig 1/10/2008 20:40'!
selectedFile
	^ selectedFile! !

!ITComponentAdminLanguages methodsFor: 'accessing' stamp: 'mig 1/10/2008 20:40'!
selectedFile: anObject
	^ selectedFile := anObject! !

ITComponent subclass: #ITComponentAdminRoles
	instanceVariableNames: 'attachedPrivileges availablePrivileges selectedRole newRolename'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentAdminRoles commentStamp: 'afk 1/14/2008 01:18' prior: 0!
I give the user (e.g. an administrator) the ability to change the assignment from roles to privileges.!

!ITComponentAdminRoles methodsFor: 'private' stamp: 'mig 1/9/2008 17:18'!
addRole
	| newRole |
	newRole := ITRole new.
	newRole rolename: self newRolename.
	
	[self session securityManager addRole: newRole.]
		on: ITUnauthorizedError
		do: [self inform: 'You are not authorized to add a role!!' tr. ^self].
		
	self selectedRole: newRole.! !

!ITComponentAdminRoles methodsFor: 'private' stamp: 'mfp 1/17/2008 19:25'!
applyChanges
	"Resets all privileges to the selected role."
	
	self session securityManager updateRole: self selectedRole privileges: self attachedPrivileges.! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 12/29/2007 15:12'!
attachedPrivileges
	^ attachedPrivileges! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 12/29/2007 15:12'!
attachedPrivileges: anObject
	attachedPrivileges := anObject! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 12/29/2007 15:12'!
availablePrivileges
	^ availablePrivileges! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 12/29/2007 15:12'!
availablePrivileges: anObject
	availablePrivileges := anObject! !

!ITComponentAdminRoles methodsFor: 'private' stamp: 'mt 1/23/2008 08:47'!
deleteRole

	self selectedRole users ifNotEmpty: [
		self inform: 'You need to detach this role from all relevant users first before you can delete this one.'.
		^self
	].

	(self inform: ('Do you really want to delete the role: ''{1}''?' tr format: {self selectedRole rolename}) title: 'Question' buttons: {'Yes' tr. 'No' tr}) = 2
		ifTrue: [^self].

	[self session securityManager deleteRole: self selectedRole]
		on: ITUnauthorizedError
		do: [self inform: 'You are not authorized to delete a role!!' tr. ^self].
		
	self selectedRole: (self session securityManager getRandomRole).

	self updatePrivileges.! !

!ITComponentAdminRoles methodsFor: 'initialization' stamp: 'mfp 1/17/2008 19:28'!
initialize
	super initialize.
	
	self newRolename: 'New Role' tr.
	
	selectedRole := self session securityManager getRandomRole.
	self updatePrivileges.! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 1/5/2008 22:17'!
newRolename
	^ newRolename! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 1/5/2008 22:17'!
newRolename: anObject
	newRolename := anObject! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/12/2008 19:59'!
renderApplyChangesButtonOn: html
	html button
		callback: [self applyChanges];
		with: [
			html image url: ITFileLibrary / #acceptPng.
			html break.		
			html text: 'Apply changes' tr]! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/6/2008 00:35'!
renderAttachedUsersOn: html
	"Shows the current users which have the selected privilege attached."
	
	html fieldSet
		legend: 'Attached Users' tr, ' (', self selectedRole users size asString, ')';
		with: [
			html unorderedList
				with: [
					self selectedRole users do: [:user | html listItem with: user username]
				]
		]! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/23/2008 08:43'!
renderContentOn: html
	
	self selectedRole isNil 
		ifTrue: [self selectedRole: self session securityManager getRandomRole].
	self selectedRole isNil
			ifTrue: [
				html fieldSet
					legend: 'Add a new role' tr;
					with: [
						html span
							style: 'color: #000; font-weight: normal; ';
							with: 'There is no role available to be configured. You need to add a new one.' tr.
						html break.
						self renderNewRoleInputOn: html ].
				^nil.
			] ifFalse: [ self updatePrivileges ].

	html heading
		id: 'role_heading';
		level: 2;
		with: [ html text: ('Edit settings for role ''{1}''' tr format: {self selectedRole rolename}) ].

		
	html fieldSet
		legend: 'Select another role:' tr;
		with: [ self renderRoleSelectionOn: html ].
	
	html div
		id: 'privilege_lists';
		with: [ self renderPrivilegeListsOn: html ].
		
	html div
		id: 'attached_users';
		with: [ self renderAttachedUsersOn: html ].
	
	html fieldSet
		legend: 'Add a new role' tr;
		with: [ self renderNewRoleInputOn: html ].
	
	html form 
	style: 'text-align: right;';
	with: [
		self renderButtonOn: html callback: [self deleteRole] text: 'Delete Role' tr image: ITFileLibrary / #deletePng.
		self renderButtonOn: html callback: [self applyChanges] text: 'Apply Changes' tr image: ITFileLibrary / #acceptPng
	].
	
	html div style: 'clear: both'.! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/5/2008 23:23'!
renderDeleteRoleButtonOn: html
	html div
		class: 'button';
		style: 'float: left; margin: 10px 5px';
		with: [
			html image url: ITFileLibrary / #deletePng.
			html break.
			html anchor
				callback: [self deleteRole];
				with: [html text: 'Delete Role' tr]
		].! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mig 1/9/2008 22:29'!
renderListItems: aSelector on: html
	html unorderedList
		id: aSelector asString;
		script: (html sortable
			constraint: false;
			dropOnEmpty: true;
			containment: #( availablePrivileges attachedPrivileges );
			onUpdate: (html updater
				id: 'actual_privileges';
				triggerSortable: aSelector	callback: [ :values | self perform: aSelector asMutator with: values ]
			)
		);
		with: [
			(self perform: aSelector) do: [ :each |
				html listItem
					passenger: each;
					class: (aSelector = #availablePrivileges ifTrue: ['available'] ifFalse: ['attached']);
					with: each asString tr ] ]! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mfp 1/17/2008 18:11'!
renderNewRoleInputOn: html
	html form
		id: 'new_role_form';
		with: [
			html textInput on: #newRolename of: self.
			html button 
				onClick: (html updater
					id: 'role_lists';
					triggerForm: 'new_role_form';
					callback: [:r | self addRole; updatePrivileges; renderPrivilegeListsOn: r]);
				with: 'add role' tr.
		].! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/5/2008 22:34'!
renderPrivilegeListsOn: html

		html fieldSet
			class: 'drag_list';
			legend: 'Available Privileges' tr;
			with: [ self renderListItems: #availablePrivileges on: html ].
		html fieldSet
			class: 'drag_list';
			legend: 'Attached Privileges' tr;
			style: 'float: right;';
			with: [ self renderListItems: #attachedPrivileges on: html ].
		html div
			style: 'clear: both'.! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mfp 1/17/2008 19:33'!
renderRoleSelectionOn: html

	| roles |
	
	[roles := self session securityManager getRoles]
		on: ITUnauthorizedError 
		do: [html text: 'You are not authorized to list all roles!!.'. ^self].

	html form
		id: 'role_drop_down';
		with: [
			html select
				list: roles;
				selected: self selectedRole;
				labels: [:role|role rolename];
				callback: [ :value | self selectedRole: value ];
				onChange: (html updater
					id: 'privilege_lists';
					triggerForm: 'role_drop_down';
					callback: [:r | self updatePrivileges; renderPrivilegeListsOn: r]);
				onChange: (html updater
					id: 'role_heading';
					triggerForm: 'role_drop_down';
					callback: [:r | r text: 'Edit settings for role: ' tr, self selectedRole rolename ]);
				onChange: (html updater
					id: 'attached_users';
					triggerForm: 'role_drop_down';
					callback: [:r | self renderAttachedUsersOn: r ]).										
		].! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 1/5/2008 22:25'!
selectedRole
	^ selectedRole! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 1/5/2008 22:25'!
selectedRole: anObject
	^ selectedRole := anObject! !

!ITComponentAdminRoles methodsFor: 'private' stamp: 'mt 1/19/2008 11:45'!
updatePrivileges
	"Updates the available and attached privileges to the current selected role."
	
	selectedRole ifNil: [^self].
	
	self attachedPrivileges: selectedRole privileges asSortedCollection.
	self availablePrivileges:	(
		ITSecurityManager privileges asSet collect: [:each |
			(selectedRole privileges includes: each) not ifTrue: [each]
		]
	) asSortedCollection
! !

ITComponent subclass: #ITComponentAdminTags
	instanceVariableNames: 'newTag newTagDescription newTags'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentAdminTags commentStamp: 'afk 1/14/2008 01:06' prior: 0!
I'm responsible for giving the user (e.g. an administrator) the ability to create / delete / change tags and their descriptions which users can add to issues.!

!ITComponentAdminTags methodsFor: 'private' stamp: 'mfp 1/17/2008 17:42'!
applyChanges

	
	[self session issueManager addTag:
		(ITTag new
			tagName: self newTag;
			tagDescription: self newTagDescription;
			yourself)]
		on: ITUnauthorizedError
			do: [self inform: 'You are not authorized to create a tag!!' tr].! !

!ITComponentAdminTags methodsFor: 'accessing' stamp: 'mfp 1/22/2008 16:29'!
deleteTag: aTag

	| allDetails |
	allDetails := ITGlorpSession getInstance read: ITIssueDetail.

	(allDetails anySatisfy: [:detail | detail tag == aTag])
		ifTrue: [self inform: 'You cannot delete that tag while it is used by an issue.' tr. ^self].

	[(self session issueManager deleteTag: aTag)]
		on: ITUnauthorizedError
		do: [self inform: 'You are not authorized to delete a tag!!' tr].! !

!ITComponentAdminTags methodsFor: 'private' stamp: 'afk 1/6/2008 23:41'!
deleteTagEntries
	self newTag: nil.
	self newTagDescription: nil.! !

!ITComponentAdminTags methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:00'!
newTag
	^ newTag! !

!ITComponentAdminTags methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:00'!
newTag: anObject
	newTag := anObject! !

!ITComponentAdminTags methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:14'!
newTagDescription
	^ newTagDescription! !

!ITComponentAdminTags methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:14'!
newTagDescription: anObject
	newTagDescription := anObject! !

!ITComponentAdminTags methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:00'!
newTags
	^ newTags! !

!ITComponentAdminTags methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:00'!
newTags: anObject
	newTags := anObject! !

!ITComponentAdminTags methodsFor: 'rendering' stamp: 'mfp 1/17/2008 17:42'!
renderAddTagOn: html

	html fieldSet
		legend: 'Add a new tag:' tr;
		style: '	font-weight: normal;
				color: black;';
		with:[
			html form
				id: 'new_tag_form';
				with: 	[
					html table:[
						
						html tableHead with: [
							html tableRow with: [
								html tableHeading: 'Name' tr.
								html tableHeading: 'Description' tr.
							]
						].
						
						html tableRow: [
							html tableData: [ html textInput 
												script: html formElement focus;
												on: #newTag of: self. 
							].
							html tableData:  [ html textInput on: #newTagDescription of: self. ].
						html tableRow: [
							html tableData: [
								html submitButton 
									callback: [	
										self applyChanges.
										self deleteTagEntries.
										self answer: true. 
									];
									value: 'Add this tag' tr.
							].
						].
					]. 	
				].
			].
		].! !

!ITComponentAdminTags methodsFor: 'rendering' stamp: 'afk 1/14/2008 19:26'!
renderContentOn: html
	"Renders the possibility to add new tags while creating a new Issue"
				
	self renderCurrentTagsOn: html.
	self renderAddTagOn: html.! !

!ITComponentAdminTags methodsFor: 'rendering' stamp: 'mfp 1/22/2008 16:25'!
renderCurrentTagsOn:  html
	"Shows all the tags that are currently in the database + abillity of deleting tags"
	
	| tags |
	tags := self session issueManager getTags.
	
	html fieldSet
		legend: 'Currently available tags:' tr;
		style: '	font-weight: normal;
				color: black;';
		with: [ html table:[
					html tableHead with: [
					html tableRow: [
						html tableHeading: 'Name' tr.
						html tableHeading: 'Description' tr.
						html tableHeading: 'Option' tr.
					]]. 	
					html tableBody with: [
					tags do: [ :tag |
						
						html tableRow: [
							html tableData: [html strong: tag tagName].
							html tableData: tag tagDescription.
							html tableData: [
								html anchor
									callback: [ self deleteTag: tag ];
									with: 'delete' tr.
								]
						]. 
					]]
				]
		]! !

ITComponent subclass: #ITComponentAdminUsers
	instanceVariableNames: 'attachedRoles availableRoles selectedUser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentAdminUsers commentStamp: 'afk 1/14/2008 01:18' prior: 0!
I give the user (e.g. an administrator) the ability to change the assignment from users to roles.!

!ITComponentAdminUsers methodsFor: 'private' stamp: 'mfp 1/17/2008 19:47'!
applyChanges
	"Resets all roles to the selected user and attaches the selected user to all roles that will be set."

	| sm |
	
	sm := self session securityManager.
	sm 
		updateUser: self selectedUser 
		roles: self attachedRoles.

	self attachedRoles do: [:role | 
		sm updateRole: role addUser: self selectedUser].
	self availableRoles do: [:role |
		sm updateRole: role deleteUser: self selectedUser].
! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 12/29/2007 10:07'!
attachedRoles
	^ attachedRoles! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 12/29/2007 10:07'!
attachedRoles: anObject
	attachedRoles := anObject! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 12/29/2007 10:07'!
availableRoles
	^ availableRoles! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 12/29/2007 10:07'!
availableRoles: anObject
	availableRoles := anObject! !

!ITComponentAdminUsers methodsFor: 'private' stamp: 'amb 1/29/2008 16:36'!
deactivateSelectedUser
	"Deactivates the selected user. An inactive account cannot log in."
	
	self selectedUser == self session securityManager defaultUser
		ifTrue: [self inform: 'You cannot deactivate the default user.' tr. ^self].
		
	self selectedUser username = 'admin'
		ifTrue: [self inform: 'You cannot deactivate the administrator account.' tr. ^self].
		
	(self
		inform: ('Do you really want to deactivate ''{1}''?' tr format: {self selectedUser username})
		title: 'Question' tr
		buttons: {'Yes' tr. 'No' tr}
	) = 2
		ifTrue: [^self].
	
	[self session securityManager updateUser: self selectedUser isActive: false]
		on: ITUnauthorizedError
		do: [self inform: 'You are not authorized to deactivate a user!!' tr]! !

!ITComponentAdminUsers methodsFor: 'private' stamp: 'mt 1/12/2008 20:38'!
heading
	^ 'Edit security settings for user ''{1}''' tr format: {self selectedUser username}! !

!ITComponentAdminUsers methodsFor: 'initialization' stamp: 'mig 1/10/2008 16:16'!
initialize
	super initialize.
	
	selectedUser := self session securityManager defaultUser.
	self updateRoles.! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'mt 1/23/2008 23:46'!
renderActualPrivilegesOn: html
	"Shows the current privileges the selected user would have if you apply/save the changes."
	
	| priv |
	priv := Set new.
	self attachedRoles do: [:role | priv := priv union: role privileges ].
	
	html fieldSet
		legend: 'Actual Privileges' tr;
		with: [
			html unorderedList
				with: [
					priv do: [:privilege | html listItem with: privilege asString tr, ' ']
				]
		]! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'mt 1/12/2008 20:41'!
renderButtonsOn: html
	html button
		callback: [self applyChanges];
		with: [
			html image url: ITFileLibrary / #acceptPng.
			html break.
			html text: 'Apply changes' tr
		]! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'mt 1/23/2008 08:43'!
renderContentOn: html
	
	html heading
		id: 'user_heading';
		level: 2;
		with: [ html text: self heading ].
	
	html fieldSet
		legend: 'Select another user:' tr;
		with: [ self renderUserSelectionOn: html ].
	
	html div
		id: 'role_lists';
		with: [ self renderRoleListsOn: html ].
		
	html div
		id: 'actual_privileges';
		with: [ self renderActualPrivilegesOn: html ].
		
	html form
		style: 'text-align: right; margin-right: 0px;';
		with: [ self renderButtonsOn: html ]! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'mig 1/9/2008 22:29'!
renderListItems: aSelector on: html
	html unorderedList
		id: aSelector asString;
		script: (html sortable
			constraint: false;
			dropOnEmpty: true;
			containment: #( availableRoles attachedRoles );
			onUpdate: (html updater
				id: 'actual_privileges';
				triggerSortable: aSelector	callback: [ :values | self perform: aSelector asMutator with: values ];
				callback: [ :renderer | self renderActualPrivilegesOn: renderer ]
			)
		);
		with: [
			(self perform: aSelector) do: [ :each |
				html listItem
					passenger: each;
					class: (aSelector = #availableRoles ifTrue: ['available'] ifFalse: ['attached']);
					with: each rolename] ]! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'mt 1/12/2008 20:39'!
renderRoleListsOn: html
	html fieldSet
		class: 'drag_list';
		legend: 'Available Roles' tr;
		with: [ self renderListItems: #availableRoles on: html ].
	html fieldSet
		class: 'drag_list';
		legend: 'Attached Roles' tr;
		style: 'float: right;';
		with: [ self renderListItems: #attachedRoles on: html ].
	html div
		style: 'clear: both'.! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'amb 1/29/2008 16:17'!
renderUserSelectionOn: html

	| users |
	
	[users := self session securityManager getUsers select: [:user |user isActive]]
		on: ITUnauthorizedError 
		do: [html text: 'You are not authorized to list all users!!.'. ^self].
	
	html form
		id: 'user_drop_down';
		with: [
			html select
				list: users;
				selected: self selectedUser;
				labels: [:user|user username, ' (', user id asString, ')'];
				callback: [ :value | self selectedUser: value ];
				onChange: (html updater
					id: 'role_lists';
					triggerForm: 'user_drop_down';
					callback: [:r | self updateRoles; renderRoleListsOn: r]);
				onChange: (html updater
					id: 'user_heading';
					triggerForm: 'user_drop_down';
					callback: [:r | r text: self heading]);
				onChange: (html updater
					id: 'actual_privileges';
					triggerForm: 'user_drop_down';
					callback: [:r | self renderActualPrivilegesOn: r ]).

			html button
				callback: [self deactivateSelectedUser];
				with: 'Deactivate' tr.
		]! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 1/6/2008 00:43'!
selectedUser
	^ selectedUser! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 1/6/2008 00:43'!
selectedUser: anObject
	^ selectedUser := anObject! !

!ITComponentAdminUsers methodsFor: 'private' stamp: 'mfp 1/17/2008 18:35'!
updateRoles
	"Updates the available and attached roles to the current selected User."
	
	self attachedRoles: self selectedUser roles.
	[ self availableRoles:	(
		(self session securityManager getRoles) asSet collect: [:each |
			(self selectedUser roles includes: each) not ifTrue: [each]
		])
	]
		on: ITUnauthorizedError
		do: [self inform: 'You are not authorized to refresh all available roles!!'. self availableRoles: Set new]! !

ITComponent subclass: #ITComponentDetails
	instanceVariableNames: 'currentIssue addComment tags selectedTag tagContent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentDetails commentStamp: 'mt 1/22/2008 11:47' prior: 0!
I show all the details for an issue and provide an interface to change and append details and comments.!

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/26/2008 22:03'!
addComment
	^addComment
		ifNil: [addComment := ITComponentAddComment new
			currentIssue: self currentIssue;
			director: self director;
			yourself]! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/4/2008 23:36'!
addComment: anObject
	addComment := anObject! !

!ITComponentDetails methodsFor: 'private' stamp: 'mig 1/26/2008 23:01'!
addDetail
	"Appends a new detail to the current issue. Overwrites the content of an already present tag."

	self tagContent isEmpty
		ifTrue: [self inform: 'Please enter some content to the selected tag.' tr. ^self].
	
	[self session issueManager
		updateIssue: self currentIssue
		setDetail: (ITIssueDetail new
			tag: self selectedTag;
			content: self tagContent;
			yourself)]
		on: ITUnauthorizedError
		do: [self inform: 'You are not authorized to set a detail!!' tr]
		on: ITInvalidIssueError
		do: [self inform: 'You cannot modify an invalid issue. Maybe another user just deleted this one...' tr title: 'Invalid Issue'. self director signal: #'invalid_issue_detected']! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/4/2008 23:41'!
children
	^ Array with: self addComment.! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/19/2008 15:06'!
currentIssue
	^ currentIssue! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/19/2008 15:06'!
currentIssue: anIssue
	currentIssue := anIssue! !

!ITComponentDetails methodsFor: 'private' stamp: 'mig 1/26/2008 23:02'!
deleteDetail: aDetail
	"Asks the user first, then deletes the detail."
	
	(self 
		inform: ('Do you really want to delete ''{1}''?' tr format: {aDetail tag tagName})
		title: 'Delete Detail' tr
		buttons: {'Yes' tr. 'No' tr}
	) = 1	ifFalse: [^self].
		
	[self session issueManager
		updateIssue: self currentIssue
		deleteDetail: aDetail
	]
		on: ITUnauthorizedError
		do: [self inform: 'You are not authorized to set a detail!!']
		on: ITInvalidIssueError
		do: [self inform: 'You cannot modify an invalid issue. Maybe another user just deleted this one...' tr title: 'Invalid Issue'. self director signal: #'invalid_issue_detected']
	
	! !

!ITComponentDetails methodsFor: 'private' stamp: 'mt 1/20/2008 22:42'!
deleteIssue
	"Removes the current issue from the database through the issue manager."

	(self
		inform: 'Do you really want to delete the current issue?' tr
		title: ('Delete Issue #{1}' tr format: {self currentIssue id asString})
		buttons: {'Yes' tr. 'No' tr}) = 2 ifTrue: [^self].
		
	[self session issueManager deleteIssue: self currentIssue. 
	 self director signal: #'issue_deleted']
		on: ITUnauthorizedError
		do: [self inform: 'You are not authorized to delete an issue!!' tr]
	! !

!ITComponentDetails methodsFor: 'private' stamp: 'mt 1/22/2008 08:41'!
getPermaLink
	"Returns a permanent, non-changing link to this detail-issue-page."
	
	| baseurl prefix|
	prefix := ''.
	baseurl := self session application resourceBaseUrl.
	baseurl ifNotNil: [prefix := baseurl asString].
	
	^prefix, '?issue=', self currentIssue id asString! !

!ITComponentDetails methodsFor: 'initialization' stamp: 'mt 1/22/2008 11:43'!
initialize
	super initialize.
	tags := self session issueManager getTags.
	selectedTag := tags first.
	tagContent := ''.! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'amb 1/29/2008 17:14'!
renderCommentsOn: html
	"Shows all comments which are attached to the current issue."
	
	html heading
		with: 'Comments:' tr;
		level: 2.

	(self currentIssue comments) do: [:comment |
		html div
			class: 'comment';
			with: [ 
				html heading
					level: 3;
					with: [
						html text: ('{1} by ' tr format: {comment timestamp printRelativeTimeStamp}).
						comment user isActive
							ifTrue: [html text: comment user username]
							ifFalse: [html span style: 'text-decoration: line-through;'; with: comment user username].
					].
				
				html withBreakUrlList: comment content withSqueakLineEndings
			]
	]! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mt 1/22/2008 08:53'!
renderContentOn: html
	"Show details of the current issue."
	
	html div class: 'add_issue'; with: [
	self currentIssue ifNotNil: [
		self renderHeadingOn: html.
		html form class: 'blue'; with: [self renderDetailsOn: html].
		self renderDeleteIssueOn: html.
		self renderCommentsOn: html.
		html render: self addComment.
	] ifNil: [
		html break.
		html text: 'You are not authorized to view the details of an issue!!' tr.
	]]! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mt 1/22/2008 23:18'!
renderDeleteIssueOn: html
	"shows a delete button if issue was created by current user"
	
	self session user == self currentIssue firstComment user
		ifFalse: [^self "Only the creator can delete its issue until now."].
		
	html form style: 'text-align: right;'; with: [
		html button
			style: 'margin: 5px 0;';
			callback: [self deleteIssue];
			with:	'Delete Issue' tr
	]
	! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mt 1/26/2008 20:16'!
renderDetailOn: html for: aDetail class: aSymbol
	"Shows one detail in a table row. Creates an in-place-editor for each detail's content."
	
	html tableRow
		class: aSymbol asString;
		with: [
			html tableData class: 'tag'; with: aDetail tag tagName.
			
			html tableData class: 'detail_content'; with: [
				html span
				script: (html inPlaceEditor
					okControl: 'false';
					cancelControl: 'false';
					submitOnBlur: true;
					highlightColor: 'none';
					highlightEndColor: 'none';
					savingText: 'Saving...' tr;
					triggerInPlaceEditor: [ :value | self setDetail: aDetail to: value ];
					callback: [ :r | r render: aDetail content]);
				with: aDetail content	
			].
		
			html tableData with: [ aDetail tag == (self session issueManager getTag: 'title') ifFalse: [
				html anchor
					callback: [self deleteDetail: aDetail];
					with: [html image style: 'background: none;'; url: ITFileLibrary / #deletePng]]
			]
		]! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mt 1/26/2008 20:28'!
renderDetailsOn: html
	"Shows all details of the current issue. #odd is used for CSS."

	| odd |
	odd := true.
	
	html table
		id: 'issue';
		class: 'list_details';
		with: [
			html tableHead with: [ html tableRow with: [
				html tableHeading class: 'tag'; with: 'Tag' tr.
				html tableHeading class: 'detail_content'; with: 'Content' tr.
				html tableHeading.
			]].
	
			html tableBody with: [
				self currentIssue details do: [:detail | 
					self
						renderDetailOn: html
						for: detail
						class: (odd ifTrue: [odd := false. #odd] ifFalse: [odd := true. #even])
				].
				
				self updateTags.
				self tags size > 0
					ifTrue: [self renderNewDetailRowOn: html].
			]		
		]
! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mfp 1/22/2008 16:15'!
renderHeadingOn: html
	"Renders a headline with a permanent link to this issue."
	
	html anchor
		url: self getPermaLink;
		with: [ 
			html heading
			level: 2;
			with: [
				html small with: '#', self currentIssue id asString, ' '.
				html text: (self currentIssue getDetail: (
					self session issueManager getTag: 'title'
				)) content]
	]! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mt 1/26/2008 20:26'!
renderNewDetailRowOn: html
	"Shows a combobox with all available tags and some editor for the selected tag. Enables the user to create new details."
	
	html tableRow style: 'height: 20px; background-color: #fff;'; with: [
		html tableData style: 'border-left: none;'.	
		html tableData style: 'border-left: none;'.
		html tableData style: 'border-left: none;'.
	].
	
	html tableRow with: [
		html tableData with: [ html div style: 'white-space: nowrap;'; with: [
			html span with: [
				html select
					id: 'tag_selection';
					list: (self tags);
					labels: [:tag| tag tagName];
					on: #selectedTag of: self;
					onChange: (html updater
						id: 'tag_description';
						triggerFormElement: 'tag_selection';
						callback: [:r |  ])
			]. html span style: 'padding: 5px'; with: [
				html anchor
					style: 'display: table-cell;';
					callback: [self inform: self selectedTag tagDescription title: self selectedTag tagName];
					with: [html image url: ITFileLibrary / #helpPng].
			]]		
		].
		html tableData class: 'detail_content'; with: [
			html div id: 'tag_description'; style: 'display: none;'.
			html textInput
				id: 'content_input';
				style: 'width: 100%;';
				on: #tagContent of: self;
				onChange: (html updater
					id: 'tag_description';
					triggerFormElement: 'content_input';
					callback: [:r |  ])
		].
		html tableData with: [
			html anchor
				callback: [self addDetail];
				with: [html image url: ITFileLibrary / #addPng]
			
		]
	]! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/21/2008 12:08'!
selectedTag
	^ selectedTag! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/21/2008 12:08'!
selectedTag: anObject
	selectedTag := anObject! !

!ITComponentDetails methodsFor: 'private' stamp: 'mig 1/26/2008 23:02'!
setDetail: aDetail to: newContent
	"Asks the user to enter a new content for a detail and apply this new content.
	Hint: We call this function from a callback of an in-place-editor. We cannot use any rendering functions here..."

	newContent isEmpty
		ifTrue: [^self].
		
	[self session issueManager
		updateIssue: self currentIssue
		setDetail: (ITIssueDetail new
			tag: aDetail tag;
			content: newContent;
			yourself)]
		on: ITUnauthorizedError
		do: [^self. "self inform: 'You are not authorized to modify a detail!!'"]
		on: ITInvalidIssueError
		do: [^self. "self director signal: #'invalid_issue_detected'"].
! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/21/2008 12:11'!
tagContent
	^ tagContent! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/21/2008 12:11'!
tagContent: anObject
	tagContent := anObject! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/21/2008 12:08'!
tags
	^ tags! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/21/2008 12:08'!
tags: anObject
	tags := anObject! !

!ITComponentDetails methodsFor: 'private' stamp: 'mt 1/26/2008 20:25'!
updateTags
	"Updates the collection with all available tags."
	
	self tags: (
		self session issueManager getTags copyWithoutAll:
			(self currentIssue details collect: [:detail | detail tag])
	).
	
	self tags isEmpty
		ifFalse: [self selectedTag: self tags first]
		ifTrue: [self selectedTag: nil].! !

ITComponent subclass: #ITComponentDirector
	instanceVariableNames: 'colleagues events menuEntries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentDirector commentStamp: 'afk 1/11/2008 18:30' prior: 0!
I'm responsible for encapsulating the interaction of components that are visible to the user. 
They can call me with and tell me what changed and I know what to do with that.!

!ITComponentDirector class methodsFor: 'testing' stamp: 'afk 1/11/2008 14:28'!
canBeRoot
	^true! !

!ITComponentDirector methodsFor: 'private' stamp: 'mig 1/16/2008 23:01'!
changeContent: anObject
	"change the content component to anObject and set menu entry accordingly"

	self colleagues at: #content put: (anObject director: self; yourself).
	(self colleagues at: #menu) currentIndex: (self menuEntries findFirst: [:entry | entry value = anObject class]).! !

!ITComponentDirector methodsFor: 'accessing' stamp: 'afk 1/11/2008 15:07'!
children
	
	^ self colleagues values! !

!ITComponentDirector methodsFor: 'accessing' stamp: 'afk 1/11/2008 15:04'!
colleagues
	^ colleagues! !

!ITComponentDirector methodsFor: 'accessing' stamp: 'afk 1/11/2008 15:04'!
colleagues: anObject
	colleagues := anObject! !

!ITComponentDirector methodsFor: 'accessing' stamp: 'afk 1/11/2008 16:27'!
events
	^ events! !

!ITComponentDirector methodsFor: 'accessing' stamp: 'afk 1/11/2008 16:27'!
events: anObject
	events := anObject! !

!ITComponentDirector methodsFor: 'initialize' stamp: 'mfp 1/17/2008 14:36'!
initialRequest: aRequest
	| inputFields selectedIssue |
	super initialRequest: aRequest.
	inputFields := aRequest fields.
	inputFields at: 'issue' ifPresent: [:issueNumber |
		selectedIssue := self session issueManager getIssueWithID: issueNumber asNumber.
		selectedIssue ifNotNil: [self signal: #'issue_selected' with: { selectedIssue }].
		selectedIssue ifNil: [self lightbox: (ITComponentMessage new title: 'Sorry!!'; message:'The specified issue does not exist.'; buttons: {'I See'})].
		].
	! !

!ITComponentDirector methodsFor: 'initialization' stamp: 'mig 1/18/2008 00:15'!
initialize
	"initialize menu entries, start components and events"

	super initialize.
	self 
		initializeMenuEntries;
		initializeEvents;
		initializeColleagues.! !

!ITComponentDirector methodsFor: 'initialization' stamp: 'mig 1/18/2008 00:11'!
initializeColleagues
	"set the initial colleages"

	colleagues := (Dictionary new
		at: #header put: (ITComponentHeader new director: self; yourself);
		at: #menu put: (ITComponentMenu new 
			director: self; 
			items: (self menuEntries collect: [:item | item key]);  
			yourself);
		at: #content put: (ITComponentHome new director: self; yourself);
		at: #footer put: (ITComponentFooter new director: self; yourself); yourself).! !

!ITComponentDirector methodsFor: 'initialization' stamp: 'mt 1/26/2008 21:42'!
initializeEvents
	"initialize event handlers"

	events := (Dictionary new 
		at: #'selection_changed' put:
			[:newIndex | self changeContent: (self menuEntries at: newIndex) value new];
		at: #'login_clicked' put: 
			[self lightbox: ITComponentLogin new];
		at: #'logout_clicked' put:
			[self changeContent: ITComponentHome new];
		at: #'register_clicked' put: 
			[self lightbox: ITComponentUserRegister new];
		at: #'usersettings_clicked' put: 
			[self changeContent: ITComponentUserSettings new];
		at: #'issue_created' put: 
			[:newIssue | self changeContent: (ITComponentDetails new currentIssue: newIssue; yourself)];
		at: #'issue_selected' put: 
			[:newIssue | self changeContent: (ITComponentDetails new currentIssue: newIssue; yourself)];
		at: #'issue_deleted' put:
			[self changeContent: ITComponentListIssues new]; 
		at: #'language_changed' put:
			[self initializeMenuEntries; initializeColleagues];
		at: #'search_selected' put: 
			[:filter | self changeContent: (ITComponentSearch new filter: filter; yourself)];
		at: #'filter_created' put: 
			[:filter | self changeContent: (ITComponentListIssues new filter: filter; yourself)];
		at: #'filter_aborted' put:
			[self changeContent: ITComponentListIssues new];
		at: #'invalid_issue_detected' put:
			[self changeContent: ITComponentListIssues new];
	yourself)
	! !

!ITComponentDirector methodsFor: 'initialization' stamp: 'mig 1/18/2008 00:15'!
initializeMenuEntries
	"(re-)initialize the menu entries"

	menuEntries := (OrderedCollection new
		add: 'Home' tr -> (ITComponentHome);
		add: 'View Issues' tr -> (ITComponentListIssues);
		add: 'New Issue' tr -> (ITComponentAddIssue);
		add: 'Administration' tr -> (ITComponentAdmin);
		yourself).! !

!ITComponentDirector methodsFor: 'accessing' stamp: 'afk 1/11/2008 16:42'!
menuEntries
	^ menuEntries! !

!ITComponentDirector methodsFor: 'accessing' stamp: 'afk 1/11/2008 16:42'!
menuEntries: anObject
	menuEntries := anObject! !

!ITComponentDirector methodsFor: 'rendering' stamp: 'mig 1/13/2008 12:55'!
renderContentOn: html
	
	#(header menu content footer) do: [:each |
		html 
			div class: each asString; 
			with: [ html render: (self colleagues at: each)]
	]

! !

!ITComponentDirector methodsFor: 'event-handling' stamp: 'afk 1/11/2008 17:17'!
signal: aSymbol
	
	self signal: aSymbol with: Array new.! !

!ITComponentDirector methodsFor: 'event-handling' stamp: 'afk 1/11/2008 17:27'!
signal: aSymbol with: someData
	
	(self events at: aSymbol ifAbsent: [^ self]) valueWithEnoughArguments: someData.! !

!ITComponentDirector methodsFor: 'accessing' stamp: 'mig 1/16/2008 00:22'!
states
	^{colleagues}! !

ITComponent subclass: #ITComponentFooter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentFooter commentStamp: 'afk 1/14/2008 01:17' prior: 0!
I'm shown at the bottom of the webpage and providing some information about the project (projectname, homepage, developers email).!

!ITComponentFooter methodsFor: 'rendering' stamp: 'afk 1/21/2008 10:36'!
renderContentOn: html
	"Display the footer on every page after the content"
	
	html div
		class: 'footer_right';
		with: [
			html withUrls: 'Visit yaiiit!! project page on http://www.yaiiit.net' tr.
		].
			
	html div
		with: [
			html image url: ITFileLibrary / #logobottomPng.
		].	! !

ITComponent subclass: #ITComponentHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentHeader commentStamp: 'afk 1/14/2008 01:16' prior: 0!
I'm the header of the application. I'm shown on the top of the homepage and am including menus for loggin in / registration / language selection and if a user is logged in he can choose to edit his settings from here (I provide the anchor).!

!ITComponentHeader methodsFor: 'rendering' stamp: 'afk 1/24/2008 00:44'!
renderContentOn: html
	"Shows the application header with the project name and the login menu."
	
	html div
		id: 'top';
		with: [
			html div
				id: 'project_name';
				with: [html heading level: 1; with: 'yaiiit!!'].
		
			html div
				id: 'login_menu';
				with: [self renderLoginMenuOn: html].
				
			html div style: 'clear:both;'.
	]! !

!ITComponentHeader methodsFor: 'rendering' stamp: 'mt 1/23/2008 23:17'!
renderLanguagesOn: html
	"Let the user choose an installed language. Render language flag icons if present."
	
	html text: 'Choose language:' tr.
	
		html text: ' [ '.
		ITTranslator getInstance allLanguages asSortedCollection do: [:language |
			html anchor
				callback: [self session language: language. self director signal: #'language_changed'];
				with: [
					| iconSelector |
					iconSelector := (language,'Png') asSymbol.
					(ITFileLibrary new fileSelectors includes: iconSelector)
						ifTrue: [html image url: (ITFileLibrary / (language,'Png') asSymbol); style: 'display: inline;']
						ifFalse: [html text: language asString]
				]
		] separatedBy: [
			html text: ' | '.
		].
	
		html html: ' ] '.! !

!ITComponentHeader methodsFor: 'rendering' stamp: 'afk 1/21/2008 10:42'!
renderLoginMenuOn: html
	"Shows the register/login menu."
	
	html span with: [
		self renderLanguagesOn: html.
		html break.
		self session isLoggedIn ifFalse: [
			html anchor
				callback: [ self director signal: #'login_clicked' ]; 
				with: [ html text: 'Login' tr].
			html text: ' | '.
			html anchor 
				callback: [ self director signal: #'register_clicked' ]; 
				with: [ html text: 'Register' tr].
		] ifTrue: [
			html span
				class: 'text';
				with: ('logged in as {1}' tr format: {self session user username}).
			html text: ' | '.
			html anchor
				callback: [
					self session user: nil.
					self director signal: #'logout_clicked'
				];
				with: 'Logout' tr.
			html text: ' | '.
			html anchor
				callback: [self director signal: #'usersettings_clicked'];
				with: 'Settings' tr.
		]
	]
! !

ITComponent subclass: #ITComponentHome
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentHome commentStamp: 'afk 1/14/2008 01:19' prior: 0!
I'm showing the menu-point home and providing the visitor basic information about this project (a welcome page)'!

!ITComponentHome methodsFor: 'rendering' stamp: 'mt 1/23/2008 22:25'!
renderContentOn: html
	"renders the welcome-page the user sees if the enters the application the first time"
	
	html heading
		level: 2;
		with: 'Yet Another Issue Issue Issue Tracker!!' tr.
	
	html paragraph
		with: 'This is just another issue tracker. You can track all tasks, issues or bugs of your software project. yaiiit!! tries to integrate easily in your project''s workflow with a clear design and simple usage.' tr.
		
	html paragraph
		style: 'font-weight: bold;';
		with: 'At this moment, yaiiit!! tracks itself during its developing phase.' tr.
		
	html paragraph
		with: 'Feel free to register and contribute to our growing community. We are 5 developers at the moment. If you face any problems do not hesitate to mail to developer@yaiiit.net. We will enable you to post your found issues and help making yaiiit!! more reliable.' tr.
	
	html paragraph	with: 'Our current features include:' tr.
	html unorderedList with: [
		html listItem with: 'add, modify and delete issues' tr.
		html listItem with: 'append comments on those issues' tr.
		html listItem with: 'a generic tag approach to customize the issue''s details' tr.
		html listItem with: 'a security system based on privileges and roles' tr.
		html listItem with: 'an administration area to control security, languages and tags' tr.
		html listItem with: 'an user area to setup username and password' tr.
		html listItem with: 'custom filters for all issues' tr.].

	html paragraph 
		with: [html withUrls: 'So enjoy your visit on http://www.yaiiit.net.' tr]
	! !

ITComponent subclass: #ITComponentListIssues
	instanceVariableNames: 'currentIssues currentPageIndex issuesPerPage sortColumn sortDescending filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentListIssues commentStamp: 'mig 1/9/2008 12:00' prior: 0!
I list all issues in the issue tracker.!

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mt 1/6/2008 12:15'!
currentIssues
	^ currentIssues! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mt 1/6/2008 12:15'!
currentIssues: anObject
	currentIssues := anObject! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mig 1/12/2008 13:57'!
currentPageIndex
	^ currentPageIndex contents! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mig 1/12/2008 13:57'!
currentPageIndex: anObject
	currentPageIndex contents: anObject! !

!ITComponentListIssues methodsFor: 'private' stamp: 'mig 1/12/2008 15:46'!
currentPageIssueRange
	"returns a range of issues indexes (in currentIssues) on the current page"
	
	^ (self currentPageIndex - 1) * self issuesPerPage + 1
		to: (self currentIssues size min: self currentPageIndex * self issuesPerPage)! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'afk 1/18/2008 19:32'!
filter
	^ filter contents! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'afk 1/18/2008 19:31'!
filter: anObject
	filter contents: anObject! !

!ITComponentListIssues methodsFor: 'as yet unclassified' stamp: 'mig 1/12/2008 15:32'!
formatForTable: anObject
	^ (anObject isKindOf: TimeStamp) ifTrue: [
		anObject printRelativeTimeStamp
	] ifFalse: [
		anObject asString
	]! !

!ITComponentListIssues methodsFor: 'initialization' stamp: 'mt 1/23/2008 08:17'!
initialize
	super initialize.
	currentPageIndex := WAValueHolder with: 1.
	issuesPerPage := WAValueHolder with: 20.
	sortColumn := WAValueHolder with: 6.
	sortDescending := WAValueHolder with: true.
	filter := WAValueHolder with: ITFilter new.! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mig 1/12/2008 13:58'!
issuesPerPage
	^ issuesPerPage contents! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mig 1/12/2008 13:58'!
issuesPerPage: anObject
	issuesPerPage contents: anObject! !

!ITComponentListIssues methodsFor: 'paging' stamp: 'mig 1/12/2008 13:11'!
pageCount
	^ (self currentIssues size alignedTo: self issuesPerPage) / self issuesPerPage! !

!ITComponentListIssues methodsFor: 'private' stamp: 'afk 1/19/2008 15:38'!
refreshCurrentIssues
	"Retrieves all available issues through the issue manager from the database."
	
	self currentIssues: (
	
		(self filter apply: (self session issueManager getIssues)) asArray
	
		sort: [:issue1 :issue2 | | value1 value2 |
		value1 := (self tableTemplate at: self sortColumn) value value: issue1.
		value2 := (self tableTemplate at: self sortColumn) value value: issue2.
		
		(value1 isKindOf: String) ifTrue: [
			value1 := value1 asLowercase.
			value2 := value2 asLowercase.
		].
	
		value1 < value2 xor: self sortDescending
	])! !

!ITComponentListIssues methodsFor: 'rendering' stamp: 'afk 1/23/2008 21:40'!
renderContentOn: html
	"checks wheter the user is allowed to list all issues and if so ... "

	[self refreshCurrentIssues]
		on: ITUnauthorizedError
		do: [html text: 'You are not authorized to list all issues!!' tr. ^nil].

	html heading 
		level: 2;
		with: ('All Issues ({1})' tr format: {self currentIssues size asString}).
		
	html div
		with: [self renderIssueTableOn: html].
	
	self renderFilterOn: html.
	self renderNavigationOn: html.
	! !

!ITComponentListIssues methodsFor: 'rendering' stamp: 'afk 1/23/2008 21:40'!
renderFilterOn: html
	"Renders anchor for the search component"

	html div
		style: 'float:left;';
		with: [html anchor
					callback: [self director signal: #'search_selected' with: {self filter}];
					with: [
						html span
							style: 'white-space: nowrap;';
							with: [
								html image url: ITFileLibrary / #findPng. 
								html space.
								html text: 'Filter Issues' tr.
							]
					]
	]! !

!ITComponentListIssues methodsFor: 'rendering' stamp: 'mt 1/12/2008 15:55'!
renderIssueTableOn: html
	"Renders a table with necessary information of all available issues. There is a link under the name of every issue which displays a detailed view. See ITComponentDetails for more information."
	
	| class |
	class := 'even'.
	
	html table class: 'list_issues'; with: [
		html tableHead with: [
			html tableRow: [
				self tableTemplate withIndexDo: [:each :index | 
					html tableHeading with: [
						html anchor
							callback: [
								self sortColumn = index
									ifTrue: [
										self sortDescending: self sortDescending not.
									] ifFalse: [
										self sortColumn: index.
										self sortDescending: false.
									].
							];
							with: [each key value: html].
						self sortColumn = index ifTrue: [
							self sortDescending
								ifFalse: [html image url: ITFileLibrary / #arrowupPng]
								ifTrue: [html image url: ITFileLibrary / #arrowdownPng].
						]
					]
				]
			]
		].

		html tableBody with: [
			self currentIssues from: self currentPageIssueRange first 	to: self currentPageIssueRange last do: [ :issue | 		
				| signal |
				signal := [self director signal: #'issue_selected' with: {issue}].
				
				html tableRow
					class: (class = 'odd' ifTrue: [class := 'even'] ifFalse: [class := 'odd']);
					with: [
						self tableTemplate do: [:each |
							html tableData with: [
								html anchor
									callback: signal;
									with: (self formatForTable: ((each value) value: issue)).
							]
						]
					]
			]
		]
	]
! !

!ITComponentListIssues methodsFor: 'rendering' stamp: 'afk 1/23/2008 21:42'!
renderNavigationOn: html

	self pageCount > 1 ifTrue: [
		html div style: 'text-align: center; padding: 3px;'; with: [	
			self currentPageIndex > 1 ifTrue: [
				html anchor
					callback: [ self currentPageIndex: self currentPageIndex - 1 ];
					with: '<<'.
				html space
			].
		
			(1 to: self pageCount) do: [ :pageIndex |
				pageIndex = self currentPageIndex
					ifFalse: [
						html anchor
							callback: [ self currentPageIndex: pageIndex ];
							with: pageIndex asString.
					] ifTrue: [
						html text: pageIndex asString
					].
			] separatedBy: [
				html space.
			].
		
			self currentPageIndex < self pageCount ifTrue: [
				html space.
				html anchor
					callback: [ self currentPageIndex: self currentPageIndex + 1 ];
					with: '>>'
			]
		]
	].

	html div style: 'text-align: right;'; with: [
		
		html text: 'Issues per page: ' tr.
		
		{5. 10. 20. 50. 100.} do: [:each |
			each = self issuesPerPage
				ifFalse: [
					html anchor
						callback: [
							self issuesPerPage: each.
							self currentPageIndex: 1.
						];
						with: each asString.
				] ifTrue: [
					html text: each asString
				]
		] separatedBy: [
			html space.
		].
	].! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mig 1/12/2008 15:10'!
sortColumn
	^ sortColumn contents! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mig 1/12/2008 15:10'!
sortColumn: anObject
	sortColumn contents: anObject! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mig 1/12/2008 15:10'!
sortDescending
	^ sortDescending contents! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mig 1/12/2008 15:10'!
sortDescending: anObject
	sortDescending contents: anObject! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'afk 1/18/2008 19:54'!
states
	^{currentPageIndex. issuesPerPage. sortColumn. sortDescending. filter}! !

!ITComponentListIssues methodsFor: 'private' stamp: 'mfp 1/22/2008 16:16'!
tableTemplate
	"returns table headings and table content blocks"
	
	^ {
		[:r | r text: '#']
			-> [:issue | issue id]. 
		[:r | r text: 'Title' tr]
			-> [:issue | (issue getDetail: (self session issueManager getTag: 'title')) content]. 
		[:r | r text: 'Created' tr]
			-> [:issue | issue firstComment timestamp]. 
		[:r | r text: 'Owner' tr]
			-> [:issue | issue firstComment user username]. 
		[:r | r image url: ITFileLibrary / #commentsPng; style: 'display: inline;']
			-> [:issue | issue comments size]. 
		[:r | r text: 'Last Comment' tr]
			-> [:issue | issue latestComment timestamp]. 
		[:r | r text: 'From' tr]
			-> [:issue | issue latestComment user username]. 
	}
	
! !

ITComponent subclass: #ITComponentMenu
	instanceVariableNames: 'items currentIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentMenu commentStamp: 'afk 1/14/2008 01:04' prior: 0!
I'm the menu where the user can choose between different other Components.!

!ITComponentMenu methodsFor: 'accessing' stamp: 'mfp 1/11/2008 15:54'!
currentIndex
	^ currentIndex! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mfp 1/11/2008 15:54'!
currentIndex: anObject
	currentIndex := anObject! !

!ITComponentMenu methodsFor: 'initialize' stamp: 'afk 1/11/2008 17:03'!
initialize
	super initialize.
	items := OrderedCollection new.
	currentIndex := 1.! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mfp 1/11/2008 15:46'!
items
	^ items! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mfp 1/11/2008 15:46'!
items: anObject
	items := anObject! !

!ITComponentMenu methodsFor: 'rendering' stamp: 'mig 1/16/2008 22:24'!
renderContentOn: html

	html div id: 'main_menu'; with: [
		html unorderedList with: [
			self items withIndexDo: [ :item :index |
				html listItem
				class: (self currentIndex = index ifTrue: ['active']);
				with: [ 
					html anchor 
					callback: [ 
						self director 
						signal: #'selection_changed' 
						with: { index }];
					with: item
				]
			]   
		]
	]! !

ITComponent subclass: #ITComponentMessage
	instanceVariableNames: 'title message buttons'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

ITComponentMessage subclass: #ITComponentAddFilterCondition
	instanceVariableNames: 'tag operation arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentAddFilterCondition methodsFor: 'accessing' stamp: 'afk 1/19/2008 14:47'!
argument1
	^ arguments at: 1! !

!ITComponentAddFilterCondition methodsFor: 'accessing' stamp: 'afk 1/19/2008 14:51'!
argument1: anObject
	arguments at: 1 put: anObject.! !

!ITComponentAddFilterCondition methodsFor: 'accessing' stamp: 'afk 1/19/2008 14:47'!
argument2
	^ arguments at: 2! !

!ITComponentAddFilterCondition methodsFor: 'accessing' stamp: 'afk 1/19/2008 14:51'!
argument2: anObject
	arguments at: 2 put: anObject.! !

!ITComponentAddFilterCondition methodsFor: 'accessing' stamp: 'amb 1/14/2008 11:36'!
arguments
	^ arguments! !

!ITComponentAddFilterCondition methodsFor: 'accessing' stamp: 'amb 1/14/2008 11:36'!
arguments: anObject
	arguments := anObject! !

!ITComponentAddFilterCondition methodsFor: 'initialization' stamp: 'afk 1/18/2008 18:04'!
initialize
	super initialize.
	tag := 'Title'.
	operation := 'none'.! !

!ITComponentAddFilterCondition methodsFor: 'accessing' stamp: 'amb 1/14/2008 11:36'!
operation
	^ operation! !

!ITComponentAddFilterCondition methodsFor: 'accessing' stamp: 'amb 1/14/2008 11:36'!
operation: anObject
	operation := anObject! !

!ITComponentAddFilterCondition methodsFor: 'rendering' stamp: 'afk 1/20/2008 17:38'!
renderArgumentFieldsOn: html
	"renders input fields for the condition depending on what option is chosen"

	| numArgs |

	numArgs := ITFilter numArgsOfTag: self tag with: self operation.

	self arguments: (OrderedCollection ofSize: (numArgs )).
	
	html fieldSet
		legend: 'Select argument(s)' tr;
		id: 'show_arguments';
		with: [
			html table with: [
					(numArgs > 0) ifTrue: [
						html tableRow with: [html textInput on: #argument1 of: self].
						html break.
					].
					
					(numArgs > 1) ifTrue: [
						html tableRow with: [html textInput on: #argument2 of: self].
						html break.
					]
			]
	]! !

!ITComponentAddFilterCondition methodsFor: 'rendering' stamp: 'mt 1/21/2008 10:11'!
renderContentOn: html
	"renders the form where the user has the possibility to add new conditions to a filter"

	html div
		class: 'messagebox';
		with: [
			html form
				id: 'search_criteria_form';
				with: [
					html fieldSet
						legend: 'Select a tag:' tr;
						with: [ self renderTagSelectionOn: html ].
			
					html div
						id: 'operation_fieldset';
						with: [].
		
					html div
						id: 'argument_fields';
						with: [].
					
					html div class: 'buttons'; with: [
							html button callback: [self submit]; with: 'Add' tr.
							html button callback: [self answer: nil]; with: 'Cancel' tr.
					]
				]
	]! !

!ITComponentAddFilterCondition methodsFor: 'rendering' stamp: 'amb 1/23/2008 19:32'!
renderOperationSelectionOn: html
	"renders the operations that are available in case of filtering for the seleted tag"

	| ops |
	ops := OrderedCollection new.
	
	ITFilter operators keys do: [:each |
		(each first = (ITFilter detailTypes at: (self tag)))
		ifTrue: [ops add: each second]
	].
	
	html fieldSet
		legend: 'Select a operation' tr;
		id: 'operation_droppert';
		with: [
			html select
				id: 'operation_drop_down';
				beOptional;
				optionalLabel: 'Select operation';
				list: ops;
				selected: self operation;
				labels: [:op | op tr.];
				callback: [ :value | self operation: value.];
				onChange: (html updater
					id: 'argument_fields';
					triggerFormElement: 'operation_drop_down';
					callback: [:r | self arguments: nil. self renderArgumentFieldsOn: r])
	]! !

!ITComponentAddFilterCondition methodsFor: 'rendering' stamp: 'amb 1/23/2008 19:32'!
renderTagSelectionOn: html
	"renders the list in which the tags that the user can filter are listed"
	
	html select
		id:'tag_drop_down';
		list: ITFilter detailTypes keys;
		selected: self tag;
		labels: [:t | t tr ];
		callback: [ :value | self tag: value ];
		onChange: (html updater
			id: 'operation_fieldset';
			triggerFormElement: 'tag_drop_down';
			callback: [:r | self operation: nil. self renderOperationSelectionOn: r]);
		onChange: (html updater
			id: 'argument_fields';
			triggerFormElement: 'tag_drop_down';
			callback: [])! !

!ITComponentAddFilterCondition methodsFor: 'private' stamp: 'mt 1/25/2008 17:15'!
submit

	(self operation isNil or: [self arguments isNil])
		ifTrue: [self inform: 'Choose a tag and an operation first.' tr. ^ self].

	1 to: ((ITFilter numArgsOfTag: self tag with: self operation)) do: [ :index |
		(self arguments at: index)
			ifNil: [self inform: 'Fill all input fields.' tr. ^ self].
		((self arguments at: index) = '')
			ifTrue: [self inform: 'Fill all input fields.' tr. ^ self]		
	].
	
	self answer: (ITCondition new
		tag: self tag;
		operation: self operation;
		arguments: self arguments;
		yourself)! !

!ITComponentAddFilterCondition methodsFor: 'accessing' stamp: 'amb 1/14/2008 11:36'!
tag
	^ tag! !

!ITComponentAddFilterCondition methodsFor: 'accessing' stamp: 'amb 1/14/2008 11:36'!
tag: anObject
	tag := anObject! !

ITComponentMessage subclass: #ITComponentInput
	instanceVariableNames: 'currentValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentInput methodsFor: 'accessing' stamp: 'mt 1/13/2008 19:03'!
currentValue
	^ currentValue! !

!ITComponentInput methodsFor: 'accessing' stamp: 'mt 1/13/2008 19:03'!
currentValue: anObject
	currentValue := anObject! !

!ITComponentInput methodsFor: 'rendering' stamp: 'mt 1/21/2008 10:11'!
renderContentOn: html
	html div
		class: 'messagebox';
		with: [
			html form with: [
				html div class: 'title'; with: self title.
				html div class: 'message'; with: [
					html paragraph with: self message.
					html textInput on: #currentValue of: self.
				].
				
				html div class: 'buttons'; with: [
					html button	callback: [self submit]; with: 'OK' tr.
					html button	callback: [self answer: nil]; with: 'Cancel' tr.
				]
			]
		]! !

!ITComponentInput methodsFor: 'private' stamp: 'mt 1/20/2008 22:33'!
submit
	"Answers the component with the current value."
	
	self currentValue isEmpty
		ifTrue: [self inform: 'Please insert a value.' title: 'Empty Value'. ^self].
		
	self answer: self currentValue.! !

ITComponentMessage subclass: #ITComponentLogin
	instanceVariableNames: 'userInput passwordInput'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentLogin commentStamp: 'mig 1/9/2008 12:01' prior: 0!
I am the component where the user can login.!

!ITComponentLogin methodsFor: 'initialization' stamp: 'mt 1/14/2008 00:04'!
initialize
	super initialize.
	
	title := 'Login' tr.
	message := 'Please enter your username and the correct password to login.' tr.! !

!ITComponentLogin methodsFor: 'private' stamp: 'afk 1/8/2008 13:01'!
login
	"Logic for logging in a user"
	| loginSucceeded |
	
	loginSucceeded := self session securityManager loginUser: self userInput withPassword: self passwordInput. 
	loginSucceeded 
		ifTrue: [self answer: true]
		ifFalse: [self inform: 'Wrong username/password combination!!'.]! !

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
passwordInput
	^ passwordInput! !

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
passwordInput: anObject
	passwordInput := anObject! !

!ITComponentLogin methodsFor: 'rendering' stamp: 'mt 1/21/2008 10:01'!
renderContentOn: html
	"renders a messagebox where the user can login with a password and a username"

	html div class: 'messagebox'; with: [
		html form: [
			html div class: 'title'; with: self title.
			html div class: 'message'; with: [
				html paragraph with: self message.
				
				html table with: [
				html tableRow with: [
					html tableData with: [html image title: 'Username' tr; url: ITFileLibrary / #userPng].
					html tableData with: [html textInput on: #userInput of: self]].
					
				
				html tableRow with: [
					html tableData with: [html image title: 'Password' tr; url: ITFileLibrary / #keyPng].
					html tableData with: [html passwordInput on: #passwordInput of: self]].	
				]
			].
					
			html div class: 'buttons'; with: [
				html button callback: [ self login ]; with: 'Login' tr.
				html button callback: [ self answer: false ]; with: 'Cancel' tr.
			]
		]
	]! !

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
userInput
	^ userInput! !

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
userInput: anObject
	userInput := anObject! !

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
buttons
	^ buttons! !

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
buttons: anObject
	buttons := anObject! !

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
message
	^ message! !

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
message: anObject
	message := anObject! !

!ITComponentMessage methodsFor: 'rendering' stamp: 'mt 1/21/2008 09:52'!
renderContentOn: html
	html div
		class: 'messagebox';
		with: [
			html form with: [
				html div class: 'title'; with: self title.
				html div class: 'message'; with: self message.
				html div class: 'buttons'; with: [
					self buttons withIndexDo: [:button :index |
						html button 
							callback: [self answer: index.];
							with: button
					]
				]
			]
		]! !

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
title
	^ title! !

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
title: anObject
	title := anObject! !

ITComponentMessage subclass: #ITComponentUserRegister
	instanceVariableNames: 'userInput passwordInput1 passwordInput2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentUserRegister commentStamp: 'afk 1/14/2008 01:14' prior: 0!
I give visitors the ability to register for an account by typing in an username and a password. The same username can not exist twice (nor equivalents of the name with some characters uppercase or lowercase).!

!ITComponentUserRegister methodsFor: 'initialization' stamp: 'mt 1/14/2008 00:09'!
initialize
	super initialize.
	title := 'Register a new user' tr.
	message := 'Please choose a username and type your password twice to verify.' tr.! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput1
	^ passwordInput1! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput1: anObject
	passwordInput1 := anObject! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput2
	^ passwordInput2! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput2: anObject
	passwordInput2 := anObject! !

!ITComponentUserRegister methodsFor: 'rendering' stamp: 'mfp 1/22/2008 13:54'!
registerUser
	"trys to register the user, but only if passwords match and user does not already exists; logs in if successful"

	| user |
	
	self userInput isEmpty ifTrue: [
		self inform: 'You cannot choose an empty username.' tr title: 'Bad Username' tr.
		^self.
	].

	self passwordInput1 = self passwordInput2 ifFalse: [
		self inform: 'The passwords you entered do not match.' tr title: 'Bad Passwords'.
		^self.
	].

	user := ITUser new.
	user
		username: self userInput;
		setPassword: self passwordInput1.
			
	[self session securityManager addUser: user]
		on: ITUserAlreadyExists, ITUnauthorizedError 
		do: [
			self inform: ('The user ''{1}'' already exists or you are not allowed to register at the moment.' tr format: {user username}) title: 'Register Error' tr.
			^self.
		].

	self session securityManager loginUser: self userInput withPassword: self passwordInput1.

	self answer: true.! !

!ITComponentUserRegister methodsFor: 'rendering' stamp: 'mt 1/21/2008 10:01'!
renderContentOn: html
	"renders a messagebox where the user gets the ability to register"

	html div class: 'messagebox'; with: [
		html form: [
			html div class: 'title'; with: self title.
			
			html div class: 'message'; with: [
				html paragraph with: self message.
				
				html table with: [
				html tableRow with: [
					html tableData with: [html image title: 'Username' tr; url: ITFileLibrary / #userPng].
					html tableData with: [html textInput on: #userInput of: self]].
					
				html tableRow with: [
					html tableData with: [html image title: 'Password' tr; url: ITFileLibrary / #keyPng].
					html tableData with: [html passwordInput on: #passwordInput1 of: self]].	
				
				html tableRow with: [
					html tableData with: [].
					html tableData with: [html passwordInput on: #passwordInput2 of: self]].	
				]				
			].
		
			html div class: 'buttons'; with: [
				html button callback: [ self registerUser. ]; with: 'Register'.
				html button callback: [ self answer: false. ]; with: 'Cancel'.
			]
		]
	].! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
userInput
	^ userInput! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
userInput: anObject
	userInput := anObject! !

ITComponent subclass: #ITComponentSearch
	instanceVariableNames: 'filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentSearch methodsFor: 'accessing' stamp: 'amb 1/15/2008 20:07'!
filter
	^ filter! !

!ITComponentSearch methodsFor: 'accessing' stamp: 'amb 1/15/2008 20:07'!
filter: anObject
	filter := anObject! !

!ITComponentSearch methodsFor: 'initialization' stamp: 'afk 1/18/2008 19:56'!
initialize
	super initialize.
	filter := ITFilter new.! !

!ITComponentSearch methodsFor: 'rendering' stamp: 'afk 1/20/2008 17:43'!
renderAnchorForConditionOn: html

	html anchor
		id:'new_cond';
		callback: [ | newCondition |
					newCondition :=  self lightbox: ITComponentAddFilterCondition new.
					newCondition
						ifNotNil: [self filter addCondition: newCondition].
		];
		onClick: [(html updater
					id: 'current_conditions';
					triggerFormElement: 'new_cond';
					callback: [:r | self renderCurrentConditionsOn: r])];
		with: 'Add condition'.! !

!ITComponentSearch methodsFor: 'rendering' stamp: 'afk 1/23/2008 21:24'!
renderContentOn: html
	"renders current conditions for a filter and let the user create new conditions"

	self renderIntroOn: html.
		
	html form
		id:'filter_form';
		with: [		
			html div
				id: 'current_conditions';
				with: [self renderCurrentConditionsOn: html].
				
			html break; break.
			html button 
				callback: [self director signal: #'filter_created' with: {self filter}];
				value: 'Apply filter' tr.
			html button 
				callback: [self director signal: #'filter_aborted'];
				value: 'Do nothing' tr.
		]! !

!ITComponentSearch methodsFor: 'rendering' stamp: 'afk 1/19/2008 17:56'!
renderCurrentConditionsOn: html

	html table 
		id: 'current_conditions';
		style: 	'width:500px;
				 border: 1px solid #aeb3b4;';
		with: [
		
		html tableHead with: [
			html tableRow 
				style: 'background-color:#cce4ec';
				with: [
					html tableHeading with: [ html text: 'Tag'].
					html tableHeading with: [ html text: 'Operation'].
					html tableHeading with: [ html text: 'Argument(s)'].
			]
		].
		
		html tableBody with: [
			self filter conditions do: [:cond |
				html tableRow with: [ 
					html tableData with: [ html text: cond tag ].
					html tableData with: [ html text: cond operation ].
					html tableData with: [ cond arguments do: [:arg | html text: arg. html text:' '] ].
				] 
			].
			html tableRow with: [
				html tableData with: [
					html break.
					self renderAnchorForConditionOn: html.
				]
			]
		]
		
	]! !

!ITComponentSearch methodsFor: 'rendering' stamp: 'afk 1/20/2008 17:42'!
renderIntroOn: html
	"renders the explanation for using filters and adding conditions"

	html heading
		with: [html text: 'Filter the issues' tr].
		
	html paragraph
		with: [html text: 'Here you are able to create and edit conditions to filter the issues you see.' tr].
		
	html paragraph
		with: [html text: 'You have several options for different tags. For example:' tr].
	
	html unorderedList with: [
		html listItem with: 'By selecting "matches" you can use two wildcards (# = one char and * = arbitrary number of chars)' tr.
		html listItem with: 'Or selecting one of those self explaining options (equals, until, between, since)' tr.].! !

ITComponent subclass: #ITComponentUserSettings
	instanceVariableNames: 'tabsRoot selectedTabKey userTabs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentUserSettings commentStamp: 'afk 1/10/2008 08:37' prior: 0!
I'm showing the users current personal settings. I also enable him to change most of them like
username, password or e-mail. I'm only shown if there is a user that is logged in.!

!ITComponentUserSettings methodsFor: 'accessing' stamp: 'afk 1/13/2008 10:59'!
children
	^Array with: self currentChild.! !

!ITComponentUserSettings methodsFor: 'private' stamp: 'afk 1/13/2008 10:59'!
currentChild
	"Returns the component which is the current child - otherwise return nil."
	
	self tabsRoot valuesDo: [:subsections |
		(subsections includesKey: self selectedTabKey)
			ifTrue: [^(subsections at: self selectedTabKey)]
		].
	^nil! !

!ITComponentUserSettings methodsFor: 'initialization' stamp: 'afk 1/13/2008 10:59'!
initialize
	"Creates all tabs once but does not select one to show the introduction for this component to the user. See #renderIntroductionOn: for more details."
	
	super initialize.
	self refreshTabs.
	self selectedTabKey: ''.! !

!ITComponentUserSettings methodsFor: 'private' stamp: 'mt 1/20/2008 20:12'!
refreshTabs
	"Creates new component objects for the children but keeps the current selected tab."
	
	self userTabs: (Dictionary new
		at: 'Change Password' tr put: ITComponentUserSettingsPassword new;
		at: 'Change Username' tr put: ITComponentUserSettingsUsername new;
		yourself).

	self tabsRoot: (Dictionary new
		at: 'User Settings' tr put: self userTabs;
		yourself).
	
				! !

!ITComponentUserSettings methodsFor: 'rendering' stamp: 'afk 1/16/2008 00:45'!
renderChildrenOn: html
	"renders the contents of the current active section of the user-settings-area"

	html div
		class: 'content_with_menu';
		with: [
			self selectedTabKey = ''
				ifTrue:  [ self renderIntroductionOn: html ]
				ifFalse: [ html render: self currentChild ]
		]! !

!ITComponentUserSettings methodsFor: 'rendering' stamp: 'mig 1/16/2008 18:16'!
renderContentOn: html
	"Renders the side menu of settings and the corrospondending components for changing username and password"
	
	self session isLoggedIn ifFalse: [
		html text: 'You are not authorized change user settings!!' tr. 
		^nil
	].

	html div
		with: [
			self renderSideMenuOn: html.
			self renderChildrenOn: html.
		]! !

!ITComponentUserSettings methodsFor: 'rendering' stamp: 'mt 1/23/2008 23:39'!
renderIntroductionOn: html
	"renders a brief introduction on what can be done in user-settings"
	
	html heading
		style: 'text-align: center;';
		level: 2;
		with: 'Welcome to the User Settings!!' tr.
	
	html paragraph
		with: 'In here you are able to change your account''s details without any need to get permissions from your webmaster or administrator. This includes the following:' tr.
	
	html unorderedList with: [
		html listItem with: 'set a new username to an unused one' tr.
		html listItem with: 'change your password' tr.].
	
	html paragraph
		with: 'Your User-ID cannot be changed. You''ll keep on being the same user as before with all created issues or posted comments.' tr.
		
	html paragraph
		with: 'Do not forget to keep any changes in mind because you''ll not be able to login with the deprecated credentials anymore!!' tr! !

!ITComponentUserSettings methodsFor: 'rendering' stamp: 'afk 1/16/2008 00:46'!
renderSideMenuOn: html
	"renders the side-menu of the user-settings-area and highlights the current selected section"

	html div	
		id: 'tab_menu'; 
		with: [
		html unorderedList with: [
			self tabsRoot keysAndValuesDo: [:section :subsections | 
				html listItem with: [ html text: section. html unorderedList with:[
					subsections keysDo: [:subsection |
						html listItem
							class: (self selectedTabKey = subsection ifTrue: ['active']);
							with: [html anchor callback: [self selectedTabKey: subsection; refreshTabs]; with: subsection]
					]
				]]
			]
		]
	]! !

!ITComponentUserSettings methodsFor: 'accessing' stamp: 'afk 1/13/2008 11:01'!
selectedTabKey
	^ selectedTabKey! !

!ITComponentUserSettings methodsFor: 'accessing' stamp: 'afk 1/13/2008 11:01'!
selectedTabKey: anObject
	selectedTabKey := anObject! !

!ITComponentUserSettings methodsFor: 'accessing' stamp: 'afk 1/13/2008 11:01'!
tabsRoot
	^ tabsRoot! !

!ITComponentUserSettings methodsFor: 'accessing' stamp: 'afk 1/13/2008 11:01'!
tabsRoot: anObject
	tabsRoot := anObject! !

!ITComponentUserSettings methodsFor: 'accessing' stamp: 'afk 1/13/2008 11:06'!
userTabs
	^ userTabs! !

!ITComponentUserSettings methodsFor: 'accessing' stamp: 'afk 1/13/2008 11:06'!
userTabs: anObject
	userTabs := anObject! !

ITComponent subclass: #ITComponentUserSettingsPassword
	instanceVariableNames: 'oldPassword newPassword1 newPassword2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentUserSettingsPassword commentStamp: 'afk 1/14/2008 01:12' prior: 0!
I'm responsible for providing the user an interface to change his password. Therefore he needs to be logged in (to see the settings menu) and need to enter his old password once and the new one twice.!

!ITComponentUserSettingsPassword methodsFor: 'accessing' stamp: 'afk 1/13/2008 23:56'!
clear
	"clears the input-fields"
	
	self oldPassword: nil.
	self newPassword1: nil.
	self newPassword2: nil.! !

!ITComponentUserSettingsPassword methodsFor: 'accessing' stamp: 'afk 1/13/2008 23:34'!
newPassword1
	^ newPassword1! !

!ITComponentUserSettingsPassword methodsFor: 'accessing' stamp: 'afk 1/13/2008 23:34'!
newPassword1: anObject
	newPassword1 := anObject! !

!ITComponentUserSettingsPassword methodsFor: 'accessing' stamp: 'afk 1/13/2008 23:34'!
newPassword2
	^ newPassword2! !

!ITComponentUserSettingsPassword methodsFor: 'accessing' stamp: 'afk 1/13/2008 23:34'!
newPassword2: anObject
	newPassword2 := anObject! !

!ITComponentUserSettingsPassword methodsFor: 'accessing' stamp: 'afk 1/13/2008 23:35'!
oldPassword
	^ oldPassword! !

!ITComponentUserSettingsPassword methodsFor: 'accessing' stamp: 'afk 1/13/2008 23:35'!
oldPassword: anObject
	oldPassword := anObject! !

!ITComponentUserSettingsPassword methodsFor: 'rendering' stamp: 'mt 1/23/2008 08:14'!
renderContentOn: html
	html heading
		level: 2;
		style: 'text-align: center;';
		with: [html text: 'Change your Password' tr].
	
	html div with: [
		html paragraph
			with: 'Please type your old password once and your new password twice to verify. Keep in mind that you cannot login with the old credentials anymore.' tr.
	
	html div
		style: 'border: 1px solid #dfe1e1; padding-bottom: .5em;';
		with:  [
			self renderPasswordFormOn: html	
		]	
	]			
			
	! !

!ITComponentUserSettingsPassword methodsFor: 'rendering' stamp: 'mt 1/23/2008 08:30'!
renderPasswordFormOn: html
	"Allows the user to change his password."
	
	html form
		class: 'blue';
		with: [
			html table style: 'margin: 0px auto'; with: [
			html tableRow style: 'height: 40px;'; with: [
				html tableData with: [html image title: 'Current Password' tr; url: ITFileLibrary / #keyPng].
				html tableData style: 'text-align: right;'; with: 'Current Password' tr.				
				html tableData with: [html passwordInput on: #oldPassword of: self]].					
				
			html tableRow with: [
				html tableData.
				html tableData style: 'text-align: right;'; with: 'New Password' tr.
				html tableData with: [html passwordInput on: #newPassword1 of: self]].	
				
			html tableRow with: [
				
				html tableData.
				html tableData style: 'text-align: right;'; with: 'Verify Password' tr.
				html tableData with: [html passwordInput on: #newPassword2 of: self]].		
				
			html tableRow with: [
				html tableData.
				html tableData.
				html tableData
					style: 'text-align: right;';
					with: [
						html button
							style: 'margin-right: -2px;';
							callback: [	self submit ];
							with: 'Apply' tr.
					]
			]]
	]	
	! !

!ITComponentUserSettingsPassword methodsFor: 'private' stamp: 'mt 1/20/2008 21:09'!
submit
	"Tests if old password was valid and new passwords are matching and informs the user about the process."

	(self session securityManager
		validatePassword: self oldPassword
		for: self session user username)
		ifFalse: [ self inform: 'Your current password was not correct. Please try again.' tr. ^self ].
		
	(self newPassword1 = self newPassword2)
		ifFalse: [ self inform: 'The new passwords do not match. Please try again.' tr. ^self ].
		
	self newPassword1 isEmpty
		ifTrue: [ self inform: 'Your password is not long enough.' tr. ^self].
		
	self session securityManager
		updateUser: self session user
		password: self newPassword1.
		
	self
		inform: 'The password was changed successfully. Don''t forget it.' tr;
		clear
	! !

ITComponent subclass: #ITComponentUserSettingsUsername
	instanceVariableNames: 'username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker-Components'!

!ITComponentUserSettingsUsername commentStamp: 'afk 1/14/2008 01:12' prior: 0!
I'm responsible for providing the user an interface to change his username. Therefore he needs to be logged in (to see the settings menu).!

!ITComponentUserSettingsUsername methodsFor: 'accessing' stamp: 'afk 1/14/2008 13:03'!
clear
	"clears the input-fields"
	
	self password: nil.
	self username: nil.! !

!ITComponentUserSettingsUsername methodsFor: 'accessing' stamp: 'afk 1/14/2008 12:09'!
password
	^ password! !

!ITComponentUserSettingsUsername methodsFor: 'accessing' stamp: 'afk 1/14/2008 12:09'!
password: anObject
	password := anObject! !

!ITComponentUserSettingsUsername methodsFor: 'rendering' stamp: 'mt 1/23/2008 23:37'!
renderContentOn: html
	html heading
		level: 2;
		style: 'text-align: center;';
		with: [html text: 'Change your Username' tr].
		
	html div with: [
		html paragraph
			with: 'Please enter your password and choose a new username. You cannot choose:' tr.
		html unorderedList with: [
			html listItem with: 'admin'.	
			html listItem with: 'guest'.
		].
		html paragraph
			with: 'And Remember that usernames are compared case-insensitive. So ''USER'' and ''uSeR'' will be treated as the same.' tr.
		
		html div
			style: 'border: 1px solid #dfe1e1; padding-bottom: .5em;';
			with:  [	self renderUsernameFormOn: html ]
	]! !

!ITComponentUserSettingsUsername methodsFor: 'rendering' stamp: 'mt 1/23/2008 08:32'!
renderUsernameFormOn: html
	"Allows the user to change his username."
	
	html form
		class: 'blue';
		with: [
			html table style: 'margin: 0px auto'; with: [
			html tableRow style: 'height: 40px;'; with: [
				html tableData with: [html image title: 'Password' tr; url: ITFileLibrary / #keyPng].
				html tableData style: 'text-align: right;'; with: 'Password' tr.
				html tableData with: [html passwordInput on: #password of: self]].					
				
			html tableRow with: [
				html tableData with: [html image title: 'New Username' tr; url: ITFileLibrary / #usereditPng].
				html tableData style: 'text-align: right;'; with: 'Username' tr.
				html tableData with: [html textInput on: #username of: self]].	
								
			html tableRow with: [
				html tableData.
				html tableData.
				html tableData
					style: 'text-align: right;';
					with: [
						html button
							style: 'margin-right: -2px;';
							callback: [	self submit ];
							with: 'Apply' tr.
					]
			]]
	]! !

!ITComponentUserSettingsUsername methodsFor: 'private' stamp: 'mt 1/20/2008 21:08'!
submit

	((self username isEmpty) | (self username = 'admin') | (self username = 'guest'))
		ifTrue: [self inform: 'You cannot choose an empty username or ''admin'' or ''guest ''' tr. ^self].
	
	(self session securityManager
		validatePassword: self password
		for: self session user username)
		ifFalse: [ self inform: 'Your current password was not correct. Please try again.' tr. ^self ].
	
	[self session securityManager
		updateUser: self session user
		username: self username]
			on: ITUserAlreadyExists 
			do: [self inform: 'That username already exists. Please choose another one.' tr]! !

!ITComponentUserSettingsUsername methodsFor: 'accessing' stamp: 'afk 1/14/2008 12:09'!
username
	^ username! !

!ITComponentUserSettingsUsername methodsFor: 'accessing' stamp: 'afk 1/14/2008 12:09'!
username: anObject
	username := anObject! !

!Date methodsFor: '*SWE-IssueTracker' stamp: 'mt 1/8/2008 17:02'!
printRelativeDate
	"Returns 'Today' or 'Yesterday' or 'MM-DD-YYYY' as String for the date."
	
	self = Date today ifTrue: [^'Today' tr].
	self = Date yesterday ifTrue: [^'Yesterday' tr].
	
	^(self printFormat: #(1 2 3 $- 1 1 2))! !

!WAHtmlCanvas methodsFor: '*SWE-IssueTracker' stamp: 'mt 1/24/2008 23:23'!
withBreakUrlList: aString
	| stream currentList |
	aString ifNil: [^ self].
	stream := aString readStream.
	currentList := OrderedCollection new.
	[stream atEnd] whileFalse:
		[|nextLine|
		nextLine := stream nextLine withBlanksTrimmed.
		(nextLine beginsWith: '*')
			ifTrue: [currentList add: nextLine allButFirst withBlanksTrimmed]
			ifFalse: [
				currentList isEmpty ifFalse: [
				self unorderedList with: [
					currentList do: [:item | self listItem with: [self withUrls: item]].
					currentList := OrderedCollection new.
				]].
				self withUrls: nextLine.
				stream atEnd ifFalse: [self break]
			].
		].
	currentList isEmpty
		ifFalse: [self unorderedList with: [
			currentList do: [:item | self listItem with: [self withUrls: item]]
		]].

	^aString! !

Error subclass: #ITError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITError commentStamp: 'mt 12/30/2007 14:39' prior: 0!
I am an generic error which should only be signaled in this application.!

ITError subclass: #ITInvalidIssueError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITInvalidIssueError commentStamp: 'mt 1/26/2008 21:14' prior: 0!
Should be raised if you try to update the data of an invalid issue object through the issue manager, e.g. if that object was deleted by someone else.!

ITError subclass: #ITUnauthorizedError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITUnauthorizedError commentStamp: 'mt 1/8/2008 12:14' prior: 0!
An ITUnauthorizedError is thrown, when a user has not sufficient privileges to perform an action.

I should be signaled by the security manager.!

ITError subclass: #ITUserAlreadyExists
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!
